Package: texanshootR
Type: Package
Title: Reproducible Audit Trails for Indefensible Research
Version: 0.1.1
Authors@R: person("Gilles", "Colling", email = "gilles.colling051@gmail.com",
                  role = c("aut", "cre"),
                  comment = c(ORCID = "0000-0003-3070-6066"))
Description: Provides a structured, terminal-first interface for
    exploratory model search, including transformation grids,
    predictor-subset enumeration, interaction screening, principled-
    sounding sample restrictions, outcome engineering, and model-form
    escalation (polynomial / spline wraps, robust M-estimation,
    generalized linear model (GLM) family swaps, random-intercept
    lifts). Persistent run history,
    achievement tracking, and reportable output generators (manuscript,
    presentation, funding letter, graphical abstract, reviewer
    response) are included.
License: MIT + file LICENSE
URL: https://gillescolling.com/texanshootR/,
        https://github.com/gcol33/texanshootR
BugReports: https://github.com/gcol33/texanshootR/issues
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: cli, yaml, jsonlite, stats, utils, tools, splines, Rcpp
LinkingTo: Rcpp
Suggests: ggplot2, officer, rmarkdown, xml2, quarto, tinytex, withr,
        testthat (>= 3.0.0), knitr, pkgdown
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-06-03 12:48:55 UTC; Gilles Colling
Author: Gilles Colling [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3070-6066>)
Maintainer: Gilles Colling <gilles.colling051@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-03 23:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-04 00:32:17 UTC; unix
Archs: texanshootR.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(format,tx_career)
S3method(format,tx_run)
S3method(print,tx_career)
S3method(print,tx_progress)
S3method(print,tx_run)
S3method(summary,tx_career)
S3method(summary,tx_run)
export(abstract)
export(achievements)
export(career)
export(funding)
export(graphical_abstract)
export(manuscript)
export(presentation)
export(progress)
export(reset_achievements)
export(reset_all)
export(reset_career)
export(reset_wardrobe)
export(reviewer_response)
export(run_log)
export(shoot)
export(validate_messages)
export(vocab_careers)
export(vocab_effects)
export(vocab_mascot_states)
export(vocab_phases)
export(vocab_tags)
export(wardrobe)
importFrom(Rcpp,evalCpp)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,model.frame)
importFrom(stats,na.omit)
importFrom(stats,runif)
importFrom(stats,sd)
useDynLib(texanshootR, .registration = TRUE)

# texanshootR

*running tests until one comes back significant*

<!-- badges: start -->
[![R-CMD-check](https://github.com/gcol33/texanshootR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/gcol33/texanshootR/actions/workflows/R-CMD-check.yaml)
[![Lifecycle: experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
<!-- badges: end -->

<p align="center">
  <img src="man/figures/mascot.svg" alt="texanshootR mascot — a sad cowboy in a terminal cycling through emotional states as the p-value drops" width="480">
</p>

**A budgeted, seeded specification search that fires at the model space until `p <= 0.05` lands somewhere, then writes the paper.**

The [Texas sharpshooter](https://en.wikipedia.org/wiki/Texas_sharpshooter_fallacy)
fires at the side of a barn all night, walks over in the morning, and paints
a target around the densest cluster of holes. `texanshootR` is that loop, made
reproducible. Every `shoot()` call opens a wall-clock budget and flails across
predictor subsets, transformations, interactions, outlier exclusions, subgroup
splits, and — once the budget runs low — model families and derived metrics,
biasing toward whatever is *almost* significant and abandoning specs that go
cold. Either a hit lands on the wall or it does not. When one does, the package
writes the manuscript around it.

```r
library(texanshootR)

run <- shoot(mtcars)
print(run)      # the shooter's face, the emotional arc, then the formula and p-value
summary(run)    # counts per search axis
```

`shoot()` is deterministic given a seed. The seed, R version, package version,
and a hash of the full search trace are recorded on the returned `tx_run`, so
any saved run replays exactly. Unpublishable data is now the shooter's problem,
not yours.

## The search engine

Seven model families, each with a native fitter and no soft dependency on
`mgcv`, `lme4`, or `lavaan` — the heavy ones are written by hand in C++ via
`Rcpp`:

- `lm` — ordinary least squares via `.lm.fit()`
- `cor` — bivariate Pearson / Spearman / Kendall
- `glm` — Gaussian / binomial / Poisson / Gamma with link choices
- `wls` — two-stage feasible weighted least squares
- `gam` — penalised B-spline regression with GCV smoothing selection
- `glmm` — Gaussian random-intercept LMM by profile likelihood
- `sem` — single-mediator path model with a Sobel test for the indirect effect

A family selector attaches a fitter to each spec from the run's escalation
state and the outcome shape: `lm` early, `glm` under pressure, and the
occasional forced coercion (binomial on a continuous outcome, Poisson on a
rounded one) when the shooter is chasing the next decimal. Every fitter returns
the same result shape, so the selector, the highlight chooser, and the run
record never branch on family.

## The publication chain

A `tx_run` that clears `p <= 0.05` is shippable, and the first shippable run
opens a six-stage publication chain, redeemed in order:

```r
abstract(run)            # one-paragraph deadpan summary
manuscript(run)          # IMRaD draft; Methods match the winning spec
presentation(run)        # 8-slide deck; residual plot on slide 7
reviewer_response(run)   # opens "we thank the reviewer for their thoughtful comments"
graphical_abstract(run)  # the figure your PI will retweet
funding(run)             # the next grant, citing the just-shipped finding
```

Finishing the chain through your unlocked prefix collects a length-bonus on
top of the per-stage XP. Firing a fresh `shoot()` mid-chain forfeits the bonus
and keeps partial XP. Locked stages, expired windows, the wrong run, and
out-of-order calls all signal a structured `tx_chain_error` with a `reason`
field your tests can branch on. Each generator writes to `tempdir()` and
returns the file path invisibly; override with `output_dir =` or
`options(texanshootR.output_dir = ...)`.

## Career and unlocks

Redeemed stages award XP. Cumulative XP grows the chain prefix you can redeem,
and your career tier is a label derived from that prefix. The tier is not
decoration: it gates which model families enter the search pool.

| Chain length | New stage              | XP needed | Career tier        | Families added |
|--------------|------------------------|-----------|--------------------|----------------|
| 1            | `abstract()`           | 0         | Junior Researcher  | `lm`           |
| 2            | `manuscript()`         | 5         | Postdoc            | `cor`, `glm`   |
| 3            | `presentation()`       | 15        | Postdoc            | —              |
| 4            | `reviewer_response()`  | 30        | Senior Scientist   | `wls`, `gam`   |
| 5            | `graphical_abstract()` | 55        | Senior Scientist   | —              |
| 6            | `funding()`            | 90        | PI                 | `glmm`, `sem`  |

```r
career()        # tier, runs, favourite method, opaque scores
achievements()  # 20 unlockable badges; hidden ones show as ???
wardrobe()      # cosmetic slots (hat, badge, cloak, poncho, lanyard)
progress()      # HUD: chain length, XP, next unlock, live chain window
run_log()       # tibble of every run on this profile
```

## The message pack

The terminal interface is driven by a YAML message registry under
`inst/messages/` with **1,257 entries** across phases (`blip`, `loading`,
`promotion`, `reviewer`, `derived_escalation`, `state_transition`, `banner`,
`event` / `event_consequence`). Each message carries a fallacy tag from
`vocab_tags` (`p_hacking`, `harking`, `subgroup_fishing`, `causal_overreach`,
...), a rarity weight, an optional `mascot_state_affinity`, and an optional
`model_family_affinity` so the GAM-specific line only fires when the run picked
a GAM. Every family has dedicated coverage.

```r
validate_messages()    # schema + tag-vocabulary check
vocab_tags             # canonical fallacy and thematic tags
vocab_phases           # canonical trigger phases
vocab_mascot_states    # composed / uncertain / anxious / desperate / resolved
vocab_careers          # tier ladder
```

The schema lives in [`MESSAGE_SCHEMA.md`](https://github.com/gcol33/texanshootR/blob/main/MESSAGE_SCHEMA.md).
Adding a message is a YAML edit and a re-run of `validate_messages()`.

## Persistent state

Your researcher profile persists under
`tools::R_user_dir("texanshootR", "data")` as flat YAML: human-readable,
version-controllable, and yours to carry between institutions. The first
interactive save prompts before writing anything to disk. Opt out with
`options(texanshootR.save_enabled = FALSE)` and the package runs stateless —
every call independent, progression inert at Junior Researcher, with no path
forward from there.

## Reset

A new institution, a co-author dispute, an opportune hard-drive failure, or
the simple urge to start clean:

```r
reset_career(force = TRUE)
reset_achievements(force = TRUE)
reset_wardrobe(force = TRUE)
reset_all(force = TRUE)
```

## Installation

```r
install.packages("pak")
pak::pak("gcol33/texanshootR")
```

The package compiles a small C++ backend (penalised least squares for `gam`,
profile-likelihood mixed model for `glmm`) on first install.

## Documentation

- [Getting Started](https://gillescolling.com/texanshootR/articles/getting-started.html)
- [Reference](https://gillescolling.com/texanshootR/reference/)
- [Message Pack Schema](https://github.com/gcol33/texanshootR/blob/main/MESSAGE_SCHEMA.md)
- [Contributing](https://github.com/gcol33/texanshootR/blob/main/CONTRIBUTING.md)

## Further reading

Brodeur, A., Cook, N. and Heyes, A. (2020). *Methods Matter: P-Hacking and
Publication Bias in Causal Analysis in Economics.* **American Economic Review**
110(11): 3634–60. <https://doi.org/10.1257/aer.20190687>

`texanshootR` is our humble contribution to a thriving field.

## Support

> "Where is the money, Lebowski?"
>
> &mdash; The Big Lebowski

I'm a PhD student who builds R packages in my free time, on the principle
that the tools of dubious research should be free and open. I started these
for my own questionable p-values, and figured the field deserved the same
head start.

If this package saved you some time (or pre-empted a fit of overfitting),
buying me a coffee is a nice way to say thanks.

[![Buy Me A Coffee](https://img.shields.io/badge/-Buy%20me%20a%20coffee-FFDD00?logo=buymeacoffee&logoColor=black)](https://buymeacoffee.com/gcol33)

## License

MIT &copy; Gilles Colling. See [LICENSE](LICENSE).

## Citation

```bibtex
@software{texanshootR,
  author = {Colling, Gilles},
  title  = {texanshootR: Reproducible Audit Trails for Indefensible Research},
  year   = {2026},
  url    = {https://github.com/gcol33/texanshootR}
}
```

#' Generate a funding letter + budget DOCX
#'
#' One-page Letter of Intent with a costed budget table.
#'
#' Chain stage: \strong{funding} (length 6). See [progress()].
#'
#' @inheritParams manuscript
#' @return A length-one character vector giving the path to the written
#'   DOCX file, returned invisibly. Called for its side effect of
#'   generating the funding letter in `output_dir` and advancing the
#'   publication chain.
#' @export
funding <- function(run, output_dir = NULL, file = NULL, force = FALSE) {
  require_chain_stage("funding", run)
  require_pkg("officer", "funding")
  d <- resolve_output_dir(output_dir)
  stem <- file %||% "funding_proposal_clean"
  out  <- versioned_filename(d, stem, "docx", force)

  hs <- run$highlighted_spec %||% list()

  doc <- officer::read_docx()
  doc <- officer::body_add_par(doc, "Letter of Intent", style = "heading 1")
  doc <- officer::body_add_par(doc, paste0(
    "We propose a research programme building directly on the analysis ",
    "summarised below. ", describe_spec(hs)))
  doc <- officer::body_add_par(doc, paste0(
    "The preliminary analysis recovered R-squared ",
    sprintf("%.3f", hs$r_squared %||% 0),
    " (p = ",
    sprintf("%.4f", hs$p_value %||% 1),
    "). The proposed work will scale this finding across additional ",
    "ecological contexts with broad policy relevance."))

  doc <- officer::body_add_par(doc, "")
  doc <- officer::body_add_par(doc, "Budget", style = "heading 2")

  budget <- data.frame(
    Item       = c("Personnel: 1 unspecified postdoc",
                   "Personnel: 0.2 FTE PI",
                   "Compute and storage",
                   "Open-access publication fees",
                   "Conference travel",
                   "Indirect costs"),
    EUR        = c(72000, 18000, 4000, 6000, 5000, 32500),
    stringsAsFactors = FALSE
  )

  doc <- officer::body_add_table(doc, value = budget,
                                 style = "table_template")

  doc <- officer::body_add_par(doc, "")
  doc <- officer::body_add_par(doc, "Total: 137 500 EUR.")

  status_open(out)
  print(doc, target = out)
  status_close()
  record_output(run, "funding", out)
  advance_chain_after_stage("funding")
  invisible(out)
}

# First-write consent prompt. CRAN-policy compliant: no writes to user
# directories without an interactive prompt or an explicit opt-in via
# `options(texanshootR.save_enabled = TRUE)` (which is the default but
# implies the user accepted the package by installing it; we still
# prompt the first time).

consent_ok <- function(d) {
  if (!isTRUE(opt("texanshootR.save_enabled"))) return(FALSE)

  # If a meta.json already exists at d, consent is implied.
  if (file.exists(file.path(d, "meta.json"))) return(TRUE)

  # Non-interactive sessions require an explicit opt-in *with*
  # `texanshootR.consent = TRUE`; otherwise we don't write.
  if (!interactive()) {
    return(isTRUE(getOption("texanshootR.consent", FALSE)))
  }

  # Honor an explicit consent option even in interactive sessions.
  if (isTRUE(getOption("texanshootR.consent", FALSE))) return(TRUE)

  prompt <- paste0(
    "texanshootR would like to persist career data to:\n  ", d,
    "\nAllow? [y/N]: "
  )
  ans <- tolower(trimws(readline(prompt)))
  isTRUE(ans %in% c("y", "yes"))
}

# Life-event encounters: random EVENT: blocks fired with probability
# 1/6 (configurable) per run. Each event applies temporary modifiers
# to the live run state.

load_event_registry <- function(force = FALSE) {
  load_registry(
    inst_dir   = "events",
    cache_key  = "events",
    entry_to_row = function(e) list(
      id               = as.character(e$id),
      event_text       = as.character(e$event_text),
      consequence_text = as.character(e$consequence_text),
      rarity           = as.character(e$rarity),
      career_min       = if (is.null(e$career_min) || is.na(e$career_min))
                           NA_character_ else as.character(e$career_min),
      display_phase    = as.character(e$display_phase %||% "any"),
      effects          = e$effects %||% list()
    ),
    rows_to_frame = function(rows) {
      out <- data.frame(
        id               = vapply(rows, `[[`, "", "id"),
        event_text       = vapply(rows, `[[`, "", "event_text"),
        consequence_text = vapply(rows, `[[`, "", "consequence_text"),
        rarity           = vapply(rows, `[[`, "", "rarity"),
        career_min       = vapply(rows, function(r) r$career_min %||% NA_character_, ""),
        display_phase    = vapply(rows, `[[`, "", "display_phase"),
        stringsAsFactors = FALSE
      )
      out$effects <- I(lapply(rows, `[[`, "effects"))
      out
    },
    empty = empty_event_registry,
    force = force
  )
}

empty_event_registry <- function() {
  new_list_col_frame(
    char_cols = c("id", "event_text", "consequence_text",
                  "rarity", "career_min", "display_phase"),
    list_col  = "effects"
  )
}

# Roll once per run. Returns NULL when nothing fires; otherwise an
# event record with applied effects.
maybe_event <- function(career_level, run_phase) {
  if (!isTRUE(opt("texanshootR.life_events"))) return(NULL)
  rate <- as.numeric(opt("texanshootR.event_rate"))
  if (!is.finite(rate) || rate <= 0) return(NULL)
  if (runif(1) > rate) return(NULL)

  reg <- load_event_registry()
  if (nrow(reg) == 0L) return(NULL)

  ok <- (reg$display_phase == "any") | (reg$display_phase == run_phase)
  ok <- ok & vapply(reg$career_min, function(c) {
    is.na(c) || career_compare(career_level, c) >= 0
  }, logical(1))
  if (!any(ok)) return(NULL)
  cand <- reg[ok, , drop = FALSE]
  w <- rarity_weight(cand$rarity)
  if (sum(w) == 0) return(NULL)
  idx <- sample.int(nrow(cand), size = 1L, prob = w)

  list(
    id               = cand$id[idx],
    event_text       = cand$event_text[idx],
    consequence_text = cand$consequence_text[idx],
    effects          = cand$effects[[idx]] %||% list()
  )
}

# Apply a single event's effects to a run-state list. `state` is the
# mutable accumulator threaded through shoot()'s loop.
apply_event_effects <- function(state, effects) {
  if (length(effects) == 0L) return(state)
  for (k in names(effects)) {
    if (!(k %in% vocab_effects)) next
    v <- effects[[k]]
    if (!is.numeric(v)) next
    v <- max(-0.5, min(0.5, v))
    state$modifiers[[k]] <- (state$modifiers[[k]] %||% 0) + v
  }
  state
}

# Model-family registry and family-specific fitters.
#
# fit_spec() in shoot_engine.R is a thin dispatcher that reads
# spec$family$fitter and calls the registered fitter from FAMILIES.
# Each fitter takes (df, outcome, spec) and returns the same result
# shape so downstream code (choose_highlight, run records, the UI)
# does not need to know which family produced the row.
#
# Result contract (all fitters):
#   list(formula, n, n_terms, r_squared, aic, p_value,
#        dropped, n_dropped, rows_excluded,
#        restriction, outcome_construction,
#        family, family_meta = list())
#
# - restriction / outcome_construction carry the structured perturbation
#   record from the search spec (or NULL when none was applied). The
#   print banner and output generators render these via
#   render_restriction_phrase() / render_outcome_phrase() in
#   output_helpers.R.
#
# - r_squared is McFadden-style pseudo-R^2 for glm (1 - dev/null_dev).
# - p_value is the F-test against intercept for lm and the LRT against
#   intercept-only for glm. Both null-with-intercept references, so
#   choose_highlight() can compare across families.
# - family is a printable string ("lm", "glm:binomial:logit") so
#   favorite_method() in career.R can tabulate it directly.

# Shared materialisation: returns mf/X/y/formula or NULL if the spec
# cannot be fit (degenerate after transforms / restriction / outcome
# construction).
build_model_frame <- function(df, outcome, spec, y_override = NULL) {
  # y_override (when supplied by fit_glm for a coerce-mode family) must
  # be applied to the FULL df before materialise_spec runs, so the
  # subsequent restriction step trims the coerced outcome alongside the
  # rest of the row. Applying after materialise would desync row counts.
  if (!is.null(y_override) && length(y_override) == nrow(df) &&
      outcome %in% names(df)) {
    df[[outcome]] <- y_override
  }
  mat <- materialise_spec(df, outcome, spec)
  d <- mat$df
  if (nrow(d) < length(spec$subset) + 5L) return(NULL)
  f <- tryCatch(stats::as.formula(mat$formula), error = function(e) NULL)
  if (is.null(f)) return(NULL)

  # model.matrix() drops rows with NAs in the predictors silently,
  # which would desync X and y. Build a model frame first so X and y
  # stay aligned, then materialise X off the frame.
  mf <- tryCatch(stats::model.frame(f, data = d, na.action = stats::na.omit),
                 error = function(e) NULL)
  if (is.null(mf) || nrow(mf) < length(spec$subset) + 5L) return(NULL)
  X <- tryCatch(stats::model.matrix(f, data = mf), error = function(e) NULL)
  if (is.null(X)) return(NULL)
  y <- mf[[outcome]]
  list(mf = mf, X = X, y = y, formula = mat$formula)
}

# Family-independent metadata block, attached after the numeric fit.
# `restriction` / `outcome_construction` are carried as structured lists
# so render_*_phrase() can produce the methods-paragraph fragment at
# print/output time without re-deriving the sub-kind from a token.
# `rows_excluded` collapses every row-dropping reason (restriction +
# NA propagation) into a single banner number.
spec_metadata <- function(df, spec, n_used) {
  used <- if (is.null(n_used)) nrow(df) else as.integer(n_used)
  list(
    dropped              = setdiff(unlist(strsplit(
      spec$subset_full %||% paste(spec$subset, collapse = ","), ",")),
      spec$subset),
    n_dropped            = length(spec$subset_full %||% character()) -
                            length(spec$subset),
    rows_excluded        = max(0L, as.integer(nrow(df) - used)),
    restriction          = spec$restriction,
    outcome_construction = spec$outcome_construction,
    model_form           = spec$model_form
  )
}

# ---- lm fitter (the original .lm.fit() path) -----------------------

fit_lm <- function(df, outcome, spec) {
  mb <- build_model_frame(df, outcome, spec)
  if (is.null(mb)) return(NULL)
  X <- mb$X; y <- mb$y
  if (!all(is.finite(y))) {
    keep <- is.finite(y)
    X <- X[keep, , drop = FALSE]; y <- y[keep]
  }
  if (length(y) < ncol(X) + 2L) return(NULL)

  fit <- ols_fit_cpp(X, y)
  if (!isTRUE(fit$ok)) return(NULL)

  c(list(
    formula     = mb$formula,
    n           = fit$n,
    n_terms     = fit$p,
    r_squared   = fit$r_squared,
    aic         = fit$aic,
    p_value     = fit$p_value,
    family      = "lm",
    family_meta = list()
  ), spec_metadata(df, spec, fit$n))
}

# Materialise the (X, y) pair an lm spec would feed to ols_fit_cpp.
# Returns NULL if the spec is degenerate after transforms / outliers /
# subgroup or if the design has too few residual degrees of freedom.
# Used by fit_lm_batch() to assemble batches without paying the
# ols_fit_cpp call per spec.
build_lm_design <- function(df, outcome, spec) {
  mb <- build_model_frame(df, outcome, spec)
  if (is.null(mb)) return(NULL)
  X <- mb$X; y <- mb$y
  if (!all(is.finite(y))) {
    keep <- is.finite(y)
    X <- X[keep, , drop = FALSE]; y <- y[keep]
  }
  if (length(y) < ncol(X) + 2L) return(NULL)
  list(X = X, y = y, formula = mb$formula)
}

# Batch-fit a list of lm specs in one C++ crossing. Specs whose design
# does not materialise are returned as NULL slots so the caller can
# preserve original ordering. Returns a list of result rows in the same
# shape as fit_lm().
fit_lm_batch <- function(df, outcome, specs) {
  if (!length(specs)) return(list())

  designs <- lapply(specs, function(s) build_lm_design(df, outcome, s))
  ok      <- !vapply(designs, is.null, logical(1))
  out     <- vector("list", length(specs))

  if (!any(ok)) return(out)

  Xs <- lapply(designs[ok], `[[`, "X")
  ys <- lapply(designs[ok], `[[`, "y")
  fits <- ols_fit_batch_cpp(Xs, ys)

  ok_idx <- which(ok)
  for (j in seq_along(ok_idx)) {
    i   <- ok_idx[j]
    fit <- fits[[j]]
    if (is.null(fit) || !isTRUE(fit$ok)) next
    out[[i]] <- c(list(
      formula     = designs[[i]]$formula,
      n           = fit$n,
      n_terms     = fit$p,
      r_squared   = fit$r_squared,
      aic         = fit$aic,
      p_value     = fit$p_value,
      family      = "lm",
      family_meta = list()
    ), spec_metadata(df, specs[[i]], fit$n))
  }
  out
}

# ---- glm fitter ----------------------------------------------------
#
# Uses stats::glm.fit() directly to match the .lm.fit() aesthetic and
# avoid formula-parse overhead. Family/link comes from spec$family.
# When spec$family$outcome_coerce is non-NULL, the outcome is coerced
# before fitting -- this is the desperation flail (binomial on
# dichotomised continuous, poisson on rounded continuous). The
# resulting family string makes the coercion visible in run records.

fit_glm <- function(df, outcome, spec) {
  fam_spec <- spec$family
  fam_name <- fam_spec$family %||% "gaussian"
  link     <- fam_spec$link %||% glm_default_link(fam_name)
  coerce   <- fam_spec$outcome_coerce  # NULL or "median_dichotomize" / "round_clip"

  fam_obj <- tryCatch(make_family(fam_name, link), error = function(e) NULL)
  if (is.null(fam_obj)) return(NULL)

  # Coerce y first so model.frame sees the version glm.fit will use.
  y_override <- NULL
  if (!is.null(coerce)) {
    y_raw <- df[[outcome]]
    y_override <- coerce_outcome(y_raw, coerce)
    if (is.null(y_override)) return(NULL)
  }

  mb <- build_model_frame(df, outcome, spec, y_override = y_override)
  if (is.null(mb)) return(NULL)
  X <- mb$X; y <- mb$y

  # Drop non-finite y rows alongside X. glm.fit() will error otherwise.
  keep <- is.finite(y)
  if (!all(keep)) { X <- X[keep, , drop = FALSE]; y <- y[keep] }
  if (length(y) < ncol(X) + 2L) return(NULL)

  # Family-specific y validation. Out-of-domain outcomes (negatives for
  # poisson, non-{0,1} for binomial without weights) make glm.fit error;
  # check before fitting.
  if (!y_in_family_domain(y, fam_name)) return(NULL)

  fit <- tryCatch(
    suppressWarnings(stats::glm.fit(X, y, family = fam_obj,
                                     control = stats::glm.control(maxit = 50L))),
    error = function(e) NULL
  )
  if (is.null(fit) || !isTRUE(fit$converged)) return(NULL)

  n <- length(y); p <- ncol(X)
  dev      <- fit$deviance
  null_dev <- fit$null.deviance
  pseudo_r2 <- if (is.finite(null_dev) && null_dev > 0)
                  max(0, 1 - dev / null_dev) else 0

  # AIC via family$aic when available, otherwise from logLik analogue.
  aic_val <- tryCatch(
    fam_obj$aic(y, n, fit$fitted.values, rep(1, n), dev) + 2 * p,
    error = function(e) NA_real_
  )

  # LRT against intercept-only: chi-square on (null_dev - dev) with
  # df = (df.null - df.residual). Mirrors anova(fit, test="Chisq")'s
  # comparison to the null model.
  df_diff <- fit$df.null - fit$df.residual
  if (df_diff < 1L || !is.finite(null_dev - dev)) {
    p_value <- NA_real_
  } else {
    p_value <- stats::pchisq(null_dev - dev, df = df_diff, lower.tail = FALSE)
  }

  fam_label <- sprintf("glm:%s:%s", fam_name, link)
  if (!is.null(coerce)) fam_label <- paste0(fam_label, ":", coerce)

  c(list(
    formula     = mb$formula,
    n           = n,
    n_terms     = p,
    r_squared   = pseudo_r2,
    aic         = aic_val,
    p_value     = p_value,
    family      = fam_label,
    family_meta = list(family = fam_name, link = link, coerce = coerce,
                        deviance = dev, null_deviance = null_dev)
  ), spec_metadata(df, spec, n))
}

# Resolve a stats::family() object. Returning NULL on unknown link/family
# combinations lets the caller skip the spec gracefully.
make_family <- function(family, link) {
  fn <- switch(family,
    gaussian = stats::gaussian,
    binomial = stats::binomial,
    poisson  = stats::poisson,
    Gamma    = stats::Gamma,
    NULL
  )
  if (is.null(fn)) return(NULL)
  fn(link = link)
}

glm_default_link <- function(family) {
  switch(family,
    gaussian = "identity",
    binomial = "logit",
    poisson  = "log",
    Gamma    = "inverse",
    "identity"
  )
}

y_in_family_domain <- function(y, family) {
  switch(family,
    gaussian = TRUE,
    binomial = all(y >= 0 & y <= 1, na.rm = TRUE),
    poisson  = all(y >= 0 & abs(y - round(y)) < 1e-8, na.rm = TRUE),
    Gamma    = all(y > 0, na.rm = TRUE),
    TRUE
  )
}

# Outcome coercions for the desperation flail. Each returns a numeric
# vector or NULL when the input is not amenable.
coerce_outcome <- function(y, kind) {
  if (!is.numeric(y)) return(NULL)
  if (kind == "median_dichotomize") {
    m <- stats::median(y, na.rm = TRUE)
    if (!is.finite(m)) return(NULL)
    return(as.integer(y > m))
  }
  if (kind == "round_clip") {
    out <- pmax(0L, as.integer(round(y)))
    return(out)
  }
  NULL
}

# ---- gam fitter (native, Gaussian) ---------------------------------
#
# Penalised B-spline regression. For each numeric predictor with enough
# distinct values we generate a cubic B-spline basis and pair it with a
# 2nd-difference penalty (P-spline, Eilers & Marx 1996). Categorical and
# low-cardinality numeric predictors enter parametrically. A single
# shared smoothing parameter is selected by GCV on a log-spaced grid in
# pls_gcv_cpp(); per-smooth lambdas would need multi-D optimisation and
# would be overkill for the satirical fitting volume.
#
# Family is fixed at gaussian-identity. The selector deliberately maps
# every outcome shape onto gaussian for gam picks; the punch is "I used
# s()", not "I picked a clever link". Out-of-family specs (binomial /
# poisson) coerce to gaussian here rather than getting dropped.

fit_gam <- function(df, outcome, spec) {
  mat <- materialise_spec(df, outcome, spec)
  d <- mat$df
  if (nrow(d) < length(spec$subset) + 5L) return(NULL)
  if (!(outcome %in% names(d))) return(NULL)
  y <- d[[outcome]]
  if (!is.numeric(y)) return(NULL)
  keep <- is.finite(y)
  d <- d[keep, , drop = FALSE]
  y <- y[keep]
  if (length(y) < length(spec$subset) + 5L) return(NULL)

  build <- build_gam_design(d, spec, outcome)
  if (is.null(build)) return(NULL)
  X <- build$X
  S <- build$S
  if (ncol(X) < 1L || any(!is.finite(X))) return(NULL)

  log_lambdas <- seq(log(1e-4), log(1e6), length.out = 25L)
  res <- pls_gcv_cpp(X, y, S, log_lambdas)
  if (!isTRUE(res$ok)) return(NULL)

  n   <- length(y)
  rss <- res$rss
  edf <- res$edf
  tss <- sum((y - mean(y))^2)
  r2  <- if (tss > 0) max(0, 1 - rss / tss) else 0

  # F-test against intercept-only with effective DF.
  df1 <- max(1, edf - 1)
  df2 <- n - edf
  if (df2 < 1 || tss <= 0) {
    p_value <- NA_real_
  } else {
    f_stat  <- ((tss - rss) / df1) / (rss / df2)
    p_value <- stats::pf(f_stat, df1, df2, lower.tail = FALSE)
  }

  aic_val <- n * (log(2 * pi) + log(rss / n) + 1) + 2 * edf
  fam_label <- "gam:gaussian:identity"

  c(list(
    formula     = build$formula,
    n           = n,
    n_terms     = max(1L, as.integer(round(edf))),
    r_squared   = r2,
    aic         = aic_val,
    p_value     = p_value,
    family      = fam_label,
    family_meta = list(family = "gaussian", link = "identity",
                        edf = edf, lambda = res$lambda,
                        smoothed = build$smoothed)
  ), spec_metadata(df, spec, n))
}

# Build the augmented design matrix and block-diagonal penalty for a
# gam spec. Numeric predictors with enough distinct values become
# cubic B-spline blocks with 2nd-difference penalty; everything else
# (categoricals, low-card numerics, the intercept) is unpenalised.
# Returns NULL if no usable predictors materialise.
build_gam_design <- function(d, spec, outcome) {
  blocks         <- list(matrix(1, nrow(d), 1,
                                 dimnames = list(NULL, "(Intercept)")))
  penalty_blocks <- list(matrix(0, 1, 1))
  formula_terms  <- character()
  smoothed       <- character()

  for (v in spec$subset) {
    x <- d[[v]]
    if (is.numeric(x)) {
      uv <- unique(x[is.finite(x)])
      if (length(uv) >= 10L) {
        k <- min(8L, length(uv) - 1L)
        B <- tryCatch(bspline_basis(x, k = k),
                       error = function(e) NULL)
        if (is.null(B) || any(!is.finite(B))) next
        colnames(B) <- sprintf("%s.s%d", v, seq_len(ncol(B)))
        blocks[[length(blocks) + 1L]]                 <- B
        penalty_blocks[[length(penalty_blocks) + 1L]] <- second_diff_penalty(ncol(B))
        smoothed      <- c(smoothed, v)
        formula_terms <- c(formula_terms,
                            sprintf("s(%s, k = %d)", v, k))
        next
      }
      M <- matrix(x, ncol = 1L, dimnames = list(NULL, v))
    } else if (is.factor(x) || is.character(x)) {
      f <- as.factor(x)
      if (length(levels(f)) < 2L) next
      d_tmp <- data.frame(.f = f)
      M <- stats::model.matrix(~ .f, data = d_tmp)[, -1L, drop = FALSE]
      colnames(M) <- paste0(v, levels(f)[-1L])
    } else {
      next
    }
    blocks[[length(blocks) + 1L]]                 <- M
    penalty_blocks[[length(penalty_blocks) + 1L]] <- matrix(0, ncol(M), ncol(M))
    formula_terms <- c(formula_terms, v)
  }

  if (length(spec$interactions) && any(nzchar(spec$interactions))) {
    for (it in spec$interactions[nzchar(spec$interactions)]) {
      vars <- strsplit(it, ":")[[1]]
      vars <- vars[vars %in% names(d)]
      if (length(vars) < 2L) next
      cols <- lapply(vars, function(v) {
        x <- d[[v]]
        if (is.numeric(x)) x else as.numeric(as.factor(x))
      })
      x_int <- Reduce(`*`, cols)
      M <- matrix(x_int, ncol = 1L,
                   dimnames = list(NULL, paste(vars, collapse = ":")))
      blocks[[length(blocks) + 1L]]                 <- M
      penalty_blocks[[length(penalty_blocks) + 1L]] <- matrix(0, 1, 1)
      formula_terms <- c(formula_terms, it)
    }
  }

  X <- do.call(cbind, blocks)
  S <- block_diag_dense(penalty_blocks)
  if (length(formula_terms) == 0L) formula_terms <- "1"

  list(X = X, S = S, smoothed = smoothed,
       formula = paste(outcome, "~", paste(formula_terms, collapse = " + ")))
}

# Cubic B-spline basis with k columns, knots placed at evenly-spaced
# quantiles of the data. Wraps splines::bs() (recommended package, so
# always available) with intercept = FALSE so the column space pairs
# cleanly with the explicit intercept term in the design.
bspline_basis <- function(x, k = 8L, degree = 3L) {
  k <- as.integer(k)
  if (k < degree + 1L) k <- as.integer(degree + 1L)
  splines::bs(x, df = k, degree = degree, intercept = FALSE)
}

# 2nd-difference penalty matrix: D2'D2 where D2 is the (k-2 x k)
# 2nd-difference operator. Penalises wiggliness in the spline
# coefficients (Eilers & Marx 1996).
second_diff_penalty <- function(k) {
  if (k <= 2L) return(matrix(0, k, k))
  D <- diff(diag(k), differences = 2L)
  crossprod(D)
}

# Manual block-diagonal builder. Avoids a Matrix dependency. Mats may
# be 1x1; zero-row blocks are skipped.
block_diag_dense <- function(mats) {
  ns <- vapply(mats, function(m) nrow(m), integer(1))
  total <- sum(ns)
  out <- matrix(0, total, total)
  off <- 0L
  for (m in mats) {
    nr <- nrow(m); nc <- ncol(m)
    if (nr == 0L || nc == 0L) next
    out[(off + 1L):(off + nr), (off + 1L):(off + nc)] <- m
    off <- off + nr
  }
  out
}

# ---- glmm fitter (native, Gaussian random intercept) ---------------
#
# Random-intercept LMM with Gaussian outcome. The variance ratio
# theta = sigma_b^2 / sigma_e^2 is selected by profile maximum
# likelihood on a log-spaced grid in lmm_profile_cpp(); theta = 0
# (the lm reference) is always evaluated so the LRT against the
# fixed-only null is consistent.
#
# Family is fixed at gaussian-identity. As with gam, the joke is "I
# used a mixed model"; the fitter coerces non-Gaussian specs to
# gaussian rather than rejecting them.

fit_glmm <- function(df, outcome, spec) {
  group <- find_glmm_group(df, outcome, spec)
  if (is.null(group)) return(NULL)

  mat <- materialise_spec(df, outcome, spec)
  d <- mat$df
  if (!(group %in% names(d))) return(NULL)
  if (!(outcome %in% names(d))) return(NULL)
  if (nrow(d) < length(spec$subset) + 10L) return(NULL)

  d[[group]] <- as.factor(d[[group]])
  G <- length(levels(d[[group]]))
  if (G < 3L) return(NULL)

  y <- d[[outcome]]
  if (!is.numeric(y)) return(NULL)
  ok <- is.finite(y)
  d <- d[ok, , drop = FALSE]
  y <- y[ok]
  if (length(y) < length(spec$subset) + 10L) return(NULL)

  build <- build_glmm_design(d, spec, outcome, group)
  if (is.null(build)) return(NULL)
  X      <- build$X
  g_int  <- build$group_int
  G_used <- build$n_groups
  if (G_used < 3L) return(NULL)
  if (any(!is.finite(X))) return(NULL)

  log_thetas <- seq(log(1e-3), log(1e3), length.out = 25L)
  res <- lmm_profile_cpp(X, y, g_int, G_used, log_thetas)
  if (!isTRUE(res$ok)) return(NULL)

  # Null fit (theta = 0 -> ordinary lm). Compute residual variance and
  # Gaussian loglik analytically rather than calling lm() so the LRT
  # uses identical sufficient statistics.
  null_res <- lmm_profile_cpp(X, y, g_int, G_used, numeric(0))
  if (!isTRUE(null_res$ok)) return(NULL)

  n        <- length(y)
  ll       <- res$loglik
  null_ll  <- null_res$loglik
  delta_ll <- if (is.finite(ll) && is.finite(null_ll)) ll - null_ll
              else NA_real_

  # 50:50 mixture of chi^2_0 and chi^2_1: standard p-value correction
  # for testing a single variance component on the boundary theta = 0.
  p_value <- if (is.finite(delta_ll) && delta_ll > 0)
                0.5 * stats::pchisq(2 * delta_ll, df = 1L,
                                     lower.tail = FALSE)
             else NA_real_

  pseudo_r2 <- if (is.finite(delta_ll) && delta_ll > 0 && n > 0)
                  max(0, 1 - exp(-2 * delta_ll / n))
               else 0

  # AIC = -2 ll + 2 * (p_fixed + 2 variance components).
  aic_val <- -2 * ll + 2 * (ncol(X) + 2L)

  fixed_rhs <- if (length(spec$subset) == 0L) "1"
               else paste(spec$subset, collapse = " + ")
  if (length(spec$interactions) && any(nzchar(spec$interactions))) {
    its <- spec$interactions[nzchar(spec$interactions)]
    fixed_rhs <- paste(fixed_rhs, paste(its, collapse = " + "),
                        sep = " + ")
  }
  formula_str <- sprintf("%s ~ %s + (1 | %s)", outcome, fixed_rhs, group)

  c(list(
    formula     = formula_str,
    n           = n,
    n_terms     = ncol(X) + 1L,
    r_squared   = pseudo_r2,
    aic         = aic_val,
    p_value     = p_value,
    family      = "glmm:gaussian:identity",
    family_meta = list(family = "gaussian", link = "identity",
                        group = group, theta = res$theta,
                        sigma_e2 = res$sigma_e2, sigma_b2 = res$sigma_b2,
                        loglik = ll, null_loglik = null_ll)
  ), spec_metadata(df, spec, n))
}

# Build the fixed-effect design matrix for a glmm spec plus the
# 0-indexed group integer vector that the kernel expects. Categorical
# fixed-effect predictors are dummy-coded; numeric ones enter linearly.
build_glmm_design <- function(d, spec, outcome, group) {
  blocks <- list(matrix(1, nrow(d), 1,
                         dimnames = list(NULL, "(Intercept)")))

  for (v in spec$subset) {
    if (identical(v, group)) next
    x <- d[[v]]
    if (is.numeric(x)) {
      M <- matrix(x, ncol = 1L, dimnames = list(NULL, v))
    } else if (is.factor(x) || is.character(x)) {
      f <- as.factor(x)
      if (length(levels(f)) < 2L) next
      d_tmp <- data.frame(.f = f)
      M <- stats::model.matrix(~ .f, data = d_tmp)[, -1L, drop = FALSE]
      colnames(M) <- paste0(v, levels(f)[-1L])
    } else {
      next
    }
    blocks[[length(blocks) + 1L]] <- M
  }

  if (length(spec$interactions) && any(nzchar(spec$interactions))) {
    for (it in spec$interactions[nzchar(spec$interactions)]) {
      vars <- strsplit(it, ":")[[1]]
      vars <- vars[vars %in% names(d) & vars != group]
      if (length(vars) < 2L) next
      cols <- lapply(vars, function(v) {
        x <- d[[v]]
        if (is.numeric(x)) x else as.numeric(as.factor(x))
      })
      x_int <- Reduce(`*`, cols)
      M <- matrix(x_int, ncol = 1L,
                   dimnames = list(NULL, paste(vars, collapse = ":")))
      blocks[[length(blocks) + 1L]] <- M
    }
  }

  X <- do.call(cbind, blocks)
  group_int <- as.integer(d[[group]]) - 1L
  list(X = X, group_int = group_int,
       n_groups = length(levels(d[[group]])))
}

# ---- sem fitter (mediation, native, no optimisation) ---------------
#
# Single-mediator path model X -> M -> Y. Fitted as two OLS regressions
# via stats::.lm.fit() (same primitive as fit_lm) and a Sobel test for
# the indirect effect a*b. Spec convention:
#   spec$subset[1] = X     (independent variable)
#   spec$subset[2] = M     (mediator)
#   spec$subset[-(1:2)]    = optional Y-equation covariates that also
#                            enter the M equation as controls
# A spec with fewer than two numeric predictors returns NULL — the
# mediation move requires at least one X and one M.
#
# The reported p-value is the two-sided Sobel z for ab. r_squared and
# AIC come from the Y equation so the row stays comparable with the
# other families' single-equation summaries. Sobel's independence
# assumption between a-hat and b-hat is the standard approximation.

fit_sem <- function(df, outcome, spec) {
  if (length(spec$subset) < 2L) return(NULL)
  X_var <- spec$subset[[1L]]
  M_var <- spec$subset[[2L]]
  controls <- spec$subset[-c(1L, 2L)]
  if (X_var == M_var) return(NULL)

  mat <- materialise_spec(df, outcome, spec)
  d <- mat$df
  if (!(outcome %in% names(d)) ||
      !(X_var %in% names(d)) ||
      !(M_var %in% names(d))) return(NULL)

  y <- d[[outcome]]; xv <- d[[X_var]]; mv <- d[[M_var]]
  if (!is.numeric(y) || !is.numeric(xv) || !is.numeric(mv)) return(NULL)

  ctrl_cols <- list()
  for (cv in controls) {
    if (!(cv %in% names(d))) next
    z <- d[[cv]]
    if (is.numeric(z)) {
      ctrl_cols[[cv]] <- z
    } else if (is.factor(z) || is.character(z)) {
      f <- as.factor(z)
      if (length(levels(f)) < 2L) next
      M <- stats::model.matrix(~ f)[, -1L, drop = FALSE]
      colnames(M) <- paste0(cv, levels(f)[-1L])
      for (cc in colnames(M)) ctrl_cols[[cc]] <- M[, cc]
    }
  }

  ok <- is.finite(y) & is.finite(xv) & is.finite(mv)
  if (length(ctrl_cols)) {
    for (cc in names(ctrl_cols)) ok <- ok & is.finite(ctrl_cols[[cc]])
  }
  if (sum(ok) < 3L + length(ctrl_cols) + 5L) return(NULL)
  y <- y[ok]; xv <- xv[ok]; mv <- mv[ok]
  if (length(ctrl_cols))
    ctrl_cols <- lapply(ctrl_cols, function(z) z[ok])

  intercept <- rep(1, length(y))
  ctrl_mat <- if (length(ctrl_cols))
                do.call(cbind, ctrl_cols) else NULL

  # Equation a: M ~ X (+ controls)
  Xa <- if (is.null(ctrl_mat)) cbind(intercept, X = xv)
        else                    cbind(intercept, X = xv, ctrl_mat)
  fit_a <- tryCatch(stats::.lm.fit(Xa, mv), error = function(e) NULL)
  if (is.null(fit_a)) return(NULL)
  a_hat  <- fit_a$coefficients[2L]
  res_a  <- fit_a$residuals
  rss_a  <- sum(res_a * res_a)
  df_a   <- length(mv) - ncol(Xa)
  if (df_a < 1L) return(NULL)
  sigma2_a <- rss_a / df_a
  XtX_inv_a <- tryCatch(chol2inv(chol(crossprod(Xa))),
                          error = function(e) NULL)
  if (is.null(XtX_inv_a)) return(NULL)
  var_a <- sigma2_a * XtX_inv_a[2L, 2L]
  if (!is.finite(var_a) || var_a <= 0) return(NULL)

  # Equation b/c': Y ~ X + M (+ controls)
  Xb <- if (is.null(ctrl_mat)) cbind(intercept, X = xv, M = mv)
        else                    cbind(intercept, X = xv, M = mv, ctrl_mat)
  fit_b <- tryCatch(stats::.lm.fit(Xb, y), error = function(e) NULL)
  if (is.null(fit_b)) return(NULL)
  c_prime <- fit_b$coefficients[2L]
  b_hat   <- fit_b$coefficients[3L]
  res_b   <- fit_b$residuals
  rss_b   <- sum(res_b * res_b)
  n       <- length(y)
  df_b    <- n - ncol(Xb)
  if (df_b < 1L) return(NULL)
  sigma2_b <- rss_b / df_b
  XtX_inv_b <- tryCatch(chol2inv(chol(crossprod(Xb))),
                          error = function(e) NULL)
  if (is.null(XtX_inv_b)) return(NULL)
  var_b <- sigma2_b * XtX_inv_b[3L, 3L]
  if (!is.finite(var_b) || var_b <= 0) return(NULL)

  ab <- a_hat * b_hat
  sobel_se <- sqrt(b_hat * b_hat * var_a + a_hat * a_hat * var_b)
  if (!is.finite(sobel_se) || sobel_se <= 0) {
    p_value <- NA_real_
    sobel_z <- NA_real_
  } else {
    sobel_z <- ab / sobel_se
    p_value <- 2 * stats::pnorm(-abs(sobel_z))
  }

  tss_y <- sum((y - mean(y))^2)
  r2 <- if (tss_y > 0) max(0, 1 - rss_b / tss_y) else 0
  aic_val <- n * (log(2 * pi) + log(rss_b / n) + 1) + 2 * ncol(Xb)

  control_label <- if (length(controls)) paste(" +", paste(controls, collapse = " + "))
                   else ""
  formula_str <- sprintf("%s <- %s (via %s)%s",
                          outcome, X_var, M_var, control_label)

  c(list(
    formula     = formula_str,
    n           = n,
    n_terms     = ncol(Xb),
    r_squared   = r2,
    aic         = aic_val,
    p_value     = p_value,
    family      = "sem:mediation",
    family_meta = list(family = "sem", model = "mediation",
                        X = X_var, M = M_var, controls = controls,
                        a = unname(a_hat), b = unname(b_hat),
                        c_prime = unname(c_prime),
                        ab = unname(ab),
                        sobel_z = unname(sobel_z),
                        sobel_se = unname(sobel_se))
  ), spec_metadata(df, spec, n))
}

# ---- cor fitter (bivariate Pearson / Spearman / Kendall) -----------
#
# The bare-correlation move: spec must contain exactly one predictor;
# we report Pearson / Spearman / Kendall r between outcome and that
# predictor, plus the standard test's p-value. The method is carried on
# spec$family$method so the selector controls which flavour gets used.

fit_cor <- function(df, outcome, spec) {
  if (length(spec$subset) != 1L) return(NULL)
  pred <- spec$subset[[1L]]
  method <- spec$family$method %||% "pearson"
  if (!(method %in% c("pearson", "spearman", "kendall"))) return(NULL)

  mat <- materialise_spec(df, outcome, spec)
  d <- mat$df
  if (!(outcome %in% names(d)) || !(pred %in% names(d))) return(NULL)
  y <- d[[outcome]]; x <- d[[pred]]
  if (!is.numeric(y) || !is.numeric(x)) return(NULL)
  ok <- is.finite(y) & is.finite(x)
  if (sum(ok) < 5L) return(NULL)
  y <- y[ok]; x <- x[ok]
  if (stats::sd(y) == 0 || stats::sd(x) == 0) return(NULL)

  ct <- tryCatch(
    suppressWarnings(stats::cor.test(x, y, method = method)),
    error = function(e) NULL
  )
  if (is.null(ct)) return(NULL)
  r <- unname(ct$estimate)
  p <- ct$p.value
  n <- length(y)

  r2 <- if (is.finite(r)) r * r else 0
  # Gaussian-style AIC of a one-parameter "predictor of y" model. Lets
  # the row sit on the same scale as the lm AICs in choose_highlight.
  rss <- (1 - r2) * sum((y - mean(y))^2)
  aic_val <- if (rss > 0)
                n * (log(2 * pi) + log(rss / n) + 1) + 2 * 2L
             else NA_real_

  fam_label <- sprintf("cor:%s", method)

  c(list(
    formula     = sprintf("%s ~ %s", outcome, pred),
    n           = n,
    n_terms     = 2L,
    r_squared   = r2,
    aic         = aic_val,
    p_value     = p,
    family      = fam_label,
    family_meta = list(family = "cor", method = method, r = r)
  ), spec_metadata(df, spec, n))
}

# ---- wls fitter (heteroscedasticity-corrected lm, two-stage) -------
#
# Standard two-stage feasible WLS: fit OLS, regress |residuals| on the
# same predictors to get a sigma trend, refit on sqrt(W)*X and
# sqrt(W)*y with W = 1 / sigma_hat^2. The result row reports the
# weighted F-test against the intercept-only model and a weighted R^2.
# All linear algebra goes through stats::.lm.fit() — the same primitive
# fit_lm uses — so no new C++ surface is needed.

fit_wls <- function(df, outcome, spec) {
  mb <- build_model_frame(df, outcome, spec)
  if (is.null(mb)) return(NULL)
  X <- mb$X; y <- mb$y
  if (!all(is.finite(y))) {
    keep <- is.finite(y)
    X <- X[keep, , drop = FALSE]; y <- y[keep]
  }
  if (length(y) < ncol(X) + 5L) return(NULL)

  # Stage 1: ordinary lm.
  fit1 <- tryCatch(stats::.lm.fit(X, y), error = function(e) NULL)
  if (is.null(fit1)) return(NULL)
  abs_resid <- abs(fit1$residuals)
  if (any(!is.finite(abs_resid))) return(NULL)
  if (max(abs_resid) <= .Machine$double.eps * 4) return(NULL)

  # Stage 2: regress |resid| on X to estimate the sigma trend. Use the
  # fitted values as sigma_hat, clipped from below so weights stay
  # finite even when the trend dips to zero on the boundary. .lm.fit()
  # returns residuals but not fitted.values, so we reconstruct
  # fitted = y - residuals.
  fit2 <- tryCatch(stats::.lm.fit(X, abs_resid), error = function(e) NULL)
  if (is.null(fit2)) return(NULL)
  sigma_hat <- abs_resid - fit2$residuals
  floor_s <- max(1e-6 * stats::median(abs_resid, na.rm = TRUE),
                  .Machine$double.eps)
  sigma_hat <- pmax(sigma_hat, floor_s)
  w <- 1 / (sigma_hat * sigma_hat)
  sw <- sqrt(w)

  # Stage 3: weighted lm via sqrt(w) rescaling — algebraically identical
  # to solving (X' W X) beta = X' W y, but routed through .lm.fit() for
  # the QR path.
  Xw <- X * sw
  yw <- y * sw
  fit3 <- tryCatch(stats::.lm.fit(Xw, yw), error = function(e) NULL)
  if (is.null(fit3)) return(NULL)

  n <- length(y); p <- ncol(X)
  res_w <- fit3$residuals
  rss_w <- sum(res_w * res_w)
  # Weighted total SS (against the weighted-mean reference).
  y_w_mean <- sum(w * y) / sum(w)
  tss_w <- sum(w * (y - y_w_mean)^2)
  r2 <- if (tss_w > 0) max(0, 1 - rss_w / tss_w) else 0

  df1 <- max(1L, p - 1L)
  df2 <- n - p
  if (df2 < 1L || tss_w <= 0) {
    p_value <- NA_real_
  } else {
    f_stat <- ((tss_w - rss_w) / df1) / (rss_w / df2)
    p_value <- stats::pf(f_stat, df1, df2, lower.tail = FALSE)
  }

  # AIC of the weighted Gaussian model: n*log(2*pi) + n*log(rss_w/n) +
  # n + 2*p - sum(log(w)). The log-weight correction makes the AIC
  # comparable in scale to fit_lm.
  aic_val <- n * (log(2 * pi) + log(rss_w / n) + 1) + 2 * p -
              sum(log(w))

  c(list(
    formula     = mb$formula,
    n           = n,
    n_terms     = p,
    r_squared   = r2,
    aic         = aic_val,
    p_value     = p_value,
    family      = "wls",
    family_meta = list(family = "wls",
                        stage2_r2 =
                          if (sum(abs_resid^2) > 0)
                            max(0, 1 - sum(fit2$residuals^2) /
                                       sum((abs_resid - mean(abs_resid))^2))
                          else 0,
                        weight_floor = floor_s)
  ), spec_metadata(df, spec, n))
}

# Pick a column suitable as a random-intercept grouping factor: outside
# the spec's fixed-effect subset, with a small-but-not-trivial number of
# distinct levels. Returns the column name or NULL.
#
# When the spec carries a model_form override naming a specific group,
# that takes priority -- the perturbation chose the group and we honor
# it as long as it's actually a viable column.
find_glmm_group <- function(df, outcome, spec) {
  forced <- spec$family$group %||% spec$model_form$group %||% NULL
  if (!is.null(forced) && forced %in% names(df) && forced != outcome) {
    x <- df[[forced]]
    levs <- unique(x[!is.na(x)])
    if (length(levs) >= 3L) return(forced)
  }
  cand <- setdiff(names(df), c(outcome, spec$subset))
  if (length(cand) == 0L) return(NULL)
  n <- nrow(df)
  cap <- max(3L, n %/% 3L)
  for (v in cand) {
    x <- df[[v]]
    levs <- unique(x[!is.na(x)])
    n_levs <- length(levs)
    if (n_levs < 3L || n_levs > cap) next
    if (is.factor(x) || is.character(x)) return(v)
    if (is.numeric(x) && all(abs(x - round(x)) < 1e-8, na.rm = TRUE))
      return(v)
  }
  NULL
}

# ---- rlm fitter (native Huber M-estimator) -------------------------
#
# Iteratively reweighted least squares with Huber's psi as the
# influence function. Mirrors the high-level behaviour of MASS::rlm
# (Huber proposal 2 scale, c = 1.345) but stays inside the package's
# .lm.fit() / chol primitives so MASS / lme4 do not become hard deps.
#
# Scale is updated alongside the coefficients via the median-absolute-
# deviation of the current residuals: s = 1.4826 * MAD. The weight at
# step t is w_i = psi(r_i / s) / (r_i / s), so leverage points whose
# scaled residual exceeds c get downweighted by c / |r/s|.
#
# Reported p_value is a weighted F-test against the weighted intercept-
# only model -- the same shape fit_wls reports, so choose_highlight() can
# put rlm rows next to lm/glm rows in the just-cleared band.

fit_rlm <- function(df, outcome, spec) {
  mb <- build_model_frame(df, outcome, spec)
  if (is.null(mb)) return(NULL)
  X <- mb$X; y <- mb$y
  if (!all(is.finite(y))) {
    keep <- is.finite(y)
    X <- X[keep, , drop = FALSE]; y <- y[keep]
  }
  n <- length(y); p <- ncol(X)
  if (n < p + 5L) return(NULL)

  fit0 <- tryCatch(stats::.lm.fit(X, y), error = function(e) NULL)
  if (is.null(fit0)) return(NULL)
  beta <- fit0$coefficients
  if (any(!is.finite(beta))) return(NULL)
  resid <- fit0$residuals

  c_huber <- 1.345
  maxit   <- 25L
  tol     <- 1e-6

  for (it in seq_len(maxit)) {
    mad_s <- stats::median(abs(resid - stats::median(resid, na.rm = TRUE)),
                            na.rm = TRUE) * 1.4826
    if (!is.finite(mad_s) || mad_s <= .Machine$double.eps)
      mad_s <- max(1e-6, stats::sd(resid, na.rm = TRUE))
    if (!is.finite(mad_s) || mad_s <= 0) return(NULL)

    r_scaled <- resid / mad_s
    w        <- ifelse(abs(r_scaled) <= c_huber, 1, c_huber / abs(r_scaled))
    w[!is.finite(w) | w < 0] <- 0
    if (sum(w) <= .Machine$double.eps) return(NULL)

    sw <- sqrt(w)
    fit_w <- tryCatch(stats::.lm.fit(X * sw, y * sw), error = function(e) NULL)
    if (is.null(fit_w)) return(NULL)
    new_beta <- fit_w$coefficients
    if (any(!is.finite(new_beta))) return(NULL)
    delta <- max(abs(new_beta - beta))
    beta <- new_beta
    resid <- as.numeric(y - X %*% beta)
    if (is.finite(delta) && delta < tol) break
  }

  # Weighted RSS / TSS for the F-test. Using the final weights so the
  # significance reading reflects the iterated estimate, not the OLS pilot.
  mad_s <- stats::median(abs(resid - stats::median(resid, na.rm = TRUE)),
                          na.rm = TRUE) * 1.4826
  if (!is.finite(mad_s) || mad_s <= 0) mad_s <- max(1e-6, stats::sd(resid))
  r_scaled <- resid / mad_s
  w_final  <- ifelse(abs(r_scaled) <= c_huber, 1, c_huber / abs(r_scaled))
  w_final[!is.finite(w_final) | w_final < 0] <- 0

  rss_w <- sum(w_final * resid * resid)
  y_w_mean <- sum(w_final * y) / max(sum(w_final), .Machine$double.eps)
  tss_w <- sum(w_final * (y - y_w_mean)^2)
  r2    <- if (tss_w > 0) max(0, 1 - rss_w / tss_w) else 0

  df1 <- max(1L, p - 1L)
  df2 <- n - p
  p_value <- if (df2 < 1L || tss_w <= 0 || rss_w <= 0) NA_real_
             else {
               f_stat <- ((tss_w - rss_w) / df1) / (rss_w / df2)
               stats::pf(f_stat, df1, df2, lower.tail = FALSE)
             }

  # AIC analogue: Gaussian-form on the weighted residuals. Keeps rlm
  # rows comparable in scale to fit_lm / fit_wls.
  aic_val <- n * (log(2 * pi) + log(rss_w / n) + 1) + 2 * p

  c(list(
    formula     = mb$formula,
    n           = n,
    n_terms     = p,
    r_squared   = r2,
    aic         = aic_val,
    p_value     = p_value,
    family      = "rlm:huber",
    family_meta = list(family = "rlm", psi = "huber", c = c_huber,
                        scale = mad_s, iterations = it)
  ), spec_metadata(df, spec, n))
}

# ---- registry ------------------------------------------------------

# Shared batch scaffold. Every non-lm family currently routes through
# this loop -- one R-level iteration per spec, calling the family's
# single-spec fitter. The named fit_*_batch wrappers below are the
# insertion points for future C++ batch kernels: replacing a wrapper
# body with a single call to a `*_fit_batch_cpp` kernel collapses the
# group into one R/C++ crossing without touching the dispatcher in
# fit_specs_batch() or the FAMILIES registry.
batch_via_loop <- function(df, outcome, specs, single_fitter) {
  lapply(specs, function(s) single_fitter(df, outcome, s))
}

# TODO(C++ batch): replace body with a glm_fit_batch_cpp() kernel that
# runs IRLS per spec, dispatches family/link internally, and returns the
# deviance / null-deviance / LRT pieces that fit_glm() reads.
fit_glm_batch <- function(df, outcome, specs) {
  batch_via_loop(df, outcome, specs, fit_glm)
}

# TODO(C++ batch): replace body with a wls_fit_batch_cpp() kernel that
# does the two-stage (OLS -> |resid| trend -> weighted refit) in one
# crossing, mirroring fit_wls().
fit_wls_batch <- function(df, outcome, specs) {
  batch_via_loop(df, outcome, specs, fit_wls)
}

# TODO(C++ batch): pls_gcv_cpp() already does the per-spec heavy lift;
# a gam_fit_batch_cpp() that builds the B-spline + penalty stack inside
# C++ would amortise the basis-construction + R/C++ crossing across the
# group.
fit_gam_batch <- function(df, outcome, specs) {
  batch_via_loop(df, outcome, specs, fit_gam)
}

# TODO(C++ batch): native Huber M-estimator is an IRLS loop; an
# rlm_fit_batch_cpp() that runs the reweighting iterations in C++ keeps
# the residual / weight vectors hot across specs.
fit_rlm_batch <- function(df, outcome, specs) {
  batch_via_loop(df, outcome, specs, fit_rlm)
}

# TODO(C++ batch): lmm_profile_cpp() is the single-spec kernel; a
# glmm_fit_batch_cpp() that loops the profile likelihood in C++ keeps
# the optimisation state hot across specs.
fit_glmm_batch <- function(df, outcome, specs) {
  batch_via_loop(df, outcome, specs, fit_glmm)
}

# TODO(C++ batch): bivariate Pearson/Spearman/Kendall is small and
# pure -- a cor_fit_batch_cpp() that loops three correlation routines
# is the cheapest kernel to write and the cleanest proof of the
# scaffold.
fit_cor_batch <- function(df, outcome, specs) {
  batch_via_loop(df, outcome, specs, fit_cor)
}

# TODO(C++ batch): linear SEM (path / mediation) reduces to a sequence
# of OLS fits; a sem_fit_batch_cpp() could reuse ols_fit_batch_cpp()
# internally for the inner regressions.
fit_sem_batch <- function(df, outcome, specs) {
  batch_via_loop(df, outcome, specs, fit_sem)
}

FAMILIES <- list(
  lm   = list(fitter = fit_lm,   batch_fitter = fit_lm_batch,
              available = function() TRUE,
              min_career = "Junior Researcher"),
  cor  = list(fitter = fit_cor,  batch_fitter = fit_cor_batch,
              available = function() TRUE,
              min_career = "Postdoc"),
  glm  = list(fitter = fit_glm,  batch_fitter = fit_glm_batch,
              available = function() TRUE,
              min_career = "Postdoc"),
  wls  = list(fitter = fit_wls,  batch_fitter = fit_wls_batch,
              available = function() TRUE,
              min_career = "Senior Scientist"),
  gam  = list(fitter = fit_gam,  batch_fitter = fit_gam_batch,
              available = function() TRUE,
              min_career = "Senior Scientist"),
  rlm  = list(fitter = fit_rlm,  batch_fitter = fit_rlm_batch,
              available = function() TRUE,
              min_career = "Senior Scientist"),
  glmm = list(fitter = fit_glmm, batch_fitter = fit_glmm_batch,
              available = function() TRUE,
              min_career = "PI"),
  sem  = list(fitter = fit_sem,  batch_fitter = fit_sem_batch,
              available = function() TRUE,
              min_career = "PI")
)

family_fitter <- function(name) {
  reg <- FAMILIES[[name]]
  if (is.null(reg)) stop(sprintf("Unknown model family: %s", name),
                          call. = FALSE)
  reg$fitter
}

family_batch_fitter <- function(name) {
  reg <- FAMILIES[[name]]
  if (is.null(reg)) stop(sprintf("Unknown model family: %s", name),
                          call. = FALSE)
  reg$batch_fitter
}

available_families <- function(career_level) {
  out <- character()
  for (name in names(FAMILIES)) {
    reg <- FAMILIES[[name]]
    if (!isTRUE(reg$available())) next
    if (tier_index(career_level) < tier_index(reg$min_career)) next
    out <- c(out, name)
  }
  if (length(out) == 0L) "lm" else out
}

#' Package options for texanshootR
#'
#' All options are read at the time of use, so changing them takes
#' effect immediately for subsequent calls.
#'
#' \describe{
#'   \item{`texanshootR.animations`}{Logical. Enable terminal
#'     animations and ANSI cursor effects. Default: `TRUE`. Forced off
#'     in non-interactive sessions and when the terminal lacks ANSI
#'     support.}
#'   \item{`texanshootR.output_dir`}{Character or `NULL`. Default
#'     directory for output files. `NULL` falls back to
#'     [tempdir()].}
#'   \item{`texanshootR.save_enabled`}{Logical. Enable persistent
#'     career data. `FALSE` runs entirely in memory. Default: `TRUE`.}
#'   \item{`texanshootR.quiet`}{Logical. Suppress status prints from
#'     output functions. Default: `FALSE`.}
#'   \item{`texanshootR.event_rate`}{Numeric in `[0, 1]`. Per-run
#'     probability of a life-event encounter. Default: `1/6`.}
#'   \item{`texanshootR.life_events`}{Logical. Enable life-event
#'     encounters during runs. Default: `TRUE`.}
#'   \item{`texanshootR.ui_mode`}{Optional character override for the
#'     TUI rendering mode. One of `"ansi"`, `"dynamic"`, `"plain"`.
#'     Unset (the default) means auto-detect. Use this if auto-detect
#'     picks the wrong mode for your terminal (e.g., a remote RStudio
#'     session that does not report `.Platform$GUI == "RStudio"`).}
#' }
#' @name texanshootR-options
NULL

.opt_defaults <- list(
  texanshootR.animations   = TRUE,
  texanshootR.output_dir   = NULL,
  texanshootR.save_enabled = TRUE,
  texanshootR.quiet        = FALSE,
  texanshootR.event_rate   = 1 / 6,
  texanshootR.life_events  = TRUE
)

#' @keywords internal
opt <- function(name, default = .opt_defaults[[name]]) {
  v <- getOption(name, default)
  if (is.null(v)) default else v
}

#' @keywords internal
set_opt_defaults <- function() {
  for (nm in names(.opt_defaults)) {
    if (is.null(getOption(nm))) {
      args <- stats::setNames(list(.opt_defaults[[nm]]), nm)
      do.call(options, args)
    }
  }
  invisible()
}

# Package environment used as a cache for the message and event
# registries, the achievement / cosmetic registries, and a few derived
# lookups. Populated by `.onLoad`.
.tx <- new.env(parent = emptyenv())

.onLoad <- function(libname, pkgname) {
  set_opt_defaults()

  # Lazy: heavy registry parsing happens on first use, not at load.
  # We just initialise empty slots here so getter helpers can dispatch.
  .tx$messages     <- NULL
  .tx$events       <- NULL
  .tx$achievements <- NULL
  .tx$cosmetics    <- NULL
  .tx$mascot_frames <- NULL

  invisible()
}

.onAttach <- function(libname, pkgname) {
  if (!interactive()) return(invisible())
  if (isTRUE(opt("texanshootR.quiet"))) return(invisible())
  if (Sys.getenv("R_TESTS") != "") return(invisible())
  banner_maybe_print()
  invisible()
}

#' Generate a chaotic conference-style PPTX
#'
#' Builds a multi-slide PowerPoint via `officer`. v1 produces the
#' structural chaos (rotated text, gradient fills, irrelevant clipart,
#' shape collisions); slide-transition and per-element animation XML
#' injection lands in a follow-up.
#'
#' Chain stage: \strong{presentation} (length 3). See [progress()].
#'
#' @inheritParams manuscript
#' @return A length-one character vector giving the path to the written
#'   PPTX file, returned invisibly. Called for its side effect of
#'   generating the slide deck in `output_dir` and advancing the
#'   publication chain.
#' @export
presentation <- function(run, output_dir = NULL, file = NULL,
                         force = FALSE) {
  require_chain_stage("presentation", run)
  require_pkg("officer", "presentation")
  d <- resolve_output_dir(output_dir)
  stem <- file %||% "presentation"
  out  <- versioned_filename(d, stem, "pptx", force)

  hs <- run$highlighted_spec %||% list()

  pp <- officer::read_pptx()

  pp <- officer::add_slide(pp, layout = "Title Slide", master = "Office Theme")
  pp <- officer::ph_with(pp, "A Specification-Aware Reanalysis",
                         location = officer::ph_location_type(type = "ctrTitle"))
  pp <- officer::ph_with(pp, format(Sys.Date(), "%B %Y"),
                         location = officer::ph_location_type(type = "subTitle"))

  pp <- officer::add_slide(pp, layout = "Title and Content", master = "Office Theme")
  pp <- officer::ph_with(pp, "Conceptual Framework",
                         location = officer::ph_location_type(type = "title"))
  pp <- officer::ph_with(pp, c(
    "* Latent ecological process",
    "* -> Mechanistic pathway",
    "* -> Observational signal",
    "* -> Methodological filter",
    "* -> Reportable result"
  ), location = officer::ph_location_type(type = "body"))

  pp <- officer::add_slide(pp, layout = "Title and Content", master = "Office Theme")
  pp <- officer::ph_with(pp, "Methods",
                         location = officer::ph_location_type(type = "title"))
  pp <- officer::ph_with(pp, sprintf(
    "%d candidate specifications. %s",
    run$spec_count %||% 0L, describe_spec(hs)
  ), location = officer::ph_location_type(type = "body"))

  pp <- officer::add_slide(pp, layout = "Title and Content", master = "Office Theme")
  pp <- officer::ph_with(pp, "Result",
                         location = officer::ph_location_type(type = "title"))
  pp <- officer::ph_with(pp, sprintf(
    "R-squared %.3f. p = %.4f.",
    hs$r_squared %||% 0, hs$p_value %||% 1
  ), location = officer::ph_location_type(type = "body"))

  pp <- officer::add_slide(pp, layout = "Title and Content", master = "Office Theme")
  pp <- officer::ph_with(pp, "Discussion",
                         location = officer::ph_location_type(type = "title"))
  pp <- officer::ph_with(pp, c(
    "The result suggests a mechanism.",
    "The mechanism is consistent with the framework.",
    "The framework is consistent with the result.",
    "Further work is required."
  ), location = officer::ph_location_type(type = "body"))

  pp <- officer::add_slide(pp, layout = "Title Slide", master = "Office Theme")
  pp <- officer::ph_with(pp, "Q & A",
                         location = officer::ph_location_type(type = "ctrTitle"))

  status_open(out)
  print(pp, target = out)
  status_close()
  record_output(run, "presentation", out)
  advance_chain_after_stage("presentation")
  invisible(out)
}

# Sandbox every test from the user's real save dir.
#
# This file is sourced by `pkgload::load_all(helpers = TRUE)`. testthat
# triggers that call from inside `test_files_setup_env()` as part of
# setting up a test run; devtools triggers it independently when the
# user is just iterating interactively. Both paths sourcing the same
# helper means a naive top-level `options(...)` would leak the test
# sandbox (notably `texanshootR.animations = FALSE`) into the
# interactive session and silently disable the TUI animation.
#
# We can't gate on `Sys.getenv("TESTTHAT") == "true"` here: testthat
# only sets that env var *after* load_all returns. By that point the
# helper has already been sourced and won't be sourced again. So we
# detect the test path the only way it's actually distinguishable at
# source time -- by looking up the call stack for testthat's test
# entry points.
in_test_setup <- function() {
  calls <- as.character(sys.calls())
  any(grepl("^(testthat::)?(test_local|test_dir|test_files|test_files_serial|test_files_setup_env)\\b",
            calls))
}

if (in_test_setup()) {
  .tx_test_dir <- file.path(tempdir(), paste0("tx-test-",
                                              as.integer(Sys.time())))
  dir.create(.tx_test_dir, showWarnings = FALSE, recursive = TRUE)
  options(
    texanshootR.save_dir     = .tx_test_dir,
    texanshootR.save_enabled = TRUE,
    texanshootR.consent      = TRUE,
    texanshootR.animations   = FALSE,
    texanshootR.quiet        = TRUE,
    texanshootR.life_events  = FALSE
  )
}

#' Generate a response-to-reviewers DOCX
#'
#' Polite, point-by-point response to imagined reviewer comments.
#'
#' Chain stage: \strong{reviewer_response} (length 4). See [progress()].
#'
#' @inheritParams manuscript
#' @return A length-one character vector giving the path to the written
#'   DOCX file, returned invisibly. Called for its side effect of
#'   generating the response-to-reviewers document in `output_dir` and
#'   advancing the publication chain.
#' @export
reviewer_response <- function(run, output_dir = NULL, file = NULL,
                              force = FALSE) {
  require_chain_stage("reviewer_response", run)
  require_pkg("officer", "reviewer_response")

  # Roll the reviewer outcome the first time the player asks for a
  # response. Subsequent calls reuse the persisted verdict. The
  # `received_a_reject` achievement (and any future outcome-driven
  # trigger) fires via record_output() below, which re-reads the
  # updated record and re-runs evaluate_achievements().
  run <- materialize_reviewer_outcome(run)

  d <- resolve_output_dir(output_dir)
  stem <- file %||% "response_to_reviewers_FINAL"
  out  <- versioned_filename(d, stem, "docx", force)

  hs <- run$highlighted_spec %||% list()
  methods <- describe_spec(hs)

  doc <- officer::read_docx()
  doc <- officer::body_add_par(doc, "Response to Reviewers", style = "heading 1")
  doc <- officer::body_add_par(doc, "We thank the reviewers for their thorough engagement with the manuscript.")
  doc <- officer::body_add_par(doc, paste0(
    "For reference, the analysis presented in the manuscript is summarised as: ",
    methods))
  doc <- officer::body_add_par(doc, "")

  comments <- list(
    list(reviewer = "Reviewer 1",
         comment  = "The conceptual framework should be tightened.",
         response = "We have tightened the conceptual framework."),
    list(reviewer = "Reviewer 2",
         comment  = "Additional analyses are warranted.",
         response = "Additional analyses, consistent with our prior framing, have been added."),
    list(reviewer = "Reviewer 1",
         comment  = "Clarify the choice of transformation.",
         response = "The chosen transformation is consistent with the literature."),
    list(reviewer = "Reviewer 2",
         comment  = "Sample size limits inference.",
         response = "We have noted this limitation. The reported specification remains robust within the analysed subset.")
  )

  for (c in comments) {
    doc <- officer::body_add_par(doc, c$reviewer, style = "heading 2")
    doc <- officer::body_add_par(doc, paste("Comment:", c$comment))
    doc <- officer::body_add_par(doc, paste("Response:", c$response))
    doc <- officer::body_add_par(doc, "")
  }

  status_open(out)
  print(doc, target = out)
  status_close()
  record_output(run, "reviewer_response", out)
  advance_chain_after_stage("reviewer_response")
  invisible(out)
}

# Reviewer 2 probabilistic encounter.
#
# Outcomes (weighted): accept_enthusiasm | minor | major | reject.
# Weights shift with the run's reviewer_resistance modifier - a more
# resistant run is more likely to land at major / reject (the joke is
# that reviewer 2 is *more* hostile when the work pushes harder).

# Lazily roll the reviewer outcome for `run`. First call rolls using
# the run's accumulated resistance, writes the outcome back to the
# on-disk run record so subsequent reviewer_response() calls reuse the
# same verdict, and applies the resistance delta to the cumulative
# meta$hidden$reviewer_resistance counter (same * 0.05 scaling
# update_career uses for the other hidden modifiers). Returns the run
# with `reviewer_outcome` populated.
#
# A run that already has a non-NA reviewer_outcome is returned
# unchanged -- the verdict is part of the audit trail, not something
# the player can re-roll.
materialize_reviewer_outcome <- function(run) {
  existing <- run$reviewer_outcome %||% NA_character_
  if (!is.na(existing) && nzchar(existing)) return(run)

  resistance <- run$modifiers$reviewer_resistance %||% 0
  rev <- reviewer_roll(reviewer_resistance = resistance)
  run$reviewer_outcome <- rev$outcome

  # Persist on the run record so the next call to reviewer_response()
  # sees the same outcome.
  d <- save_dir()
  if (!is.null(d)) {
    fpath <- file.path(d, "runs", paste0(run$run_id, ".rds"))
    if (file.exists(fpath)) {
      rec <- readRDS(fpath)
      rec$reviewer_outcome <- rev$outcome
      saveRDS(rec, fpath)
    }
  }

  # Cumulative resistance bookkeeping. Mirrors the * 0.05 scaling that
  # update_career() applies to the other run$modifiers entries at
  # finalize time.
  meta <- read_meta()
  if (!is.null(meta)) {
    meta$hidden <- meta$hidden %||% list()
    meta$hidden$reviewer_resistance <-
      (meta$hidden$reviewer_resistance %||% 0) + rev$resist_delta * 0.05
    write_meta(meta)
  }

  run
}

reviewer_roll <- function(reviewer_resistance = 0,
                          career_level = "Junior Researcher") {
  base <- c(accept_enthusiasm = 0.05,
            minor             = 0.30,
            major             = 0.45,
            reject            = 0.20)
  # Higher resistance shifts mass toward the harsh end. Clip to [-0.3, 0.3].
  r <- max(-0.3, min(0.3, reviewer_resistance))
  base["accept_enthusiasm"] <- max(0, base["accept_enthusiasm"] - r * 0.05)
  base["minor"]             <- max(0, base["minor"]             - r * 0.10)
  base["major"]             <- max(0, base["major"]             + r * 0.10)
  base["reject"]            <- max(0, base["reject"]            + r * 0.05)
  base <- base / sum(base)

  outcome <- sample(names(base), size = 1L, prob = base)

  resist_delta <- switch(outcome,
    accept_enthusiasm = 0.10,
    minor             = 0.05,
    major             = -0.05,
    reject            = -0.15
  )

  list(outcome = outcome, resist_delta = resist_delta)
}

# Spawn a shoot() run in an external terminal so the full ANSI multi-zone
# TUI can render even when the caller is in RStudio Console (which only
# supports \r-overwrite, not cursor positioning).
#
# Flow:
#   1. Serialize df + args + the calling session's texanshootR options
#      and the package path to a temp RDS.
#   2. Write a bootstrap R script that the spawned R will source: it
#      restores the options, loads the package (devtools::load_all when
#      we're in a dev tree, library() otherwise), calls shoot(), saves
#      the result to another RDS, and pauses for Enter so the user can
#      read the post-run output before the window closes.
#   3. Launch Windows Terminal (wt.exe) with R.exe --interactive running
#      the bootstrap. Block until the terminal exits.
#   4. Read the result RDS and return it to the calling session.

spawn_shoot_in_terminal <- function(df,
                                    formula = NULL,
                                    seed    = NULL,
                                    depth   = "default",
                                    ...) {
  if (.Platform$OS.type != "windows") {
    stop("`terminal = TRUE` currently only supports Windows. ",
         "On macOS/Linux, run R from a real terminal directly.",
         call. = FALSE)
  }

  args_rds   <- tempfile("tx_args_",   fileext = ".rds")
  result_rds <- tempfile("tx_result_", fileext = ".rds")
  bootstrap  <- tempfile("tx_boot_",   fileext = ".R")
  marker     <- tempfile("tx_done_",   fileext = ".flag")

  # Forward all texanshootR.* options so the spawned R sees the same
  # save_dir, consent, budget, etc. as the calling session.
  all_opts <- options()
  tx_opts <- all_opts[grepl("^texanshootR\\.", names(all_opts))]

  saveRDS(list(
    df         = df,
    formula    = formula,
    seed       = seed,
    depth      = depth,
    extra      = list(...),
    options    = tx_opts,
    load_info  = detect_pkg_load(),
    result_rds = result_rds,
    marker     = marker
  ), args_rds)

  writeLines(c(
    "# Auto-generated bootstrap for texanshootR terminal spawn.",
    sprintf('args <- readRDS("%s")', forward_slashes(args_rds)),
    "do.call(options, args$options)",
    "li <- args$load_info",
    'if (identical(li$mode, "load_all")) {',
    '  suppressMessages(devtools::load_all(li$path, quiet = TRUE))',
    "} else {",
    '  suppressMessages(library(texanshootR))',
    "}",
    "result <- tryCatch(",
    "  do.call(texanshootR::shoot,",
    "          c(list(df = args$df, formula = args$formula,",
    "                 seed = args$seed, depth = args$depth),",
    "            args$extra)),",
    "  error = function(e) { message('shoot() error: ', conditionMessage(e)); NULL }",
    ")",
    "if (!is.null(result)) saveRDS(result, args$result_rds)",
    'file.create(args$marker)',
    'cat("\\n[shoot() finished. Press Enter to close.]\\n")',
    'invisible(tryCatch(readLines(file("stdin"), n = 1L), error = function(e) NULL))'
  ), bootstrap)

  r_exe <- file.path(R.home("bin"), "R.exe")
  wt    <- Sys.which("wt.exe")

  bootstrap_cmd <- sprintf("source('%s')", forward_slashes(bootstrap))

  if (nzchar(wt)) {
    message("Opening shoot() in Windows Terminal...")
    system2("wt.exe",
            c("-d", shQuote(getwd()),
              shQuote(r_exe),
              "--interactive", "--no-save",
              "-e", shQuote(bootstrap_cmd)),
            wait = TRUE)
  } else {
    message("wt.exe not found on PATH; opening shoot() in a new ",
            "console window via cmd.exe.")
    system2("cmd.exe",
            c("/c", "start", "/wait", "cmd", "/k",
              shQuote(r_exe), "--interactive", "--no-save",
              "-e", shQuote(bootstrap_cmd)),
            wait = TRUE)
  }

  # `wt` returns as soon as it hands off to the new window — wait=TRUE
  # doesn't block on the spawned R process. Poll the marker file
  # instead. The bootstrap touches it after readLines() unblocks, i.e.
  # after the user presses Enter.
  if (!file.exists(marker)) {
    while (!file.exists(marker)) Sys.sleep(0.2)
  }

  if (file.exists(result_rds)) {
    invisible(readRDS(result_rds))
  } else {
    message("Terminal session exited without producing a tx_run.")
    invisible(NULL)
  }
}

# Detect whether texanshootR is currently loaded via devtools::load_all
# (dev tree) or via library() (installed). The bootstrap uses this to
# pick the right load path in the spawned R.
detect_pkg_load <- function() {
  ns <- tryCatch(getNamespace("texanshootR"), error = function(e) NULL)
  if (is.null(ns)) return(list(mode = "library", path = NULL))
  path <- tryCatch(getNamespaceInfo(ns, "path"), error = function(e) NULL)
  if (is.null(path) || !dir.exists(path)) {
    return(list(mode = "library", path = NULL))
  }
  is_dev <- file.exists(file.path(path, "DESCRIPTION")) &&
            dir.exists(file.path(path, "R")) &&
            !dir.exists(file.path(path, "Meta"))
  list(mode = if (is_dev) "load_all" else "library",
       path = if (is_dev) path else NULL)
}

# Path normalization for embedding inside R-source strings: backslashes
# would be parsed as escape sequences when the bootstrap is read back.
forward_slashes <- function(p) gsub("\\\\", "/", p, fixed = FALSE)

#' Reset persisted texanshootR state
#'
#' These functions remove pieces of the persistent save written under
#' [tools::R_user_dir()]. They prompt for confirmation in interactive
#' sessions; pass `force = TRUE` to suppress the prompt. Non-interactive
#' sessions require `force = TRUE`.
#'
#' @param force Logical. Skip the confirmation prompt.
#' @return `TRUE` invisibly on success, `FALSE` if cancelled.
#' @name reset
NULL

confirm <- function(prompt, force) {
  if (isTRUE(force)) return(TRUE)
  if (!interactive()) {
    stop("Refusing to reset state in a non-interactive session ",
         "without `force = TRUE`.", call. = FALSE)
  }
  ans <- tolower(trimws(readline(prompt)))
  isTRUE(ans %in% c("y", "yes"))
}

# Shared body for the per-artifact resets. Returns TRUE on success,
# FALSE on cancellation, TRUE invisibly when the save dir is missing.
reset_save_artifact <- function(prompt, paths, force) {
  d <- save_dir()
  if (is.null(d) || !dir.exists(d)) return(invisible(TRUE))
  if (!confirm(prompt, force)) return(invisible(FALSE))
  for (p in paths) unlink(file.path(d, p), recursive = TRUE)
  invisible(TRUE)
}

#' @rdname reset
#' @export
reset_career <- function(force = FALSE) {
  reset_save_artifact(
    "Reset career? This deletes all saved meta and runs. [y/N]: ",
    c("meta.json", "runs", "recent_messages.rds"),
    force
  )
}

#' @rdname reset
#' @export
reset_achievements <- function(force = FALSE) {
  reset_save_artifact("Reset achievements? [y/N]: ", "achievements.rds", force)
}

#' @rdname reset
#' @export
reset_wardrobe <- function(force = FALSE) {
  reset_save_artifact("Reset wardrobe? [y/N]: ", "wardrobe.rds", force)
}

#' @rdname reset
#' @export
reset_all <- function(force = FALSE) {
  d <- save_dir()
  if (is.null(d) || !dir.exists(d)) return(invisible(TRUE))
  if (!confirm("Reset everything (career + achievements + wardrobe + sightings)? [y/N]: ",
               force)) {
    return(invisible(FALSE))
  }
  unlink(d, recursive = TRUE)
  invisible(TRUE)
}

# Helpers shared across all output_*.R files.

# ---- Methods-paragraph rendering ----------------------------------
#
# The search attaches structured `restriction` and `outcome_construction`
# records to each fit's result row (see R/families.R spec_metadata).
# describe_spec() turns those records plus the formula into a single
# methods-paragraph sentence that every output generator and the print
# banner share. This is the single source of truth for "what did the
# shooter actually do to get this result" -- glyph/labels stay aligned
# across the abstract, manuscript, presentation, reviewer response,
# and DATA USED block by routing through here.

render_restriction_phrase <- function(restriction) {
  if (is.null(restriction)) return(NA_character_)
  switch(restriction$kind,
    complete_cases = "restricting to complete cases",
    outcome_iqr    = "winsorising extreme observations on the outcome",
    predictor_iqr  = sprintf(
      "restricting to observations within the typical range of %s",
      restriction$var %||% "the predictor"),
    cooks_d        = "excluding observations with disproportionate leverage",
    factor_level   = sprintf("restricting to the %s subgroup of %s",
                              restriction$level %||% "selected",
                              restriction$var   %||% "the grouping variable"),
    NA_character_
  )
}

render_outcome_phrase <- function(construction, outcome) {
  if (is.null(construction)) return(NA_character_)
  switch(construction$kind,
    residualise = sprintf("partialling out variation due to %s",
                           construction$covariate %||% "the covariate"),
    ratio       = sprintf("expressing the outcome as a rate per unit of %s",
                           construction$baseline %||% "the baseline"),
    z_within    = sprintf("z-scoring the outcome within each %s",
                           construction$factor %||% "subgroup"),
    NA_character_  # composite_index handled as a lead sentence in describe_spec()
  )
}

# Model-form phrase. Renders as the model-clause tail of the methods
# sentence, e.g. "we fit a robust M-estimator of y on x". polynomial /
# spline produce predictor-wrap phrases that compose into the same
# clause; robust / family_swap / random_effect rewrite the fitter and
# produce the full model-tail.
render_model_form_phrase <- function(model_form) {
  if (is.null(model_form)) return(NA_character_)
  switch(model_form$kind,
    polynomial = sprintf("allowing for a %s trend in %s",
                          degree_word(model_form$degree %||% 2L),
                          model_form$var %||% "the predictor"),
    spline     = sprintf("modelling %s flexibly via natural splines",
                          model_form$var %||% "the predictor"),
    robust     = "using a robust M-estimator",
    family_swap = render_family_swap(model_form),
    random_effect = sprintf("with a random intercept for %s",
                              model_form$group %||% "the grouping factor"),
    NA_character_
  )
}

degree_word <- function(d) {
  switch(as.character(as.integer(d)),
    "2" = "quadratic",
    "3" = "cubic",
    sprintf("degree-%d", as.integer(d)))
}

render_family_swap <- function(mf) {
  fam_label <- switch(mf$family %||% "",
    binomial = "logistic",
    poisson  = "Poisson",
    Gamma    = "Gamma",
    mf$family %||% "GLM")
  base <- sprintf("modelled with a %s GLM", fam_label)
  coerce <- mf$outcome_coerce
  if (is.null(coerce) || !nzchar(coerce)) return(base)
  tail <- switch(coerce,
    median_dichotomize = " after dichotomising the outcome at the median",
    round_clip         = " after rounding the outcome to non-negative integers",
    "")
  paste0(base, tail)
}

# Methods-paragraph sentence for the highlighted spec. Composes the
# spec's restriction + outcome-construction + model_form records into a
# single sentence:
#   "[We constructed ...] After X and Y, we fit FORMULA, M."
# The polynomial / spline wraps are stripped from FORMULA before
# rendering so the prose clause M carries the description -- a
# methods paragraph reads "we fit weight ~ age + sex, allowing for a
# quadratic trend in age", not "we fit weight ~ poly(age, 2) + sex".
describe_spec <- function(spec) {
  if (is.null(spec)) return("We fit y ~ x.")
  formula <- strip_model_form_wraps(spec$formula %||% "y ~ x",
                                     spec$model_form)

  prefix_clauses <- character()
  lead <- NULL

  r_phrase <- render_restriction_phrase(spec$restriction)
  if (!is.na(r_phrase)) prefix_clauses <- c(prefix_clauses, r_phrase)

  oc <- spec$outcome_construction
  if (!is.null(oc)) {
    if (identical(oc$kind, "composite_index")) {
      vars <- oc$vars %||% character()
      if (length(vars)) {
        lead <- sprintf("We constructed a composite outcome from %s.",
                        oxford_join(vars))
      }
    } else {
      o_phrase <- render_outcome_phrase(oc, formula)
      if (!is.na(o_phrase)) prefix_clauses <- c(prefix_clauses, o_phrase)
    }
  }

  main <- if (length(prefix_clauses)) {
    joined <- oxford_join(prefix_clauses)
    sprintf("After %s, we fit %s", joined, formula)
  } else {
    sprintf("We fit %s", formula)
  }

  mf_phrase <- render_model_form_phrase(spec$model_form)
  main <- if (!is.na(mf_phrase)) sprintf("%s, %s.", main, mf_phrase)
          else                    paste0(main, ".")

  if (is.null(lead)) main else paste(lead, main)
}

# Rewrite poly(var, k) and splines::ns(var, df = k) terms back to bare
# `var` for the methods-paragraph rendering. The formula was built by
# materialise_spec via string concatenation -- it was never deparsed
# user code -- so a structural string substitution is safe here.
strip_model_form_wraps <- function(f, model_form) {
  if (is.null(model_form) || is.null(model_form$var)) return(f)
  var <- model_form$var
  if (identical(model_form$kind, "polynomial")) {
    pat <- sprintf("poly\\(\\Q%s\\E,\\s*\\d+\\)", var)
    return(sub(pat, var, f))
  }
  if (identical(model_form$kind, "spline")) {
    pat <- sprintf("splines::ns\\(\\Q%s\\E,\\s*df\\s*=\\s*\\d+\\)", var)
    return(sub(pat, var, f))
  }
  f
}

oxford_join <- function(x) {
  x <- as.character(x)
  if (length(x) == 0L) return("")
  if (length(x) == 1L) return(x)
  if (length(x) == 2L) return(paste(x, collapse = " and "))
  paste0(paste(x[-length(x)], collapse = ", "), ", and ", x[length(x)])
}

capitalize_first <- function(s) {
  if (!nzchar(s)) return(s)
  paste0(toupper(substr(s, 1L, 1L)), substr(s, 2L, nchar(s)))
}

# Compact methods summary for the print banner FINDING line. Returns a
# semicolon-joined list of perturbation phrases, or "none" when no
# restriction / outcome construction was applied. Distinct from
# describe_spec(): no leading capital, no "we fit FORMULA" tail.
methods_summary <- function(spec) {
  if (is.null(spec)) return("none")
  parts <- character()
  r <- render_restriction_phrase(spec$restriction)
  if (!is.na(r)) parts <- c(parts, r)
  oc <- spec$outcome_construction
  if (!is.null(oc)) {
    if (identical(oc$kind, "composite_index")) {
      vars <- oc$vars %||% character()
      if (length(vars)) {
        parts <- c(parts,
                   sprintf("composite outcome from %s", oxford_join(vars)))
      }
    } else {
      o <- render_outcome_phrase(oc, "")
      if (!is.na(o)) parts <- c(parts, o)
    }
  }
  mf <- render_model_form_phrase(spec$model_form)
  if (!is.na(mf)) parts <- c(parts, mf)
  if (!length(parts)) return("none")
  paste(parts, collapse = "; ")
}

# ---- File-output helpers ------------------------------------------

resolve_output_dir <- function(output_dir = NULL) {
  d <- output_dir %||% getOption("texanshootR.output_dir") %||% tempdir()
  if (!dir.exists(d)) dir.create(d, recursive = TRUE, showWarnings = FALSE)
  d
}

require_pkg <- function(pkg, fn) {
  if (!requireNamespace(pkg, quietly = TRUE)) {
    stop(sprintf("`%s()` requires the '%s' package. Install it with install.packages('%s').",
                 fn, pkg, pkg), call. = FALSE)
  }
  invisible(TRUE)
}

# Filename versioning rotor - if a default-named file already exists
# we step through ("file.ext", "file_v2.ext", "file_v2_REAL.ext",
# "file_v2_REAL_FINAL.ext", "file_v2_REAL_FINAL_clean.ext").
SUFFIX_LADDER <- c("", "_v2", "_v2_REAL", "_v2_REAL_FINAL", "_v2_REAL_FINAL_clean")

versioned_filename <- function(dir, stem, ext, force = FALSE) {
  for (sfx in SUFFIX_LADDER) {
    f <- file.path(dir, paste0(stem, sfx, ".", ext))
    if (!file.exists(f) || isTRUE(force)) return(f)
  }
  # Final fallback: timestamp.
  file.path(dir, sprintf("%s_%s.%s", stem,
                         format(Sys.time(), "%Y%m%d%H%M%S"), ext))
}

status_open  <- function(file) {
  if (isTRUE(opt("texanshootR.quiet"))) return(invisible())
  cat("Saving ", basename(file), "...\n", sep = "")
}
status_close <- function() {
  if (isTRUE(opt("texanshootR.quiet"))) return(invisible())
  cat("Done.\n")
}

# Track that a run produced a given output. Updates the per-run record
# on disk and re-evaluates achievements so output-driven triggers (HARK,
# all-outputs, filename archaeologist, publication pipeline) can fire.
record_output <- function(run, output_name, file) {
  d <- save_dir()
  if (is.null(d)) return(invisible())
  fpath <- file.path(d, "runs", paste0(run$run_id, ".rds"))
  if (!file.exists(fpath)) return(invisible())
  rec <- readRDS(fpath)
  rec$outputs_generated       <- unique(c(rec$outputs_generated %||% character(), output_name))
  rec$outputs_generated_files <- unique(c(rec$outputs_generated_files %||% character(), basename(file)))

  # Carry over runtime-set flags from the in-memory run (e.g. `harked`
  # is set by manuscript() before this call). The on-disk record is
  # what evaluate_achievements() reads.
  for (k in c("harked", "peak_mascot",
              "stopped_early", "resolved_at_progress", "ultra_rare_seen")) {
    v <- run[[k]]
    if (!is.null(v)) rec[[k]] <- v
  }
  saveRDS(rec, fpath)

  meta <- read_meta()
  if (!is.null(meta)) {
    meta$hidden$output_complexity <- (meta$hidden$output_complexity %||% 0) +
      output_complexity_score(output_name)
    write_meta(meta)
    award_and_equip(rec, meta)
  }
  invisible()
}

output_complexity_score <- function(name) {
  switch(name,
    manuscript         = 3,
    presentation       = 2,
    reviewer_response  = 1,
    graphical_abstract = 1,
    funding            = 2,
    1
  )
}

# Map between the CHAIN_STAGES token and the exported generator. Kept
# here so generate_flagged_outputs() doesn't have to know the function
# names directly and so a future stage rename touches one table.
OUTPUT_GENERATORS <- list(
  abstract           = function(run) abstract(run),
  manuscript         = function(run) manuscript(run),
  presentation       = function(run) presentation(run),
  reviewer_response  = function(run) reviewer_response(run),
  graphical_abstract = function(run) graphical_abstract(run),
  funding            = function(run) funding(run)
)

# Auto-generate the chain prefix needed to satisfy the caller's output
# flags. Walks CHAIN_STAGES in order and calls each generator up to the
# highest enabled flag. A locked stage (xp_threshold not yet met) or an
# expired chain window stops the walk -- the generator throws a
# tx_chain_error which we catch and surface as a one-line note. A
# non-shippable run has no active chain, so the first call short-circuits.
generate_flagged_outputs <- function(run, flags) {
  if (!length(flags)) return(run)
  enabled <- vapply(flags, isTRUE, logical(1))
  if (!any(enabled)) return(run)
  if (!isTRUE(run$shippable)) return(run)
  highest <- max(stage_index(names(flags)[enabled]), na.rm = TRUE)
  for (i in seq_len(highest)) {
    stage <- CHAIN_STAGES[[i]]
    gen   <- OUTPUT_GENERATORS[[stage]]
    if (is.null(gen)) next
    res <- tryCatch(gen(run),
                    tx_chain_error = function(e) e,
                    error          = function(e) e)
    if (inherits(res, "tx_chain_error")) {
      say(sprintf("Skipping %s and later stages: %s.", stage, res$reason))
      break
    }
  }
  # Merge the on-disk record's outputs_generated fields back into the
  # in-memory run -- record_output() updates the rds but doesn't touch
  # the local `run` object, so without this re-read shoot() would
  # return a run with empty outputs_generated.
  d <- save_dir()
  if (!is.null(d)) {
    fpath <- file.path(d, "runs", paste0(run$run_id, ".rds"))
    if (file.exists(fpath)) {
      updated <- readRDS(fpath)
      for (k in c("outputs_generated", "outputs_generated_files")) {
        if (!is.null(updated[[k]])) run[[k]] <- updated[[k]]
      }
    }
  }
  run
}

#' Validate the on-disk message registry
#'
#' Lints every YAML file under `inst/messages/` (and an additional path
#' if supplied), enforcing required fields, vocabulary membership, id
#' uniqueness, and combo-chain integrity. Used by the test suite and
#' available to contributors.
#'
#' @param path Optional character path to a directory of YAML files.
#'   When `NULL`, validates the package's bundled messages.
#' @return A data.frame of the validated registry, returned invisibly.
#' @export
validate_messages <- function(path = NULL) {
  reg <- load_message_registry(path = path)
  invisible(reg)
}

# Load and validate. Cached on the package environment for repeat use.
load_message_registry <- function(path = NULL, force = FALSE) {
  if (!force && is.null(path) && !is.null(.tx$messages)) {
    return(.tx$messages)
  }

  dir <- path %||% system.file("messages", package = "texanshootR")
  if (!nzchar(dir) || !dir.exists(dir)) {
    # Empty registry is allowed during development.
    reg <- empty_message_registry()
    if (is.null(path)) .tx$messages <- reg
    return(reg)
  }

  files <- list.files(dir, pattern = "\\.ya?ml$", full.names = TRUE)
  rows  <- list()
  for (f in files) {
    entries <- yaml::read_yaml(f)
    if (is.null(entries)) next
    for (i in seq_along(entries)) {
      rows[[length(rows) + 1L]] <- normalise_message_entry(entries[[i]], f, i)
    }
  }
  if (length(rows) == 0L) {
    reg <- empty_message_registry()
  } else {
    reg <- rbind_messages(rows)
    validate_registry(reg)
  }
  if (is.null(path)) .tx$messages <- reg
  reg
}

empty_message_registry <- function() {
  data.frame(
    id                    = character(),
    text                  = character(),
    tags                  = I(list()),
    rarity                = character(),
    career_min            = character(),
    trigger_phase         = character(),
    model_family_affinity = I(list()),
    mascot_state_affinity = I(list()),
    combo_start           = character(),
    combo_next            = character(),
    requires              = I(list()),
    source_file           = character(),
    stringsAsFactors      = FALSE
  )
}

# Coerce one YAML entry into a normalised one-row record.
normalise_message_entry <- function(e, f, i) {
  required <- c("id", "text", "rarity", "trigger_phase")
  missing <- setdiff(required, names(e))
  if (length(missing)) {
    stop(sprintf(
      "Message in %s entry %d missing required fields: %s",
      basename(f), i, paste(missing, collapse = ", ")
    ), call. = FALSE)
  }
  list(
    id                    = as.character(e$id),
    text                  = as.character(e$text),
    tags                  = as.character(e$tags %||% character()),
    rarity                = as.character(e$rarity),
    career_min            = if (is.null(e$career_min) || is.na(e$career_min))
                              NA_character_ else as.character(e$career_min),
    trigger_phase         = as.character(e$trigger_phase),
    model_family_affinity = as.character(e$model_family_affinity %||% character()),
    mascot_state_affinity = as.character(e$mascot_state_affinity %||% character()),
    combo_start           = as.character(e$combo_chain$start %||% NA_character_),
    combo_next            = as.character(e$combo_chain[["next"]] %||% NA_character_),
    requires              = as.character(e$requires %||% character()),
    source_file           = basename(f)
  )
}

rbind_messages <- function(rows) {
  flat <- function(field) lapply(rows, `[[`, field)
  data.frame(
    id                    = vapply(rows, `[[`, "", "id"),
    text                  = vapply(rows, `[[`, "", "text"),
    tags                  = I(flat("tags")),
    rarity                = vapply(rows, `[[`, "", "rarity"),
    career_min            = vapply(rows, function(r) r$career_min %||% NA_character_, ""),
    trigger_phase         = vapply(rows, `[[`, "", "trigger_phase"),
    model_family_affinity = I(flat("model_family_affinity")),
    mascot_state_affinity = I(flat("mascot_state_affinity")),
    combo_start           = vapply(rows, `[[`, "", "combo_start"),
    combo_next            = vapply(rows, `[[`, "", "combo_next"),
    requires              = I(flat("requires")),
    source_file           = vapply(rows, `[[`, "", "source_file"),
    stringsAsFactors      = FALSE
  )
}

validate_registry <- function(reg) {
  # Unique ids.
  dups <- reg$id[duplicated(reg$id)]
  if (length(dups)) {
    stop("Duplicate message ids: ", paste(unique(dups), collapse = ", "), call. = FALSE)
  }

  # Vocabulary membership.
  bad_phase <- setdiff(reg$trigger_phase, vocab_phases)
  if (length(bad_phase)) {
    stop("Unknown trigger_phase value(s): ", paste(bad_phase, collapse = ", "),
         ". See `vocab_phases`.", call. = FALSE)
  }
  bad_rarity <- setdiff(reg$rarity, c("common", "uncommon", "rare", "legendary"))
  if (length(bad_rarity)) {
    stop("Unknown rarity value(s): ", paste(bad_rarity, collapse = ", "), call. = FALSE)
  }
  career <- reg$career_min[!is.na(reg$career_min)]
  bad_career <- setdiff(career, vocab_careers)
  if (length(bad_career)) {
    stop("Unknown career_min value(s): ", paste(bad_career, collapse = ", "), call. = FALSE)
  }
  all_tags <- unique(unlist(reg$tags))
  bad_tags <- setdiff(all_tags, vocab_tags)
  if (length(bad_tags)) {
    stop("Unknown tag(s): ", paste(bad_tags, collapse = ", "),
         ". See `vocab_tags`.", call. = FALSE)
  }
  all_states <- unique(unlist(reg$mascot_state_affinity))
  bad_states <- setdiff(all_states, vocab_mascot_states)
  if (length(bad_states)) {
    stop("Unknown mascot_state_affinity value(s): ",
         paste(bad_states, collapse = ", "), call. = FALSE)
  }

  # Phases that ARE the methodological mechanic must carry at least
  # one fallacy tag (per the content design contract). Phases that are
  # institutional ceremony (banner, promotion, reviewer, state
  # transition, daily) are exempt.
  # Phases that are institutional ceremony (banner, promotion, reviewer,
  # state transition, daily, chain transitions) are exempt from the
  # fallacy-tag requirement.
  fallacy_phases <- c("blip", "loading", "derived_escalation",
                       "ultra_rare", "event", "event_consequence")
  fallacy_tags <- vocab_tags[seq_len(25)]
  fallacy_rows <- reg$trigger_phase %in% fallacy_phases
  has_fallacy <- vapply(reg$tags, function(t) any(t %in% fallacy_tags), logical(1))
  bad <- fallacy_rows & !has_fallacy
  if (any(bad)) {
    stop("Messages without a fallacy tag: ",
         paste(reg$id[bad], collapse = ", "), call. = FALSE)
  }

  # Combo-chain integrity: combo_next must reference a known combo_start.
  starts <- unique(reg$combo_start[!is.na(reg$combo_start)])
  bad_combo <- reg$combo_next[!is.na(reg$combo_next) & !(reg$combo_next %in% starts)]
  if (length(bad_combo)) {
    stop("Orphan combo_chain.next reference(s): ",
         paste(unique(bad_combo), collapse = ", "), call. = FALSE)
  }

  # Loading-slot budget: messages routed through ui_loading() in the
  # dynamic single-line TUI share one slot with the mascot and bar.
  # Anything longer than DYN_LOADING_BUDGET gets ellipsised at the
  # 120-col target width. Phases that hit ui_loading directly --
  # "loading" and "state_transition" -- are linted here. Other phases
  # write to the blip stream in multi-zone ANSI mode and don't share
  # the slot, so they're exempt.
  loading_phases <- c("loading", "state_transition")
  loading_rows <- reg$trigger_phase %in% loading_phases
  over_budget <- loading_rows & nchar(reg$text) > DYN_LOADING_BUDGET
  if (any(over_budget)) {
    offenders <- sprintf("%s (%d)", reg$id[over_budget],
                          nchar(reg$text[over_budget]))
    stop("Message text exceeds DYN_LOADING_BUDGET = ",
         DYN_LOADING_BUDGET, " chars: ",
         paste(offenders, collapse = ", "), call. = FALSE)
  }

  invisible(TRUE)
}

`%||%` <- function(a, b) if (is.null(a) || (length(a) == 1 && is.na(a))) b else a

#' Inspect or modify the mascot wardrobe
#'
#' Without arguments, prints the equipped + unlocked cosmetics. With
#' `slot` and `id`, equips the given cosmetic in the given slot.
#'
#' @param slot Optional slot name: hat, poncho, badge, lanyard, cloak.
#' @param id   Optional cosmetic id.
#' @return The wardrobe state (invisible when modifying, visible when
#'   listing).
#' @export
wardrobe <- function(slot = NULL, id = NULL) {
  reg <- load_cosmetic_registry()
  state <- read_wardrobe_state()
  if (is.null(slot)) {
    print_wardrobe(state, reg)
    return(invisible(state))
  }
  slot <- match.arg(slot, c("hat", "poncho", "badge", "lanyard", "cloak"))
  if (is.null(id)) {
    state$equipped[[slot]] <- NA
    write_wardrobe_state(state)
    return(invisible(state))
  }
  if (!(id %in% reg$id[reg$slot == slot])) {
    stop("Unknown cosmetic '", id, "' for slot '", slot, "'.", call. = FALSE)
  }
  if (!(id %in% state$unlocked)) {
    stop("Cosmetic '", id, "' is locked.", call. = FALSE)
  }
  state$equipped[[slot]] <- id
  write_wardrobe_state(state)
  invisible(state)
}

print_wardrobe <- function(state, reg) {
  rule <- strrep("-", 48)
  cat(style_header(rule), "\n", sep = "")
  cat(style_header("Wardrobe"), "\n", sep = "")
  cat(style_header(rule), "\n", sep = "")
  for (sl in c("hat", "poncho", "badge", "lanyard", "cloak")) {
    eq <- state$equipped[[sl]]
    cat(sprintf("%-9s %s\n", paste0(sl, ":"),
                if (is.na(eq)) "-" else eq))
  }
  cat("\nUnlocked: ",
      if (length(state$unlocked) == 0L) "(none)"
      else paste(state$unlocked, collapse = ", "),
      "\n", sep = "")
}

# Auto-equip a newly-unlocked cosmetic in its slot, replacing whatever
# was equipped there before. Called from the achievement-award path.
auto_equip <- function(cosmetic_id) {
  reg <- load_cosmetic_registry()
  spec <- reg[reg$id == cosmetic_id, ]
  if (nrow(spec) != 1L) return(invisible(NULL))
  state <- read_wardrobe_state()
  if (!(cosmetic_id %in% state$unlocked)) {
    state$unlocked <- c(state$unlocked, cosmetic_id)
  }
  state$equipped[[spec$slot]] <- cosmetic_id
  write_wardrobe_state(state)
  invisible(state)
}

# Lazy registry loader. Reads inst/cosmetics/*.yaml.
load_cosmetic_registry <- function(force = FALSE) {
  load_registry(
    inst_dir   = "cosmetics",
    cache_key  = "cosmetics",
    entry_to_row = function(e) list(
      id         = as.character(e$id),
      slot       = as.character(e$slot),
      overlay    = as.list(strsplit(e$ascii_overlay %||% "", "\n", fixed = TRUE)[[1]]),
      unlock_via = as.character(e$unlock_via %||% NA_character_)
    ),
    rows_to_frame = function(rows) {
      out <- data.frame(
        id         = vapply(rows, `[[`, "", "id"),
        slot       = vapply(rows, `[[`, "", "slot"),
        unlock_via = vapply(rows, `[[`, NA_character_, "unlock_via"),
        stringsAsFactors = FALSE
      )
      out$overlay <- I(lapply(rows, `[[`, "overlay"))
      out
    },
    empty = empty_cosmetic_registry,
    force = force
  )
}

empty_cosmetic_registry <- function() {
  new_list_col_frame(
    char_cols = c("id", "slot", "unlock_via"),
    list_col  = "overlay"
  )
}

test_that("career() prints without erroring on a fresh save", {
  texanshootR:::ensure_save_dir()
  c1 <- career()
  expect_s3_class(c1, "tx_career")
  expect_true(nzchar(format(c1)))
  expect_output(print(c1), "Career:")
})

test_that("achievements() returns the registry with hidden labels", {
  a <- achievements()
  expect_s3_class(a, "data.frame")
  expect_true(nrow(a) >= 20L)
  expect_true(any(a$name == "???"))
})

test_that("wardrobe() list view does not error", {
  expect_invisible(wardrobe())
})

#' Generate a graphical abstract PNG
#'
#' A single-figure summary of the highlighted specification, composed
#' as a multi-panel layout with conceptual arrows. Renders via ggplot2
#' to PNG.
#'
#' Chain stage: \strong{graphical_abstract} (length 5). See [progress()].
#'
#' @inheritParams manuscript
#' @return A length-one character vector giving the path to the written
#'   PNG file, returned invisibly. Called for its side effect of
#'   generating the graphical abstract in `output_dir` and advancing the
#'   publication chain.
#' @export
graphical_abstract <- function(run, output_dir = NULL, file = NULL,
                               force = FALSE) {
  require_chain_stage("graphical_abstract", run)
  require_pkg("ggplot2", "graphical_abstract")
  d <- resolve_output_dir(output_dir)
  stem <- file %||% "figure_final"
  out  <- versioned_filename(d, stem, "png", force)

  hs <- run$highlighted_spec %||% list()

  df_plot <- data.frame(
    x = seq_len(20),
    y = cumsum(stats::rnorm(20))
  )

  p <- ggplot2::ggplot(df_plot, ggplot2::aes(x = x, y = y)) +
    ggplot2::geom_line(linewidth = 1) +
    ggplot2::geom_point(size = 2) +
    ggplot2::labs(
      title    = "Conceptual relationship",
      subtitle = describe_spec(hs),
      caption  = sprintf("R^2 = %.3f, p = %.4f",
                          hs$r_squared %||% 0,
                          hs$p_value %||% 1),
      x = "Latent gradient",
      y = "Response"
    ) +
    ggplot2::theme_minimal(base_size = 11) +
    ggplot2::theme(plot.title = ggplot2::element_text(face = "bold"))

  status_open(out)
  ggplot2::ggsave(filename = out, plot = p, width = 6, height = 4,
                  dpi = 150)
  status_close()
  record_output(run, "graphical_abstract", out)
  advance_chain_after_stage("graphical_abstract")
  invisible(out)
}

# Stateful, opportunistic spec generator.
#
# The shooter does not enumerate a clean cartesian product. He starts
# from a small seed pool (one spec per predictor subset, identity
# transforms, no perturbations) and then drifts toward whatever subset
# is producing the most encouraging p-values. Subsets that go cold
# get abandoned. Near-misses (best_p in (0.05, 0.20]) get re-perturbed
# along the other axes. A wildcard is injected periodically to keep
# him moving when nothing is pulsing.
#
# This is desperation, not exploration. Each fork in the search is the
# shooter trying one more thing because the last thing was almost.
#
# Mechanic-to-fallacy contract:
#   subsets       = predictor dredging / multiple comparisons
#   transforms    = researcher degrees of freedom
#   interactions  = p-hacking
#   restrictions  = principled-sounding sample exclusion
#                   (complete cases, IQR fence, Cook's D, factor restriction)
#   outcome_con   = outcome engineering
#                   (composites, residualisation, ratios, within-group z)
#   model_form    = model-complexity escalation
#                   (polynomial / spline wraps, robust M-estimator,
#                   GLM family swap, random-intercept lift)

NEAR_MISS_RANGE       <- c(0.05, 0.20)
DEAD_AFTER_TRIES      <- 5L
DEAD_P_THRESHOLD      <- 0.5
WILDCARD_EVERY        <- 10L
PERTURBATIONS_PER_HIT <- 3L
PERTURB_KINDS         <- c("transform", "interaction", "restriction",
                            "outcome", "model")

new_spec <- function(subset,
                     transforms           = NULL,
                     interactions         = character(),
                     restriction          = NULL,
                     outcome_construction = NULL,
                     model_form           = NULL,
                     source               = "seed",
                     family               = list(fitter = "lm")) {
  if (is.null(transforms)) {
    transforms <- stats::setNames(rep("identity", length(subset)), subset)
  }
  list(subset               = subset,
       transforms           = transforms,
       interactions         = interactions,
       restriction          = restriction,
       outcome_construction = outcome_construction,
       model_form           = model_form,
       source               = source,
       family               = family)
}

subset_key <- function(spec) paste(sort(spec$subset), collapse = ",")

# Initial pool: one identity-only spec per non-empty subset of size 1..k,
# in random order so the shooter doesn't grind methodically.
seed_specs <- function(df, outcome, predictors, max_subset_size = 4L) {
  predictors <- intersect(predictors, names(df))
  predictors <- setdiff(predictors, outcome)
  if (length(predictors) == 0L) return(list())
  ks <- seq_len(min(max_subset_size, length(predictors)))
  specs <- list()
  for (k in ks) {
    if (length(predictors) >= k) {
      combs <- utils::combn(predictors, k, simplify = FALSE)
      for (s in combs) {
        specs[[length(specs) + 1L]] <- new_spec(s, source = "seed")
      }
    }
  }
  specs[sample.int(length(specs))]
}

# Perturb a single dimension of an existing spec. Used both for
# near-miss follow-ups and for assembling wildcards.
perturb_spec <- function(spec, kind, df, outcome) {
  s <- spec
  s$source <- paste0("perturb_", kind)
  if (kind == "transform") {
    nums <- intersect(s$subset, names(df))
    nums <- nums[vapply(nums, function(p) is.numeric(df[[p]]), logical(1))]
    if (length(nums)) {
      var <- if (length(nums) == 1L) nums else sample(nums, 1L)
      s$transforms[[var]] <- sample(c("log", "sqrt", "scale"), 1L)
    }
  } else if (kind == "interaction" && length(s$subset) >= 2L) {
    pair <- sample(s$subset, 2L)
    s$interactions <- unique(c(s$interactions,
                               paste(sort(pair), collapse = ":")))
  } else if (kind == "restriction") {
    pick <- sample_restriction(df, outcome, s)
    if (!is.null(pick)) s$restriction <- pick
  } else if (kind == "outcome") {
    pick <- sample_outcome_construction(df, outcome, s)
    if (!is.null(pick)) s$outcome_construction <- pick
  } else if (kind == "model") {
    pick <- sample_model_form(df, outcome, s)
    if (!is.null(pick)) s$model_form <- pick
  }
  s
}

# Random subset + 1-3 random perturbations. Injected when the search
# stalls or periodically to keep things flailing.
wildcard_spec <- function(df, outcome, predictors, max_subset_size = 4L) {
  predictors <- intersect(predictors, names(df))
  predictors <- setdiff(predictors, outcome)
  if (length(predictors) == 0L) return(NULL)
  k_max <- min(max_subset_size, length(predictors))
  k <- if (k_max == 1L) 1L else sample.int(k_max, 1L)
  subset <- if (length(predictors) == k) predictors
            else sample(predictors, k)
  s <- new_spec(subset, source = "wildcard")
  kinds <- sample(PERTURB_KINDS, size = sample.int(3L, 1L))
  for (kk in kinds) s <- perturb_spec(s, kk, df, outcome)
  s$source <- "wildcard"
  s
}

new_search_state <- function(seeds) {
  list(
    seed_queue     = seeds,
    perturb_queue  = list(),
    dead_subsets   = character(),
    subset_results = list()
  )
}

# Update state with the just-completed fit: append p-value to the
# subset's history; if the subset is in the near-miss band, enqueue a
# few perturbations of THIS spec; if the subset has gone stone cold,
# abandon it.
#
# `bias_perturb` (NA_character_ by default) is the kind to over-sample
# when the player has injected a perturbation bias (+subgroup, +outliers).
# When set and valid, ~2/3 of the enqueued perturbations are that kind;
# the rest stay random so the search doesn't collapse onto one axis.
record_result <- function(state, spec, result, df, outcome,
                          bias_perturb = NA_character_) {
  if (is.null(result)) return(state)
  key <- subset_key(spec)
  ps  <- c(state$subset_results[[key]] %||% numeric(),
           result$p_value %||% NA_real_)
  state$subset_results[[key]] <- ps

  best_p <- suppressWarnings(min(ps, na.rm = TRUE))
  if (!is.finite(best_p)) best_p <- NA_real_

  if (is.finite(best_p) &&
      best_p > NEAR_MISS_RANGE[1] && best_p <= NEAR_MISS_RANGE[2]) {
    kinds <- biased_perturb_kinds(bias_perturb, PERTURBATIONS_PER_HIT)
    for (kk in kinds) {
      state$perturb_queue[[length(state$perturb_queue) + 1L]] <-
        perturb_spec(spec, kk, df, outcome)
    }
  }

  if (length(ps) >= DEAD_AFTER_TRIES &&
      all(is.na(ps) | ps > DEAD_P_THRESHOLD)) {
    state$dead_subsets <- unique(c(state$dead_subsets, key))
  }
  state
}

# Sample N perturbation kinds, optionally over-sampling a biased tag.
# Without a bias the result is a plain uniform draw -- matches the
# previous behaviour exactly. With a bias, 2/3 of slots are forced to
# the biased kind and 1/3 fall back to uniform random.
biased_perturb_kinds <- function(bias_tag, n) {
  if (is.null(bias_tag) || is.na(bias_tag) || !nzchar(bias_tag) ||
      !(bias_tag %in% PERTURB_KINDS)) {
    return(sample(PERTURB_KINDS, size = n))
  }
  rolls <- stats::runif(n)
  ifelse(rolls < 2/3, bias_tag,
         sample(PERTURB_KINDS, size = n, replace = TRUE))
}

# Decide what to fit next. Priority: scheduled wildcard tick > pending
# perturbation > next live seed. Falls back to a wildcard when seeds
# are exhausted, so the loop can run to budget instead of stopping
# politely.
next_spec <- function(state, df, outcome, predictors,
                      iteration, max_subset_size = 4L) {
  if (iteration > 0L && iteration %% WILDCARD_EVERY == 0L) {
    return(list(state = state,
                spec  = wildcard_spec(df, outcome, predictors,
                                      max_subset_size)))
  }
  if (length(state$perturb_queue)) {
    spec <- state$perturb_queue[[1L]]
    state$perturb_queue <- state$perturb_queue[-1L]
    return(list(state = state, spec = spec))
  }
  while (length(state$seed_queue)) {
    spec <- state$seed_queue[[1L]]
    state$seed_queue <- state$seed_queue[-1L]
    if (!(subset_key(spec) %in% state$dead_subsets)) {
      return(list(state = state, spec = spec))
    }
  }
  list(state = state,
       spec  = wildcard_spec(df, outcome, predictors, max_subset_size))
}

# Apply transformations + restriction + outcome construction to a data
# frame and return the modified copy plus the formula string for the
# spec.
materialise_spec <- function(df, outcome, spec) {
  d <- df

  for (var in names(spec$transforms)) {
    t <- spec$transforms[[var]]
    if (t == "identity") next
    x <- d[[var]]
    if (!is.numeric(x)) next
    d[[var]] <- switch(t,
      log   = ifelse(x > 0, log(x), NA_real_),
      sqrt  = ifelse(x >= 0, sqrt(x), NA_real_),
      scale = as.numeric(scale(x)),
      x
    )
  }

  d <- apply_restriction(d, outcome, spec$restriction, spec)
  d <- apply_outcome_construction(d, outcome, spec$outcome_construction)

  # Predictor wraps: polynomial / spline rewrite a single bare term in
  # the RHS from `x` to `poly(x, k)` / `splines::ns(x, df = k)`. Both
  # are formula-level only -- the data column is untouched, so the
  # existing model.frame / model.matrix path materialises the basis at
  # fit time. Other model_form sub-kinds (robust, family_swap,
  # random_effect) are fitter-level and don't touch the RHS here.
  predictor_terms <- spec$subset
  mf <- spec$model_form
  if (!is.null(mf) && mf$kind %in% c("polynomial", "spline") &&
      !is.null(mf$var) && mf$var %in% predictor_terms) {
    idx <- which(predictor_terms == mf$var)
    predictor_terms[idx] <- if (mf$kind == "polynomial") {
      sprintf("poly(%s, %d)", mf$var, mf$degree %||% 2L)
    } else {
      sprintf("splines::ns(%s, df = %d)", mf$var, mf$df %||% 3L)
    }
  }

  rhs <- if (length(predictor_terms) == 0L) "1"
         else paste(predictor_terms, collapse = " + ")
  if (any(nzchar(spec$interactions))) {
    its <- spec$interactions[nzchar(spec$interactions)]
    rhs <- paste(rhs, paste(its, collapse = " + "), sep = " + ")
  }
  formula_str <- paste(outcome, "~", rhs)

  list(df = d, formula = formula_str)
}

# ---- restriction sampling + application ----------------------------

# Build the candidate list of restrictions whose triggers fire on this
# df + spec, then pick one uniformly. Returns NULL when no candidate
# qualifies (e.g. df has no factor columns and the outcome is character).
sample_restriction <- function(df, outcome, spec) {
  candidates <- list(list(kind = "complete_cases"))

  n <- nrow(df)
  y <- if (outcome %in% names(df)) df[[outcome]] else NULL

  if (!is.null(y) && is.numeric(y) && n >= 30L) {
    candidates[[length(candidates) + 1L]] <- list(kind = "outcome_iqr")
    candidates[[length(candidates) + 1L]] <- list(kind = "cooks_d")
  }

  nums_in_subset <- intersect(spec$subset, names(df))
  nums_in_subset <- nums_in_subset[vapply(nums_in_subset,
                                          function(p) is.numeric(df[[p]]),
                                          logical(1))]
  if (length(nums_in_subset)) {
    var <- if (length(nums_in_subset) == 1L) nums_in_subset
           else sample(nums_in_subset, 1L)
    candidates[[length(candidates) + 1L]] <- list(kind = "predictor_iqr",
                                                  var  = var)
  }

  factors <- categorical_columns_outside_subset(df, outcome, spec$subset)
  for (fac in factors) {
    levels_pool <- viable_factor_levels(df[[fac]], min_per_level = 10L)
    if (length(levels_pool)) {
      lvl <- if (length(levels_pool) == 1L) levels_pool
             else sample(levels_pool, 1L)
      candidates[[length(candidates) + 1L]] <- list(kind  = "factor_level",
                                                    var   = fac,
                                                    level = lvl)
      break  # one factor-level candidate per perturbation
    }
  }

  candidates[[sample.int(length(candidates), 1L)]]
}

apply_restriction <- function(d, outcome, restriction, spec) {
  if (is.null(restriction)) return(d)
  kind <- restriction$kind

  if (kind == "complete_cases") {
    needed <- intersect(unique(c(outcome, spec$subset)), names(d))
    if (!length(needed)) return(d)
    keep <- stats::complete.cases(d[, needed, drop = FALSE])
    return(d[keep, , drop = FALSE])
  }

  if (kind == "outcome_iqr") {
    if (!(outcome %in% names(d))) return(d)
    y <- d[[outcome]]
    if (!is.numeric(y)) return(d)
    return(d[iqr_keep(y), , drop = FALSE])
  }

  if (kind == "predictor_iqr") {
    v <- restriction$var
    if (!(v %in% names(d)) || !is.numeric(d[[v]])) return(d)
    return(d[iqr_keep(d[[v]]), , drop = FALSE])
  }

  if (kind == "cooks_d") {
    if (!(outcome %in% names(d))) return(d)
    if (nrow(d) < 30L) return(d)
    needed <- intersect(c(outcome, spec$subset), names(d))
    d_clean <- d[stats::complete.cases(d[, needed, drop = FALSE]), ,
                 drop = FALSE]
    if (nrow(d_clean) < 30L) return(d)
    rhs <- if (length(spec$subset)) paste(spec$subset, collapse = " + ")
           else "1"
    f <- tryCatch(stats::as.formula(paste(outcome, "~", rhs)),
                  error = function(e) NULL)
    if (is.null(f)) return(d)
    fit <- tryCatch(stats::lm(f, data = d_clean), error = function(e) NULL)
    if (is.null(fit)) return(d)
    cd <- tryCatch(stats::cooks.distance(fit), error = function(e) NULL)
    if (is.null(cd) || !any(is.finite(cd))) return(d)
    drop_n <- max(1L, as.integer(0.05 * length(cd)))
    drop_rows <- order(cd, decreasing = TRUE)[seq_len(drop_n)]
    keep_rows <- setdiff(seq_len(nrow(d_clean)), drop_rows)
    return(d_clean[keep_rows, , drop = FALSE])
  }

  if (kind == "factor_level") {
    v <- restriction$var
    lvl <- restriction$level
    if (!(v %in% names(d))) return(d)
    keep <- !is.na(d[[v]]) & as.character(d[[v]]) == lvl
    return(d[keep, , drop = FALSE])
  }

  d
}

iqr_keep <- function(x) {
  qs <- stats::quantile(x, c(0.25, 0.75), na.rm = TRUE)
  iqr <- qs[2] - qs[1]
  lo <- qs[1] - 1.5 * iqr
  hi <- qs[2] + 1.5 * iqr
  !is.na(x) & x >= lo & x <= hi
}

# ---- outcome-construction sampling + application -------------------

# Build the candidate list of outcome-engineering moves whose triggers
# fire on this df + spec, then pick one uniformly. Returns NULL when
# nothing qualifies (e.g. df has no other numeric columns to compose).
sample_outcome_construction <- function(df, outcome, spec) {
  candidates <- list()
  n <- nrow(df)
  y <- if (outcome %in% names(df)) df[[outcome]] else NULL

  numeric_cols <- names(df)[vapply(names(df),
                                    function(nm) is.numeric(df[[nm]]),
                                    logical(1))]
  outside <- setdiff(numeric_cols, c(outcome, spec$subset))

  if (length(outside) >= 2L) {
    k_max <- min(4L, length(outside))
    k <- if (k_max == 2L) 2L else sample(2L:k_max, 1L)
    picked <- if (length(outside) == k) outside else sample(outside, k)
    candidates[[length(candidates) + 1L]] <- list(kind = "composite_index",
                                                  vars = picked)
  }

  if (length(outside) >= 1L) {
    cov <- if (length(outside) == 1L) outside else sample(outside, 1L)
    candidates[[length(candidates) + 1L]] <- list(kind      = "residualise",
                                                  covariate = cov)
  }

  positive_outside <- outside[vapply(outside, function(v) {
    x <- df[[v]]
    finite <- is.finite(x)
    any(finite) && all(x[finite] > 0)
  }, logical(1))]
  if (length(positive_outside) && !is.null(y) && is.numeric(y)) {
    base <- if (length(positive_outside) == 1L) positive_outside
            else sample(positive_outside, 1L)
    candidates[[length(candidates) + 1L]] <- list(kind     = "ratio",
                                                  baseline = base)
  }

  factors <- categorical_columns_outside_subset(df, outcome, spec$subset)
  if (length(factors) && !is.null(y) && is.numeric(y)) {
    fac <- if (length(factors) == 1L) factors else sample(factors, 1L)
    candidates[[length(candidates) + 1L]] <- list(kind   = "z_within",
                                                  factor = fac)
  }

  if (!length(candidates)) return(NULL)
  candidates[[sample.int(length(candidates), 1L)]]
}

apply_outcome_construction <- function(d, outcome, construction) {
  if (is.null(construction)) return(d)
  kind <- construction$kind

  if (kind == "composite_index") {
    vars <- intersect(construction$vars, names(d))
    if (length(vars) < 2L) return(d)
    M <- as.matrix(d[, vars, drop = FALSE])
    M_scaled <- scale(M)
    new_y <- rowMeans(M_scaled, na.rm = TRUE)
    new_y[is.nan(new_y)] <- NA_real_
    d[[outcome]] <- new_y
    return(d)
  }

  if (kind == "residualise") {
    cov <- construction$covariate
    if (!(cov %in% names(d)) || !(outcome %in% names(d))) return(d)
    y <- d[[outcome]]
    z <- d[[cov]]
    if (!is.numeric(y) || !is.numeric(z)) return(d)
    ok <- is.finite(y) & is.finite(z)
    if (sum(ok) < 5L) return(d)
    fit <- tryCatch(stats::lm(y[ok] ~ z[ok]), error = function(e) NULL)
    if (is.null(fit)) return(d)
    resids <- rep(NA_real_, length(y))
    resids[ok] <- stats::residuals(fit)
    d[[outcome]] <- resids
    return(d)
  }

  if (kind == "ratio") {
    base <- construction$baseline
    if (!(base %in% names(d)) || !is.numeric(d[[base]])) return(d)
    if (!(outcome %in% names(d)) || !is.numeric(d[[outcome]])) return(d)
    b <- d[[base]]
    y <- d[[outcome]]
    ok <- is.finite(b) & b > 0
    y[!ok] <- NA_real_
    y[ok]  <- y[ok] / b[ok]
    d[[outcome]] <- y
    return(d)
  }

  if (kind == "z_within") {
    fac <- construction$factor
    if (!(fac %in% names(d)) || !(outcome %in% names(d))) return(d)
    y <- d[[outcome]]
    if (!is.numeric(y)) return(d)
    f <- as.character(d[[fac]])
    new_y <- y
    for (lvl in unique(f[!is.na(f)])) {
      idx <- which(f == lvl)
      if (length(idx) < 3L) next
      mu <- mean(y[idx], na.rm = TRUE)
      sdv <- stats::sd(y[idx], na.rm = TRUE)
      if (!is.finite(sdv) || sdv == 0) next
      new_y[idx] <- (y[idx] - mu) / sdv
    }
    d[[outcome]] <- new_y
    return(d)
  }

  d
}

# ---- shared helpers -----------------------------------------------

categorical_columns_outside_subset <- function(df, outcome, subset) {
  cols <- setdiff(names(df), c(outcome, subset))
  cols[vapply(cols, function(cn) {
    x <- df[[cn]]
    is.factor(x) ||
      (is.character(x) && length(unique(x[!is.na(x)])) >= 2L)
  }, logical(1))]
}

viable_factor_levels <- function(x, min_per_level = 10L) {
  if (is.character(x)) x <- factor(x)
  tab <- table(x)
  names(tab)[tab >= min_per_level]
}

# Compact digest for one spec restriction/outcome_construction record.
# Used by digest_grid()/summarise_trace() in shoot.R to detect distinct
# trace entries without committing to a specific field order.
restriction_digest <- function(r) {
  if (is.null(r)) return("none")
  paste(c(r$kind, r$var %||% "", r$level %||% ""), collapse = ":")
}

outcome_construction_digest <- function(o) {
  if (is.null(o)) return("none")
  paste(c(o$kind,
          paste(o$vars %||% character(), collapse = ","),
          o$covariate %||% "", o$baseline %||% "", o$factor %||% ""),
        collapse = ":")
}

# ---- model-form sampling + application -----------------------------
#
# Five sub-kinds. polynomial / spline are formula-level wraps applied
# inside materialise_spec(); the remaining three switch spec$family
# at fit time via apply_model_form(). Each candidate guards its own
# trigger so the search never enqueues a move that the data can't
# support (e.g. random_effect without a viable grouping factor).
#
#   polynomial(var, degree)   -> poly(var, degree) in RHS
#   spline(var, df)           -> splines::ns(var, df) in RHS
#   robust                    -> family$fitter = "rlm"
#   family_swap(family, link, outcome_coerce)
#                             -> family$fitter = "glm" with chosen family
#   random_effect(group)      -> family$fitter = "glmm" honoring group

sample_model_form <- function(df, outcome, spec) {
  candidates <- list()
  n <- nrow(df)
  y <- if (outcome %in% names(df)) df[[outcome]] else NULL

  nums_in_subset <- intersect(spec$subset, names(df))
  nums_in_subset <- nums_in_subset[vapply(nums_in_subset,
                                          function(p) is.numeric(df[[p]]) &&
                                            length(unique(df[[p]][is.finite(df[[p]])])) >= 4L,
                                          logical(1))]

  if (length(nums_in_subset)) {
    var <- if (length(nums_in_subset) == 1L) nums_in_subset
           else sample(nums_in_subset, 1L)
    degree <- sample(c(2L, 3L), 1L)
    candidates[[length(candidates) + 1L]] <- list(kind   = "polynomial",
                                                  var    = var,
                                                  degree = degree)
  }

  nums_for_spline <- nums_in_subset[vapply(nums_in_subset, function(p) {
    length(unique(df[[p]][is.finite(df[[p]])])) >= 10L
  }, logical(1))]
  if (length(nums_for_spline) && n >= 30L) {
    var <- if (length(nums_for_spline) == 1L) nums_for_spline
           else sample(nums_for_spline, 1L)
    candidates[[length(candidates) + 1L]] <- list(kind = "spline",
                                                  var  = var,
                                                  df   = 3L)
  }

  if (!is.null(y) && is.numeric(y) && n >= 20L) {
    candidates[[length(candidates) + 1L]] <- list(kind = "robust")
  }

  if (!is.null(y) && is.numeric(y) && n >= 30L) {
    swap_pool <- list(
      list(kind = "family_swap", family = "binomial", link = "logit",
           outcome_coerce = "median_dichotomize"),
      list(kind = "family_swap", family = "poisson",  link = "log",
           outcome_coerce = "round_clip")
    )
    finite_y <- y[is.finite(y)]
    if (length(finite_y) && all(finite_y > 0)) {
      swap_pool[[length(swap_pool) + 1L]] <- list(kind = "family_swap",
                                                  family = "Gamma",
                                                  link   = "log",
                                                  outcome_coerce = NULL)
    }
    candidates[[length(candidates) + 1L]] <- swap_pool[[
      sample.int(length(swap_pool), 1L)]]
  }

  factors <- categorical_columns_outside_subset(df, outcome, spec$subset)
  factors <- factors[vapply(factors, function(v) {
    x <- df[[v]]
    levs <- unique(x[!is.na(x)])
    length(levs) >= 3L && length(levs) <= max(3L, nrow(df) %/% 3L)
  }, logical(1))]
  if (length(factors) && !is.null(y) && is.numeric(y) && n >= 30L) {
    g <- if (length(factors) == 1L) factors else sample(factors, 1L)
    candidates[[length(candidates) + 1L]] <- list(kind  = "random_effect",
                                                  group = g)
  }

  if (!length(candidates)) return(NULL)
  candidates[[sample.int(length(candidates), 1L)]]
}

# Project a sampled model_form onto spec$family for fitter-switching
# sub-kinds (robust / family_swap / random_effect). polynomial and
# spline are RHS-only and leave spec$family alone -- the existing
# family selector still gets to pick lm vs gam vs whatever else.
apply_model_form <- function(spec, model_form = spec$model_form) {
  if (is.null(model_form)) return(spec)
  switch(model_form$kind,
    polynomial = spec,
    spline     = spec,
    robust     = {
      spec$family <- list(fitter = "rlm")
      spec
    },
    family_swap = {
      spec$family <- list(fitter = "glm",
                          family = model_form$family,
                          link   = model_form$link,
                          outcome_coerce = model_form$outcome_coerce)
      spec
    },
    random_effect = {
      spec$family <- list(fitter = "glmm",
                          family = "gaussian",
                          link   = "identity",
                          group  = model_form$group)
      spec
    },
    spec
  )
}

model_form_digest <- function(m) {
  if (is.null(m)) return("none")
  paste(c(m$kind,
          m$var %||% "",
          as.character(m$degree %||% m$df %||% ""),
          m$family %||% "", m$link %||% "",
          m$outcome_coerce %||% "",
          m$group %||% ""),
        collapse = ":")
}

# Phase 2: model_form perturbation axis. Covers:
#   - sample_model_form() produces records with trigger-aware kinds
#   - apply_model_form() projects fitter-switching kinds onto spec$family
#   - materialise_spec() wraps polynomial / spline predictors in the RHS
#   - fit_rlm() returns a well-formed result row
#   - describe_spec() / methods_summary() render each sub-kind

make_phase2_df <- function() {
  set.seed(11)
  n <- 150
  data.frame(
    y     = rnorm(n, 0, 1),
    x1    = rnorm(n, 0, 1),
    x2    = rnorm(n, 0, 1),
    pos   = abs(rnorm(n, 5, 1)) + 0.1,
    grp   = factor(sample(c("A","B","C"), n, TRUE)),
    other = factor(sample(c("p","q","r"), n, TRUE))
  )
}

test_that("sample_model_form returns NULL on a df with no viable column", {
  d <- data.frame(y = c(1, 2, 3), z = c("a", "a", "a"),
                  stringsAsFactors = FALSE)
  spec <- texanshootR:::new_spec("z")
  expect_null(texanshootR:::sample_model_form(d, "y", spec))
})

test_that("sample_model_form yields one of the supported kinds", {
  d <- make_phase2_df()
  spec <- texanshootR:::new_spec(c("x1", "x2"))
  picks <- replicate(40,
                     texanshootR:::sample_model_form(d, "y", spec),
                     simplify = FALSE)
  kinds <- vapply(picks, function(p) if (is.null(p)) NA_character_ else p$kind,
                  character(1))
  expect_true(all(kinds %in% c("polynomial", "spline", "robust",
                                "family_swap", "random_effect")))
  expect_true(length(unique(stats::na.omit(kinds))) >= 2L)
})

test_that("apply_model_form rewrites spec$family for fitter-switching kinds", {
  spec <- texanshootR:::new_spec(c("x1"))
  spec$family <- list(fitter = "lm")

  s_rob <- texanshootR:::apply_model_form(
    spec, list(kind = "robust"))
  expect_equal(s_rob$family$fitter, "rlm")

  s_swap <- texanshootR:::apply_model_form(
    spec, list(kind = "family_swap", family = "poisson", link = "log",
               outcome_coerce = "round_clip"))
  expect_equal(s_swap$family$fitter, "glm")
  expect_equal(s_swap$family$family, "poisson")
  expect_equal(s_swap$family$outcome_coerce, "round_clip")

  s_re <- texanshootR:::apply_model_form(
    spec, list(kind = "random_effect", group = "grp"))
  expect_equal(s_re$family$fitter, "glmm")
  expect_equal(s_re$family$group, "grp")
})

test_that("apply_model_form leaves spec$family alone for predictor wraps", {
  spec <- texanshootR:::new_spec(c("x1"))
  spec$family <- list(fitter = "lm")
  s_poly <- texanshootR:::apply_model_form(
    spec, list(kind = "polynomial", var = "x1", degree = 2L))
  expect_equal(s_poly$family$fitter, "lm")
  s_spl <- texanshootR:::apply_model_form(
    spec, list(kind = "spline", var = "x1", df = 3L))
  expect_equal(s_spl$family$fitter, "lm")
})

test_that("materialise_spec wraps polynomial / spline predictors in RHS", {
  d <- make_phase2_df()
  spec <- texanshootR:::new_spec(c("x1", "x2"))
  spec$model_form <- list(kind = "polynomial", var = "x1", degree = 2L)
  mat <- texanshootR:::materialise_spec(d, "y", spec)
  expect_match(mat$formula, "poly(x1, 2)", fixed = TRUE)

  spec$model_form <- list(kind = "spline", var = "x2", df = 3L)
  mat <- texanshootR:::materialise_spec(d, "y", spec)
  expect_match(mat$formula, "splines::ns(x2, df = 3)", fixed = TRUE)
})

test_that("fit_rlm returns a well-formed result row", {
  d <- make_phase2_df()
  spec <- texanshootR:::new_spec(c("x1", "x2"))
  spec$subset_full <- "x1,x2"
  spec$family <- list(fitter = "rlm")
  r <- texanshootR:::fit_spec(d, "y", spec)
  expect_type(r, "list")
  expect_equal(r$family, "rlm:huber")
  expect_true(is.finite(r$p_value))
  expect_true(is.finite(r$r_squared))
  expect_true(is.finite(r$aic))
  expect_equal(r$n, nrow(d))
})

test_that("describe_spec renders each model_form sub-kind", {
  spec_poly <- list(
    formula = "y ~ poly(x1, 2) + x2",
    model_form = list(kind = "polynomial", var = "x1", degree = 2L)
  )
  expect_match(texanshootR:::describe_spec(spec_poly),
               "allowing for a quadratic trend in x1")
  expect_match(texanshootR:::describe_spec(spec_poly),
               "y ~ x1 + x2", fixed = TRUE)

  spec_spl <- list(
    formula = "y ~ splines::ns(x1, df = 3) + x2",
    model_form = list(kind = "spline", var = "x1", df = 3L)
  )
  expect_match(texanshootR:::describe_spec(spec_spl),
               "modelling x1 flexibly via natural splines")

  spec_rob <- list(formula = "y ~ x1", model_form = list(kind = "robust"))
  expect_match(texanshootR:::describe_spec(spec_rob),
               "using a robust M-estimator")

  spec_re  <- list(formula = "y ~ x1 + (1 | grp)",
                   model_form = list(kind = "random_effect", group = "grp"))
  expect_match(texanshootR:::describe_spec(spec_re),
               "with a random intercept for grp")

  spec_swap <- list(
    formula = "y ~ x1",
    model_form = list(kind = "family_swap", family = "binomial",
                      link = "logit",
                      outcome_coerce = "median_dichotomize"))
  expect_match(texanshootR:::describe_spec(spec_swap),
               "logistic GLM after dichotomising")
})

test_that("methods_summary chains restriction + outcome + model_form", {
  spec <- list(
    formula = "y ~ poly(x1, 2)",
    restriction = list(kind = "outcome_iqr"),
    outcome_construction = list(kind = "z_within", factor = "grp"),
    model_form = list(kind = "polynomial", var = "x1", degree = 2L)
  )
  summary <- texanshootR:::methods_summary(spec)
  expect_match(summary, "winsorising")
  expect_match(summary, "z-scoring")
  expect_match(summary, "quadratic trend")
})

test_that("apply_model_form with NULL is a no-op", {
  spec <- texanshootR:::new_spec("x1")
  spec$family <- list(fitter = "lm")
  expect_identical(texanshootR:::apply_model_form(spec, NULL), spec)
})

# Shared registry loader. Three on-disk registries (achievements, life
# events, wardrobe cosmetics) all follow the same pattern: lazy-cache
# under .tx, read inst/<dir>/*.yaml, flatten entries to rows, build a
# data.frame. The variation is the per-entry shape and the final frame
# (some carry I()-wrapped list columns). Those go in via the row +
# frame callbacks.

load_registry <- function(inst_dir, cache_key,
                          entry_to_row, rows_to_frame, empty,
                          force = FALSE) {
  if (!force && !is.null(.tx[[cache_key]])) return(.tx[[cache_key]])
  dir <- system.file(inst_dir, package = "texanshootR")
  if (!nzchar(dir) || !dir.exists(dir)) {
    .tx[[cache_key]] <- empty()
    return(.tx[[cache_key]])
  }
  files <- list.files(dir, pattern = "\\.ya?ml$", full.names = TRUE)
  rows <- list()
  for (f in files) {
    entries <- yaml::read_yaml(f)
    if (is.null(entries)) next
    for (e in entries) {
      rows[[length(rows) + 1L]] <- entry_to_row(e)
    }
  }
  out <- if (length(rows) == 0L) empty() else rows_to_frame(rows)
  .tx[[cache_key]] <- out
  out
}

# Build an empty data.frame whose character columns are listed in
# `char_cols`, with one trailing list-column attached via I(list()).
# Used by the event and cosmetic empty-registry constructors.
new_list_col_frame <- function(char_cols, list_col) {
  args <- stats::setNames(
    replicate(length(char_cols), character(), simplify = FALSE),
    char_cols
  )
  args$stringsAsFactors <- FALSE
  out <- do.call(data.frame, args)
  out[[list_col]] <- I(list())
  out
}

#' Controlled vocabularies for the message and event registries
#'
#' These exported character vectors define the valid values for the
#' `tags`, `career_min`, `trigger_phase`, `mascot_state_affinity`, and
#' event `effects` fields used in `inst/messages/*.yaml` and
#' `inst/events/*.yaml`. [validate_messages()] enforces them.
#'
#' @name vocab
NULL

#' @describeIn vocab Valid `tags` values. The lower-case fallacy
#'   names from the design contract plus thematic content tags.
#' @export
vocab_tags <- c(
  # Fallacy/sin tags (every message must carry at least one)
  "texas_sharpshooter",
  "p_hacking",
  "harking",
  "multiple_comparisons",
  "optional_stopping",
  "data_dredging",
  "cherry_picking",
  "outlier_exclusion",
  "subgroup_fishing",
  "target_leakage",
  "researcher_dof",
  "overfitting",
  "stepwise_abuse",
  "collider_bias",
  "omitted_variable",
  "post_hoc",
  "survivorship",
  "publication_bias",
  "salami_slicing",
  "story_first",
  "model_shopping",
  "causal_overreach",
  "validation_leakage",
  "aggregation_artifact",
  "deflection",
  # Thematic tags
  "ecology",
  "ml",
  "phylo",
  "spatial",
  "temporal",
  "bayesian",
  "frequentist",
  "admin",
  "reviewer",
  "funding",
  "presentation",
  "narrative",
  "career",
  "onboarding"
)

#' @describeIn vocab Valid `trigger_phase` values.
#' @export
vocab_phases <- c(
  "blip",
  "loading",
  "promotion",
  "promotion_committee",
  "reviewer",
  "derived_escalation",
  "state_transition",
  "ultra_rare",
  "daily",
  "banner",
  "event",
  "event_consequence",
  "chain_opened",
  "stage_advanced",
  "chain_completed",
  "chain_broken"
)

#' @describeIn vocab Valid `career_min` values.
#' @export
vocab_careers <- c(
  "Junior Researcher",
  "Postdoc",
  "Senior Scientist",
  "PI"
)

#' @describeIn vocab Valid `mascot_state_affinity` values.
#' @export
vocab_mascot_states <- c(
  "composed",
  "uncertain",
  "anxious",
  "desperate",
  "resolved",
  "polishing",
  "submitting",
  "granted",
  "rejected"
)

#' @describeIn vocab Valid event-effect keys for `inst/events/*.yaml`.
#' @export
vocab_effects <- c(
  "throughput",
  "search_budget",
  "reviewer_resistance",
  "narrative_strength",
  "interpretation_confidence",
  "presentation_pressure",
  "funding_uncertainty",
  "typo_probability",
  "instability"
)

#' @describeIn vocab Career level ordering (low to high).
#' @keywords internal
career_levels <- function() {
  vocab_careers
}

#' Compare two career levels.
#'
#' @param a,b Career level strings.
#' @return Integer: -1 if a < b, 0 if equal, 1 if a > b.
#' @keywords internal
career_compare <- function(a, b) {
  ai <- match(a, vocab_careers)
  bi <- match(b, vocab_careers)
  if (is.na(ai) || is.na(bi)) {
    stop("Unknown career level: ", paste(c(a, b)[is.na(c(ai, bi))], collapse = ", "))
  }
  sign(ai - bi)
}

# Message selection algorithm.
#
# 1. Filter the registry by trigger_phase, career_min, requested tags,
#    model_family_affinity, mascot_state_affinity, and installed
#    package availability (entries with a `requires` list are dropped
#    if any required package is unavailable).
# 2. Assign rarity weights (common=1000, uncommon=100, rare=10,
#    legendary=1).
# 3. Apply recency suppression using the recent-id ring buffer.
#    Combo-chain follow-ups are exempt.
# 4. Apply combo-chain bias: if `combo_state` names a chain start,
#    boost weights for messages whose `combo_next` matches.
# 5. Weighted-sample one entry, return its id+text.

select_message <- function(phase,
                           career = NULL,
                           tags = NULL,
                           model_family = NULL,
                           mascot_state = NULL,
                           recent = character(),
                           combo_state = NULL,
                           tag_boost = NULL,
                           registry = NULL) {
  reg <- registry %||% load_message_registry()
  if (nrow(reg) == 0L) return(NULL)

  # 1. Filter.
  ok <- reg$trigger_phase == phase

  if (!is.null(career)) {
    ok <- ok & vapply(reg$career_min, function(c) {
      is.na(c) || career_compare(career, c) >= 0
    }, logical(1))
  }

  if (!is.null(tags) && length(tags)) {
    ok <- ok & vapply(reg$tags, function(t) any(t %in% tags), logical(1))
  }

  if (!is.null(model_family) && length(model_family)) {
    # If a message specifies affinities, at least one must match.
    # Messages with empty affinity are universal.
    ok <- ok & vapply(reg$model_family_affinity, function(a) {
      length(a) == 0L || any(a %in% model_family)
    }, logical(1))
  }

  if (!is.null(mascot_state)) {
    ok <- ok & vapply(reg$mascot_state_affinity, function(a) {
      length(a) == 0L || any(a %in% mascot_state)
    }, logical(1))
  }

  ok <- ok & vapply(reg$requires, function(req) {
    if (length(req) == 0L) return(TRUE)
    all(vapply(req, function(p) requireNamespace(p, quietly = TRUE), logical(1)))
  }, logical(1))

  if (!any(ok)) return(NULL)
  candidates <- reg[ok, , drop = FALSE]

  # 2. Rarity weights.
  w <- rarity_weight(candidates$rarity)

  # 3. Recency suppression: zero-out recent ids unless they're a combo
  #    follow-up to combo_state.
  if (length(recent)) {
    is_recent <- candidates$id %in% recent
    if (!is.null(combo_state)) {
      exempt <- candidates$combo_next == combo_state
      exempt[is.na(exempt)] <- FALSE
      is_recent <- is_recent & !exempt
    }
    w[is_recent] <- 0
  }

  # 4. Combo-chain bias: 10x weight for valid follow-ups.
  if (!is.null(combo_state)) {
    boost <- !is.na(candidates$combo_next) & candidates$combo_next == combo_state
    w[boost] <- w[boost] * 10
  }

  # 4b. Daily-flavor tag boost: 3x weight for entries carrying any of
  #     the boosted tags (e.g. Tabulation Tuesday boosts `p_hacking`).
  if (!is.null(tag_boost) && length(tag_boost)) {
    has_boost <- vapply(candidates$tags,
                        function(t) any(t %in% tag_boost), logical(1))
    w[has_boost] <- w[has_boost] * 3
  }

  if (sum(w) == 0) return(NULL)

  # 5. Weighted sample.
  idx <- sample.int(nrow(candidates), size = 1L, prob = w)
  list(
    id          = candidates$id[idx],
    text        = candidates$text[idx],
    combo_start = candidates$combo_start[idx]
  )
}

rarity_weight <- function(r) {
  out <- numeric(length(r))
  out[r == "common"]    <- 1000
  out[r == "uncommon"]  <- 100
  out[r == "rare"]      <- 10
  out[r == "legendary"] <- 1
  out
}

# Small ring-buffer helper for recent-message suppression.
recent_buffer <- function(size = 50L) {
  list(buf = character(size), head = 0L, n = 0L, size = size)
}

recent_push <- function(rb, id) {
  rb$head <- (rb$head %% rb$size) + 1L
  rb$buf[rb$head] <- id
  rb$n <- min(rb$n + 1L, rb$size)
  rb
}

recent_ids <- function(rb) {
  if (rb$n == 0L) character() else utils::head(rb$buf, rb$n)
}

test_that("message registry validates", {
  reg <- validate_messages()
  expect_s3_class(reg, "data.frame")
  expect_gt(nrow(reg), 0)
  expect_true(all(reg$trigger_phase %in% vocab_phases))
  expect_true(all(reg$rarity %in% c("common", "uncommon", "rare", "legendary")))
})

test_that("selection algorithm returns a candidate for the blip phase", {
  withr::local_seed(1)
  draw <- texanshootR:::select_message(phase = "blip")
  expect_false(is.null(draw))
  expect_true(nzchar(draw$text))
})

test_that("recency suppression hides repeat ids", {
  withr::local_seed(1)
  rb <- texanshootR:::recent_buffer()
  d1 <- texanshootR:::select_message(phase = "blip")
  rb <- texanshootR:::recent_push(rb, d1$id)
  for (i in 1:50) {
    d <- texanshootR:::select_message(phase = "blip",
                                       recent = texanshootR:::recent_ids(rb))
    if (!is.null(d)) rb <- texanshootR:::recent_push(rb, d$id)
  }
  expect_true(d1$id %in% texanshootR:::recent_ids(rb) ||
                length(texanshootR:::recent_ids(rb)) > 0)
})

# TUI manager with three rendering modes.
#
# 1. ANSI multi-zone (real terminals: Windows Terminal, MINGW64, etc.)
#    Layout (top to bottom):
#      shooter    ~6 lines  the mascot ASCII; redraws on state transition
#                           and on heartbeat tick (small gun-line change)
#      flavour    4-6 lines fixed-height ring buffer for blip stream
#      modifiers  1 line    live-available +mod tokens
#      outputs   1 line    publication-chain stages, locked / unlocked
#      bar        2 lines   progress bar + most recent loading line
#    Each zone gets a 1-line blank gutter. Row offsets are computed in
#    ui_session_open() and cached on state.
#
# 2. Dynamic single-line (RStudio Console and similar: \r honored but
#    cursor positioning isn't). All zones compress to ONE line that
#    redraws in place via \r. Composed layout, left to right:
#      [bar + pct] | face + gun | +mods... | loading-text
#    Blip stream and full ASCII art are suppressed in this mode (they
#    would force scrolling). Mascot still ticks (heartbeat) and changes
#    on emotional transitions, just as a single face+gun line.
#
# 3. Plain (CI, RMarkdown chunks, anything without \r). Sequential
#    printing, one line per update.

# ---- ANSI helpers ---------------------------------------------------

# Reliable RStudio detection. `.Platform$GUI == "RStudio"` works in
# RStudio Desktop's R Console but reports "unknown" in some
# remote/server/embedded setups. RStudio sets the RSTUDIO env var to
# "1" in *all* of its R processes, so we OR the two signals.
in_rstudio <- function() {
  identical(.Platform$GUI, "RStudio") ||
    identical(Sys.getenv("RSTUDIO"), "1")
}

# UI-activation gate. Wraps `interactive()` in a texanshootR-owned
# binding so tests can mock it via assignInNamespace -- `base::interactive`
# itself has a locked binding and can't be patched directly.
is_interactive_ui <- function() interactive()

ansi_supported <- function() {
  if (!isTRUE(opt("texanshootR.animations"))) return(FALSE)
  if (!is_interactive_ui()) return(FALSE)
  # User override: force a specific mode regardless of auto-detect.
  forced <- getOption("texanshootR.ui_mode", NA_character_)
  if (identical(forced, "ansi"))    return(TRUE)
  if (identical(forced, "dynamic")) return(FALSE)
  if (identical(forced, "plain"))   return(FALSE)
  # We need MORE than SGR colors -- cursor save/restore (\033[s/u) and
  # cursor positioning (\033[<n>A/B/H). RStudio's console honors SGR but
  # not cursor positioning, so cli::ansi_has_any() returning TRUE is not
  # enough on its own.
  if (in_rstudio()) return(FALSE)
  isTRUE(cli::ansi_has_any())
}

# Dynamic TTY: stdout respects \r so we can overwrite a single line.
# Works in RStudio console, modern Windows terminals, and most others
# where the multi-zone ANSI manager fails. We use this as a graceful
# downgrade path for the bar.
dynamic_tty_supported <- function() {
  if (!isTRUE(opt("texanshootR.animations"))) return(FALSE)
  if (!is_interactive_ui()) return(FALSE)
  forced <- getOption("texanshootR.ui_mode", NA_character_)
  if (identical(forced, "dynamic")) return(TRUE)
  if (identical(forced, "ansi"))    return(FALSE)
  if (identical(forced, "plain"))   return(FALSE)
  # RStudio's console honors \r overwrite even though cli::is_dynamic_tty()
  # returns FALSE there (it's not a real TTY). Opt in explicitly so the bar
  # redraws in place instead of printing a fresh line per percent.
  if (in_rstudio()) return(TRUE)
  tryCatch(isTRUE(cli::is_dynamic_tty()),
           error = function(e) FALSE)
}

# Diagnostic helper: prints the signals that drive mode selection.
# Call as `texanshootR:::ui_mode_diagnostic()` if the wrong mode is
# being picked.
#' @keywords internal
ui_mode_diagnostic <- function() {
  list(
    .Platform_GUI = .Platform$GUI,
    RSTUDIO_env   = Sys.getenv("RSTUDIO"),
    interactive   = interactive(),
    animations    = isTRUE(opt("texanshootR.animations")),
    forced_mode   = getOption("texanshootR.ui_mode", NA_character_),
    in_rstudio    = in_rstudio(),
    ansi          = ansi_supported(),
    dynamic       = dynamic_tty_supported(),
    cli_is_dyn    = tryCatch(cli::is_dynamic_tty(),
                              error = function(e) conditionMessage(e)),
    cli_ansi_has  = tryCatch(cli::ansi_has_any(),
                              error = function(e) conditionMessage(e))
  )
}

# In dynamic (non-rich-ANSI) mode, blanket-clear the current line with
# \r + spaces + \r so any leftover bar text is wiped before we print a
# scrolling message. RStudio's console honors \r but not \033[2K, so we
# can't rely on the line-clear escape.
dyn_clear_line <- function() {
  cat("\r", strrep(" ", getOption("width", 80L)), "\r", sep = "")
}

# Compose the single-line status from the per-zone buffers. Three slots
# only -- shooter | message | bar -- because RStudio's console isn't wide
# enough for more, and modifier injection is non-interactive in this mode
# (pre-rolled at run start, see shoot.R) so there is no live picker to
# advertise. Order: mascot anchors the left edge at a fixed pad width,
# the message is the variable middle, the bar sits flush right with a
# stable width. Empty slots are dropped so the line stays compact early
# in a run before every zone has been populated.
dyn_compose <- function(state) {
  parts <- character()
  if (nzchar(state$dyn_mascot))  parts <- c(parts, state$dyn_mascot)
  if (nzchar(state$dyn_loading)) parts <- c(parts, state$dyn_loading)
  if (nzchar(state$dyn_bar))     parts <- c(parts, state$dyn_bar)
  paste(parts, collapse = " | ")
}

# Display widths for the dynamic single-line slots. Each slot pads to
# its target so the column separators line up frame-over-frame and the
# bar never jitters left/right.
#
# Full line at the 120-col target terminal:
#
#   |- mascot 18 -| | |- loading <=70 -| | |-- bar 25 --|
#                 18 + 3 + 70 + 3 + 25 = 119 cells
#
# Mascot's max content is face(<=7) + space + gun(10) = 18 cells, so
# DYN_MASCOT_WIDTH = 18 fits the widest face with no trailing slack.
# Bar is exactly 20 fill cells + space + "100%" = 25.
#
# DYN_LOADING_BUDGET is the hard cap on bundled message text length.
# validate_registry() lints every "loading" / "state_transition" entry
# in inst/messages/*.yaml against this cap so they fit cleanly at the
# 120-col target. Narrower terminals still ellipsise -- that is the
# graceful-degradation case, not the design point.
DYN_MASCOT_WIDTH      <- 18L
DYN_BAR_WIDTH         <- 25L   # 20 cells of blocks + " " + "100%"
DYN_LOADING_BUDGET    <- 70L   # cap on bundled message text length (chars)
DYN_LOADING_WIDTH_MIN <- 20L

# Compute the loading slot's target cell width from the current terminal
# width. Read live (not at session open) so resizing the host terminal
# between renders takes effect on the next ui_loading() call. The
# reserved budget covers mascot + " | " + " | " + bar. Capped at
# DYN_LOADING_BUDGET so we don't pad the slot with excess spaces on
# very wide terminals.
dyn_loading_width <- function() {
  w <- max(DYN_LOADING_WIDTH_MIN,
           as.integer(getOption("width", 80L)) - 1L)
  reserved <- DYN_MASCOT_WIDTH + 3L + 3L + DYN_BAR_WIDTH
  min(DYN_LOADING_BUDGET,
      max(DYN_LOADING_WIDTH_MIN, w - reserved))
}

# Right-pad / truncate to a target *display* width. For mascot glyphs
# we pass in the pre-computed cell width because nchar(type="width")
# misreports box-drawing and CJK glyphs on Windows -- see the
# FACE_CELL_WIDTH / GUN_FRAME_CELL_WIDTH tables in mascot.R. For plain
# ASCII text (loading messages) we fall back to nchar(type="chars"),
# which matches cells exactly when there's no Unicode.
pad_to_width <- function(s, w, cell_width = NULL) {
  cur <- if (is.null(cell_width)) {
    tryCatch(nchar(s, type = "width"),
             error = function(e) nchar(s))
  } else {
    as.integer(cell_width)
  }
  if (cur > w) {
    if (w >= 3L) return(paste0(substr(s, 1L, w - 3L), "..."))
    return(substr(s, 1L, w))
  }
  if (cur == w) return(s)
  paste0(s, strrep(" ", w - cur))
}

# Emit the composed line via \r-overwrite. Pads to the previous length
# so trailing characters from a longer earlier render get wiped. We
# can't rely on \033[2K to clear-to-EOL because RStudio's console
# doesn't honor it, hence the explicit padding.
dyn_render <- function(state) {
  line <- dyn_compose(state)
  if (identical(line, state$dyn_last_line)) return(invisible())
  prev_len <- nchar(state$dyn_last_line %||% "")
  w <- max(20L, getOption("width", 80L) - 1L)
  if (nchar(line) > w) {
    line <- paste0(substr(line, 1L, w - 3L), "...")
  }
  state$dyn_last_line <- line
  pad <- max(0L, prev_len - nchar(line))
  cat("\r", line, strrep(" ", pad), sep = "")
  utils::flush.console()
}

ansi_save    <- function() "\033[s"
ansi_restore <- function() "\033[u"
ansi_move    <- function(row, col = 1L) sprintf("\033[%d;%dH", row, col)
ansi_up      <- function(n) if (n > 0) sprintf("\033[%dA", n) else ""
ansi_down    <- function(n) if (n > 0) sprintf("\033[%dB", n) else ""
ansi_clear_line <- function() "\033[2K"
ansi_dim     <- function(s) paste0("\033[2m", s, "\033[22m")
ansi_cyan    <- function(s) paste0("\033[36m", s, "\033[39m")

# ---- Theme palette --------------------------------------------------

style_header  <- function(s) if (ansi_supported()) ansi_cyan(s) else s
style_blip    <- function(s) if (ansi_supported()) ansi_dim(s)  else s
style_status  <- function(s) s
style_event   <- function(s) {
  if (ansi_supported()) paste0("\033[31mEVENT:\033[39m ", s)
  else paste0("EVENT: ", s)
}

# ---- Session manager ------------------------------------------------

# Reserve N blank lines below the cursor and remember the absolute row
# where the block starts. We use ESC[s/ESC[u for save/restore - if the
# host terminal moves the cursor in unexpected ways, we still draw to
# the same relative offsets.

ui_session_open <- function(blip_lines = 4L,
                            ansi = ansi_supported(),
                            dynamic = dynamic_tty_supported()) {
  shooter_h  <- length(read_body_template())
  blip_h     <- blip_lines
  modifier_h <- 1L
  output_h   <- 1L
  bar_h      <- 2L

  # Each zone is followed by a 1-line gutter, except the bar's loading
  # line which sits flush against the progress bar.
  row_blip     <- shooter_h + 1L
  row_modifier <- row_blip + blip_h + 1L
  row_output   <- row_modifier + modifier_h + 1L
  row_bar      <- row_output + output_h + 1L
  row_loading  <- row_bar + 1L
  total_h      <- row_loading + 1L

  # Environment, not list. R lists are copy-on-modify, so cache fields
  # set inside ui_set_*() would be lost when the caller (e.g.
  # maybe_animate()) discards the return value. An env mutates in place.
  state <- new.env(parent = emptyenv())
  state$ansi         <- ansi
  state$dynamic      <- !ansi && dynamic   # only use \r path when rich ANSI isn't available
  state$shooter_h    <- shooter_h
  state$blip_h       <- blip_h
  state$modifier_h   <- modifier_h
  state$output_h     <- output_h
  state$bar_h        <- bar_h
  state$row_blip     <- row_blip
  state$row_modifier <- row_modifier
  state$row_output   <- row_output
  state$row_bar      <- row_bar
  state$row_loading  <- row_loading
  state$total_h      <- total_h
  state$blips        <- character(blip_h)
  state$last_state   <- NA_character_
  state$last_tick    <- -1L
  state$last_loading <- ""
  state$last_modifiers_render <- NA_character_
  state$last_outputs_render   <- NA_character_
  state$last_progress_render  <- NA_character_
  # Dynamic single-line composed-status buffers. Each ui_set_* writer
  # updates its own slot, then dyn_render() composes them into a single
  # \r-overwritten status line. Unused in ANSI or plain mode.
  state$dyn_bar       <- ""
  state$dyn_mascot    <- ""
  state$dyn_loading   <- ""
  state$dyn_last_line <- ""
  if (ansi) {
    # Reserve space by printing newlines, then move back up.
    cat(strrep("\n", total_h))
    cat(ansi_up(total_h))
    cat(ansi_save())
  }
  state
}

ui_session_close <- function(state) {
  if (state$ansi) {
    cat(ansi_restore())
    cat(ansi_down(state$total_h))
    cat("\n")
  } else if (isTRUE(state$dynamic)) {
    # Wipe the carriage-return-overwritten line so the banner that
    # follows can claim the row instead of stacking under a leftover
    # animation frame. The animation served its purpose during the run
    # (mascot walked the emotional arc, bar filled to 100%); after the
    # run, the print.tx_run banner is the canonical end-state display,
    # and printing both produces a "duplicated shooter" effect with
    # two mascot faces stacked on top of each other.
    dyn_clear_line()
    utils::flush.console()
  } else {
    cat("\n")
  }
  invisible()
}

# ---- Zone writers ---------------------------------------------------

ui_set_mascot <- function(state, mstate, tick, cosmetics = NULL,
                          force = FALSE) {
  if (!force && identical(state$last_state, mstate) &&
      identical(state$last_tick, tick)) {
    return(state)
  }
  lines <- mascot_render(mstate, tick = tick, cosmetics = cosmetics)
  # Pad / truncate to shooter_h.
  if (length(lines) < state$shooter_h) {
    lines <- c(lines, rep("", state$shooter_h - length(lines)))
  } else if (length(lines) > state$shooter_h) {
    lines <- lines[seq_len(state$shooter_h)]
  }
  transition <- is.na(state$last_state) || !identical(state$last_state, mstate)
  if (state$ansi) {
    cat(ansi_restore())
    for (i in seq_along(lines)) {
      cat(ansi_clear_line(), lines[i], "\n", sep = "")
    }
  } else if (isTRUE(state$dynamic)) {
    # Single-line mode: render face + 2-frame dot-blink gun. Uses
    # heartbeat_frame_dyn (NOT heartbeat_frame) because the full
    # 8-frame cycle changes the wide-char composition tick-over-tick,
    # which jitters the mascot slot in RStudio Console (which renders
    # `一` at a different cell width than the standard claims). The
    # _dyn variant keeps `︻デ═一` constant and only toggles the
    # trailing `·` ↔ space, so the slot stays anchored at the same
    # column on every tick. Heartbeat ticks AND state transitions
    # both update the slot; dyn_render dedups so identical frames
    # cost nothing.
    face <- read_face(mstate)
    gun  <- heartbeat_frame_dyn(tick)
    combined  <- paste(face, gun)
    combined_w <- face_cell_width(mstate) + 1L + gun_frame_cell_width(tick)
    state$dyn_mascot <- pad_to_width(combined, DYN_MASCOT_WIDTH,
                                      cell_width = combined_w)
    dyn_render(state)
  } else if (transition) {
    # Plain mode: print only on state transition, not on every tick.
    for (l in lines) cat(l, "\n", sep = "")
  }
  state$last_state <- mstate
  state$last_tick  <- tick
  state
}

ui_blip <- function(state, text) {
  state$blips <- c(state$blips[-1L], text)
  if (state$ansi) {
    cat(ansi_restore())
    cat(ansi_down(state$row_blip))
    for (i in seq_len(state$blip_h)) {
      cat(ansi_clear_line(), style_blip(state$blips[i]), "\n", sep = "")
    }
  } else if (isTRUE(state$dynamic)) {
    # Single-line mode: blip ring buffer still updates (state$blips) so
    # message dedup / recent-buffer logic stays intact, but we don't
    # display the blip stream — it would force scrolling.
  } else {
    cat(style_blip(text), "\n", sep = "")
  }
  state
}

ui_progress <- function(state, pct, label = "") {
  pct <- max(0, min(1, pct))
  # 20 cells in both modes. In ANSI multi-zone, the bar gets its own
  # row. In dynamic single-line, the bar shares its row with mascot,
  # modifiers, and loading text, so 20 cells leaves room for the rest.
  width <- 20L
  filled <- as.integer(round(pct * width))
  # Unicode block + light-shade glyphs (U+2588, U+2591). Built via
  # intToUtf8() so this source file stays ASCII-only.
  bar_full  <- intToUtf8(9608L)
  bar_empty <- intToUtf8(9617L)
  bar <- paste0(strrep(bar_full, filled),
                strrep(bar_empty, width - filled))
  pct_lbl <- sprintf("%3d%%", as.integer(round(pct * 100)))
  line <- if (nzchar(label)) paste(bar, pct_lbl, label) else paste(bar, pct_lbl)
  if (identical(line, state$last_progress_render)) return(state)
  state$last_progress_render <- line
  if (state$ansi) {
    cat(ansi_restore())
    cat(ansi_down(state$row_bar))
    cat(ansi_clear_line(), line, "\n", sep = "")
    cat(ansi_clear_line(), "> ", style_status(state$last_loading),
        "\n", sep = "")
  } else if (isTRUE(state$dynamic)) {
    # Update just the bar slot and let dyn_render compose with the
    # other zones. Drop the `label` arg in this mode — the loading
    # text is already its own slot via ui_loading().
    state$dyn_bar <- paste(bar, pct_lbl)
    dyn_render(state)
  } else {
    cat(line, "\n", sep = "")
  }
  state
}

ui_loading <- function(state, text) {
  state$last_loading <- text
  if (state$ansi) {
    cat(ansi_restore())
    cat(ansi_down(state$row_loading))
    cat(ansi_clear_line(), "> ", style_status(text), "\n", sep = "")
  } else if (isTRUE(state$dynamic)) {
    # Pad / truncate the message to a fixed cell width so the bar slot
    # to its right stays anchored frame-over-frame. Long messages are
    # ellipsised; short ones get trailing spaces.
    state$dyn_loading <- pad_to_width(text, dyn_loading_width())
    dyn_render(state)
  } else {
    cat("> ", text, "\n", sep = "")
  }
  state
}

# Live-available modifier tokens. Renders once per shoot() batch (so the
# row tracks state$consumed_modifiers as the player burns through them).
# Cached on state$last_modifiers_render to skip redundant repaints when
# the row text hasn't changed since the last call.
ui_set_modifiers <- function(state, available_tokens) {
  line <- if (length(available_tokens))
            paste0("MODIFIERS  ",
                   format_modifier_row(available_tokens))
          else
            "MODIFIERS  (all consumed)"
  if (identical(line, state$last_modifiers_render)) return(state)
  state$last_modifiers_render <- line
  if (state$ansi) {
    cat(ansi_restore())
    cat(ansi_down(state$row_modifier))
    cat(ansi_clear_line(), style_header(line), "\n", sep = "")
  } else if (isTRUE(state$dynamic)) {
    # Dynamic single-line mode skips the modifiers row entirely. The
    # row would compete with the mascot, bar, and loading text for one
    # narrow console line, and modifier injection is non-interactive
    # here (pre-rolled at run start), so there is no live picker to
    # advertise. The cache update above is still useful for ANSI mode.
  } else {
    cat(line, "\n", sep = "")
  }
  state
}

# Publication-chain stages with locked / unlocked styling at the
# player's career tier. `unlocked_length` is the prefix length the
# player has earned (1..6). `current_due` (NA when not in chain phase)
# is the stage in [brackets] -- not used in search phase yet, reserved
# for Slice 5 when the chain plays out inside the same TUI session.
ui_set_outputs <- function(state, stages, unlocked_length,
                            current_due = NA_character_) {
  parts <- vapply(seq_along(stages), function(i) {
    s <- stages[i]
    if (!is.na(current_due) && identical(s, current_due)) {
      sprintf("[%s]", s)
    } else if (i <= unlocked_length) {
      s
    } else {
      ansi_or_plain_dim(state, s)
    }
  }, character(1))
  line <- paste0("OUTPUTS    ", paste(parts, collapse = "  "))
  if (identical(line, state$last_outputs_render)) return(state)
  state$last_outputs_render <- line
  if (state$ansi) {
    cat(ansi_restore())
    cat(ansi_down(state$row_output))
    cat(ansi_clear_line(), line, "\n", sep = "")
  } else if (isTRUE(state$dynamic)) {
    # Outputs row hidden in single-line mode — chain progression is a
    # between-runs concept, not an in-run heartbeat. The unlocked count
    # is still tracked on state for finalize_run().
  } else {
    cat(line, "\n", sep = "")
  }
  state
}

# Helper: dim if ANSI is active on this session, else passthrough.
# Kept here so the OUTPUTS renderer doesn't have to branch inline.
ansi_or_plain_dim <- function(state, s) {
  if (isTRUE(state$ansi)) ansi_dim(s) else s
}

ui_event <- function(state, event_text, consequence_text) {
  if (state$ansi) {
    cat(ansi_restore())
    cat(ansi_down(state$row_blip))
    cat(ansi_clear_line(), style_event(event_text), "\n", sep = "")
    cat(ansi_clear_line(), consequence_text, "\n", sep = "")
    Sys.sleep(0.6)
    cat(ansi_restore())
    cat(ansi_down(state$row_blip))
    for (i in seq_len(state$blip_h)) {
      cat(ansi_clear_line(), style_blip(state$blips[i]), "\n", sep = "")
    }
  } else if (isTRUE(state$dynamic)) {
    # Flash the event briefly through the loading slot. The composed
    # status line stays put; only the loading text changes.
    prev <- state$dyn_loading
    state$dyn_loading <- paste0("EVENT: ", event_text)
    dyn_render(state)
    Sys.sleep(0.6)
    state$dyn_loading <- consequence_text
    dyn_render(state)
    # Don't restore `prev` — let the next ui_loading() call overwrite.
  } else {
    cat(style_event(event_text), "\n", consequence_text, "\n", sep = "")
  }
  state
}

# ---- Plain (deadpan) message printing ------------------------------

# For one-off sober lines outside a UI session (e.g. promotion
# announcements, output status prints).
say <- function(text) {
  if (isTRUE(opt("texanshootR.quiet"))) return(invisible())
  cat(text, "\n", sep = "")
  invisible()
}

# End-to-end tests for the dynamic single-line render.
#
# Each test:
#   - Forces dynamic mode (texanshootR.ui_mode = "dynamic") and spoofs
#     interactive() so shoot() actually opens a UI session under
#     Rscript / testthat.
#   - Replaces dyn_render with a logger that captures every composed
#     line. The bytes never hit stdout so the testthat output stays
#     clean; we assert on the captured trace.
#   - Runs shoot() to completion and inspects the trace.
#
# Why we mock interactive(): ansi_supported() and dynamic_tty_supported()
# both short-circuit on `!interactive()`. With Rscript that's FALSE and
# shoot() would skip the UI entirely. We have to patch the binding.

with_dynamic_ui <- function(code) {
  # Force the UI-activation gate to TRUE so shoot()'s open_tui branch
  # passes under Rscript / testthat. `base::interactive` itself has a
  # locked binding; we route every UI check through texanshootR's own
  # `is_interactive_ui` for exactly this reason.
  orig_gate <- texanshootR:::is_interactive_ui
  assignInNamespace("is_interactive_ui", function() TRUE,
                    ns = "texanshootR")
  on.exit(assignInNamespace("is_interactive_ui", orig_gate,
                            ns = "texanshootR"), add = TRUE)

  # Capture every composed status line dyn_render would have emitted.
  log_env <- new.env(parent = emptyenv())
  log_env$lines <- character()
  orig_render <- texanshootR:::dyn_render
  assignInNamespace("dyn_render", function(state) {
    log_env$lines <- c(log_env$lines, texanshootR:::dyn_compose(state))
    invisible()
  }, ns = "texanshootR")
  on.exit(assignInNamespace("dyn_render", orig_render, ns = "texanshootR"),
          add = TRUE)

  force(code)
  log_env$lines
}

face_glyph_of <- function(line) {
  m <- regmatches(line, regexpr("^\\([^)]*\\)", line))
  if (length(m) == 0L) NA_character_ else m
}

test_that("shoot() walks the face ladder end-to-end and flashes resolved", {
  testthat::skip_if_not_installed("withr")
  withr::local_options(
    texanshootR.animations = TRUE,
    texanshootR.ui_mode    = "dynamic",
    texanshootR.budget     = 4L,
    texanshootR.quiet      = TRUE,
    texanshootR.consent    = TRUE,
    texanshootR.life_events = FALSE
  )

  lines <- with_dynamic_ui(invisible(shoot(mtcars, seed = 1)))

  expect_gt(length(lines), 0L)

  faces <- unique(stats::na.omit(vapply(lines, face_glyph_of, character(1))))

  # The progress ladder should cover at least three states by the time
  # progress runs from 0 to 1 with a 4-second budget. We don't pin all
  # six because random per-fit timing can occasionally clip the ladder
  # at one end, but three is the floor below which something is wrong.
  expect_gte(length(faces), 3L)

  # The very last rendered face must be `resolved` (` -_-`) because the
  # final-frame flash fires when best_p <= 0.05, and mtcars hits that
  # almost immediately.
  last_face <- tail(stats::na.omit(vapply(lines, face_glyph_of, character(1))), 1)
  expect_equal(unname(last_face), "( -_-)")

  # The progress bar must reach 100% on its last update -- the loop
  # keeps animating after the spec cap is hit so the bar fills the
  # full budget regardless of how fast fits finish.
  pct <- regmatches(lines, regexpr("\\d{1,3}%", lines))
  pct <- as.integer(sub("%", "", pct))
  expect_equal(max(pct, na.rm = TRUE), 100L)
})

test_that("shoot() auto-generates output files when flagged", {
  testthat::skip_if_not_installed("withr")
  out_dir <- tempfile("tx_auto_outputs_")
  dir.create(out_dir, recursive = TRUE)
  on.exit(unlink(out_dir, recursive = TRUE), add = TRUE)

  withr::local_options(
    texanshootR.animations = FALSE,   # no UI work needed for this assertion
    texanshootR.budget     = 2L,
    texanshootR.quiet      = TRUE,
    texanshootR.consent    = TRUE,
    texanshootR.life_events = FALSE,
    texanshootR.output_dir = out_dir
  )

  run <- shoot(mtcars, seed = 1, abstract = TRUE)
  expect_s3_class(run, "tx_run")
  # If the run was shippable, the abstract file should have landed.
  # If it wasn't, the flag should have been a silent no-op.
  if (isTRUE(run$shippable)) {
    expect_true(any(grepl("abstract", list.files(out_dir))))
    expect_true("abstract" %in% (run$outputs_generated %||% character()))
  } else {
    expect_length(list.files(out_dir), 0L)
  }
})

test_that("powerpoint alias maps to presentation flag", {
  testthat::skip_if_not_installed("withr")
  testthat::skip_if_not_installed("officer")
  testthat::skip_if_not_installed("rmarkdown")
  out_dir <- tempfile("tx_alias_")
  dir.create(out_dir, recursive = TRUE)
  on.exit(unlink(out_dir, recursive = TRUE), add = TRUE)

  withr::local_options(
    texanshootR.animations = FALSE,
    texanshootR.budget     = 2L,
    texanshootR.quiet      = TRUE,
    texanshootR.consent    = TRUE,
    texanshootR.life_events = FALSE,
    texanshootR.output_dir = out_dir
  )

  # Force a generous chain unlock so all four prefix stages are
  # reachable for this test regardless of prior persisted state.
  meta <- texanshootR:::read_meta() %||% list()
  meta$progression <- list(xp = 999L, length_unlocked = 3L,
                            chains_completed = 0L, chains_broken = 0L)
  meta$career_level <- "Postdoc"
  texanshootR:::write_meta(meta)

  run <- shoot(mtcars, seed = 1, powerpoint = TRUE)
  expect_s3_class(run, "tx_run")
  if (isTRUE(run$shippable)) {
    # Chain order: abstract -> manuscript -> presentation. All three
    # should have been generated to reach the powerpoint stage.
    expect_setequal(run$outputs_generated,
                    c("abstract", "manuscript", "presentation"))
  }
})

# Publication-chain state machine.
#
# A shippable run (one that cleared p <= 0.05) opens a *chain*. The
# chain is a sequence of output stages that must be redeemed in order.
# Hitting every stage in the currently-unlocked prefix awards full XP;
# starting a new shoot() before finishing breaks the chain and forfeits
# the length-bonus (per-stage XP already banked is kept).
#
# The chain length you may attempt grows with cumulative XP. At day 0
# only `abstract` is in the chain. As XP accrues, the chain grows
# rightward through the publication lifecycle.
#
# State lives on meta:
#   meta$progression$xp                 cumulative XP (integer)
#   meta$progression$length_unlocked    1..length(CHAIN_STAGES)
#   meta$progression$chains_completed   count of fully-completed chains
#   meta$active_chain                   NULL or live-chain state
#     $run_id            which run is being published
#     $stage_idx         1-based index of the NEXT-due stage
#     $completed_stages  character vector of stages landed so far
#     $opened_at         POSIXct, when shoot() opened this chain

CHAIN_STAGES <- c(
  "abstract",
  "manuscript",
  "presentation",
  "reviewer_response",
  "graphical_abstract",
  "funding"
)

# Cumulative XP at which each chain length unlocks. Index = length.
# length 1 is free; lengths 2..6 cost more each step.
CHAIN_XP_THRESHOLDS <- c(
  0L,    # length 1: abstract (free)
  5L,    # length 2: + manuscript
  15L,   # length 3: + presentation
  30L,   # length 4: + reviewer_response
  55L,   # length 5: + graphical_abstract
  90L    # length 6: + funding
)

# Career-tier label derived from how many stages are unlocked.
CHAIN_TIER_BY_LENGTH <- c(
  "Junior Researcher",   # length 1
  "Postdoc",             # length 2
  "Postdoc",             # length 3
  "Senior Scientist",    # length 4
  "Senior Scientist",    # length 5
  "PI"                   # length 6
)

stage_index <- function(stage) {
  match(stage, CHAIN_STAGES)
}

length_unlocked_for_xp <- function(xp) {
  xp <- as.integer(xp %||% 0L)
  out <- 1L
  for (i in seq_along(CHAIN_XP_THRESHOLDS)) {
    if (xp >= CHAIN_XP_THRESHOLDS[i]) out <- i
  }
  out
}

career_level_for_length <- function(length_unlocked) {
  i <- max(1L, min(as.integer(length_unlocked %||% 1L),
                    length(CHAIN_TIER_BY_LENGTH)))
  CHAIN_TIER_BY_LENGTH[[i]]
}

# Read the progression block from meta, filling defaults for missing
# fields so callers can rely on a stable shape.
progression_of <- function(meta) {
  p <- meta$progression %||% list()
  list(
    xp                = as.integer(p$xp %||% 0L),
    length_unlocked   = as.integer(p$length_unlocked %||% 1L),
    chains_completed  = as.integer(p$chains_completed %||% 0L),
    chains_broken     = as.integer(p$chains_broken %||% 0L)
  )
}

set_progression <- function(meta, prog) {
  meta$progression <- prog
  meta$career_level <- career_level_for_length(prog$length_unlocked)
  meta
}

# Award XP and recompute length_unlocked. Returns the updated meta
# plus attributes capturing what changed (used for the unlock banner).
award_xp <- function(meta, n) {
  prog   <- progression_of(meta)
  prev_l <- prog$length_unlocked
  prog$xp <- prog$xp + as.integer(n)
  new_l  <- length_unlocked_for_xp(prog$xp)
  prog$length_unlocked <- new_l
  meta <- set_progression(meta, prog)
  attr(meta, "xp_awarded")    <- as.integer(n)
  attr(meta, "length_grew")   <- new_l > prev_l
  attr(meta, "new_length")    <- new_l
  attr(meta, "prev_length")   <- prev_l
  meta
}

# Open a new chain on a shippable run. Aborts any existing chain
# (the player committed to a fresh run before redeeming the previous
# one).
open_chain <- function(meta, run_id) {
  prog <- progression_of(meta)
  if (!is.null(meta$active_chain)) {
    prog$chains_broken <- prog$chains_broken + 1L
    meta <- set_progression(meta, prog)
  }
  meta$active_chain <- list(
    run_id            = run_id,
    stage_idx         = 1L,
    completed_stages  = character(),
    opened_at         = as.numeric(Sys.time())
  )
  meta
}

close_chain <- function(meta, reason = c("complete", "broken")) {
  reason <- match.arg(reason)
  prog <- progression_of(meta)
  if (reason == "complete") {
    prog$chains_completed <- prog$chains_completed + 1L
  } else {
    prog$chains_broken <- prog$chains_broken + 1L
  }
  meta <- set_progression(meta, prog)
  meta$active_chain <- NULL
  meta
}

# Advance the chain past `stage` (the just-landed stage). If we've
# reached the end of the unlocked prefix, close the chain as
# complete and award the chain-bonus XP.
advance_chain <- function(meta, stage) {
  ac <- meta$active_chain
  if (is.null(ac)) return(meta)
  prog <- progression_of(meta)
  ac$completed_stages <- c(ac$completed_stages, stage)
  ac$stage_idx <- ac$stage_idx + 1L
  meta$active_chain <- ac
  meta <- award_xp(meta, 1L)

  # If the next stage index is beyond what the player has unlocked,
  # the chain is complete: bonus XP, close it.
  prog <- progression_of(meta)
  if (ac$stage_idx > prog$length_unlocked) {
    meta <- award_xp(meta, prog$length_unlocked)
    meta <- close_chain(meta, "complete")
    attr(meta, "chain_completed") <- TRUE
  }
  meta
}

# Reasons a stage call can fail. Each returns a structured condition
# we can both print and use in tests.
tx_chain_error <- function(reason, message, fields = list()) {
  structure(
    class = c("tx_chain_error", "error", "condition"),
    c(list(message = message, call = NULL, reason = reason), fields)
  )
}

# The single check called from every output_*.R generator. Verifies
# everything about chain state. On pass, advances the chain and
# persists the new meta. On any failure, signals tx_chain_error with
# a reason code so callers (and tests) can branch on it.
require_chain_stage <- function(stage, run) {
  if (!stage %in% CHAIN_STAGES) {
    stop(sprintf("`%s` is not a known chain stage.", stage), call. = FALSE)
  }
  meta <- read_meta() %||% default_meta(save_dir() %||% tempfile())
  prog <- progression_of(meta)

  s_idx <- stage_index(stage)
  if (s_idx > prog$length_unlocked) {
    next_thr <- CHAIN_XP_THRESHOLDS[min(s_idx,
                                          length(CHAIN_XP_THRESHOLDS))]
    msg <- format_locked_stage(stage, prog$xp, next_thr)
    stop(tx_chain_error("not_unlocked", msg,
                        list(stage = stage, xp = prog$xp,
                             xp_needed = next_thr)))
  }

  ac <- meta$active_chain
  if (is.null(ac)) {
    msg <- paste0(stage, "() requires:\n",
                  "an active publication chain\n\n",
                  "Active chain:\n",
                  "none. Run shoot() and clear p <= 0.05 first.")
    stop(tx_chain_error("no_active_chain", msg,
                        list(stage = stage)))
  }

  if (!identical(run$run_id, ac$run_id)) {
    msg <- paste0(stage, "() failed:\n",
                  "this run is not the active publication\n\n",
                  "Active run:\n",
                  ac$run_id)
    stop(tx_chain_error("wrong_run", msg,
                        list(stage = stage,
                             expected = ac$run_id,
                             provided = run$run_id)))
  }

  due_stage <- CHAIN_STAGES[[ac$stage_idx]]
  if (!identical(stage, due_stage)) {
    msg <- paste0(stage, "() failed:\n",
                  "next due stage is ", due_stage, "\n\n",
                  "Completed so far:\n",
                  if (length(ac$completed_stages))
                    paste(ac$completed_stages, collapse = ", ")
                  else "none")
    stop(tx_chain_error("wrong_stage", msg,
                        list(stage = stage, expected = due_stage,
                             completed = ac$completed_stages)))
  }

  # All checks pass. Caller will write its file, then must call
  # advance_chain_after_stage() to persist the advance. (Splitting
  # the check from the persist lets the caller fail without
  # advancing.)
  invisible(list(meta = meta, ok = TRUE))
}

# Called by an output generator after it has successfully produced
# its file. Persists chain advance + XP, then announces the transition
# (per-stage HUD line and a flavour pick from the message bank).
# Returns the new meta with attributes describing the transition.
advance_chain_after_stage <- function(stage) {
  meta <- read_meta()
  if (is.null(meta) || is.null(meta$active_chain)) return(invisible(NULL))
  meta <- advance_chain(meta, stage)
  write_meta(meta)
  say_stage_landed(meta, stage)
  invisible(meta)
}

# ---- Chain announcement helpers ------------------------------------
#
# These print one or two lines after a chain transition so the player
# sees the chain state evolve. Flavour is drawn from the message bank
# (inst/messages/chain.yaml). Each helper respects `texanshootR.quiet`
# via say(). The TUI session has already closed at this point, so we
# print plain lines rather than redrawing into the three-zone manager.

say_chain_flavor <- function(phase, mascot_state) {
  draw <- tryCatch(
    select_message(phase = phase, mascot_state = mascot_state),
    error = function(e) NULL
  )
  if (!is.null(draw)) say(draw$text)
}

say_chain_opened <- function(meta) {
  ac <- meta$active_chain
  if (is.null(ac)) return(invisible())
  due <- CHAIN_STAGES[[ac$stage_idx]]
  say_chain_flavor("chain_opened", "polishing")
  say(sprintf("Publication chain opened. Next: %s().", due))
  invisible()
}

say_stage_landed <- function(meta, stage) {
  if (isTRUE(attr(meta, "chain_completed"))) {
    say_chain_flavor("chain_completed", "granted")
    prog <- progression_of(meta)
    bonus <- as.integer(attr(meta, "xp_awarded") %||% 0L)
    say(sprintf("Chain complete. %s landed (+%d XP this stage incl. bonus). Total XP: %d.",
                stage, bonus, prog$xp))
    return(invisible())
  }
  ac <- meta$active_chain
  if (is.null(ac)) return(invisible())
  state_pick <- if (stage == "abstract") "polishing" else "submitting"
  say_chain_flavor("stage_advanced", state_pick)
  prog <- progression_of(meta)
  due  <- CHAIN_STAGES[[ac$stage_idx]]
  say(sprintf("Stage %s landed (+1 XP, total %d). Next: %s().",
              stage, prog$xp, due))
  invisible()
}

say_chain_broken <- function(reason, completed = character()) {
  say_chain_flavor("chain_broken", "rejected")
  completed_str <- if (length(completed))
                     paste(completed, collapse = ", ") else "none"
  say(sprintf("Publication chain broken; landed before abandon: %s.",
              completed_str))
  invisible()
}

format_locked_stage <- function(stage, xp, xp_needed) {
  paste0(stage, "() requires:\n",
         xp_needed, " XP (chain length ", stage_index(stage), ")\n\n",
         "Current XP:\n",
         xp)
}

# Public-facing one-line HUD for the active chain. Used by the print
# method of tx_run and by progress().
format_chain_hud <- function(meta) {
  ac <- meta$active_chain
  if (is.null(ac)) return(NULL)
  prog <- progression_of(meta)
  due <- CHAIN_STAGES[[ac$stage_idx]]
  sprintf(
    "Active chain: next %s()   [%d / %d done]",
    due,
    length(ac$completed_stages),
    prog$length_unlocked
  )
}

set_chain_length <- function(length_unlocked) {
  texanshootR:::ensure_save_dir()
  meta <- texanshootR:::read_meta()
  if (is.null(meta)) {
    meta <- texanshootR:::default_meta(getOption("texanshootR.save_dir"))
  }
  meta$progression$length_unlocked <- as.integer(length_unlocked)
  meta$progression$xp <-
    texanshootR:::CHAIN_XP_THRESHOLDS[as.integer(length_unlocked)]
  meta$career_level <-
    texanshootR:::career_level_for_length(length_unlocked)
  texanshootR:::write_meta(meta)
  invisible(meta)
}

clear_active_chain <- function() {
  meta <- texanshootR:::read_meta()
  if (is.null(meta)) return(invisible())
  meta$active_chain <- NULL
  texanshootR:::write_meta(meta)
}

test_that("CHAIN_STAGES carries six ordered stages", {
  expect_equal(length(texanshootR:::CHAIN_STAGES), 6L)
  expect_equal(texanshootR:::CHAIN_STAGES[[1]], "abstract")
  expect_equal(texanshootR:::CHAIN_STAGES[[6]], "funding")
})

test_that("length_unlocked_for_xp climbs as XP crosses thresholds", {
  expect_equal(texanshootR:::length_unlocked_for_xp(0L),   1L)
  expect_equal(texanshootR:::length_unlocked_for_xp(4L),   1L)
  expect_equal(texanshootR:::length_unlocked_for_xp(5L),   2L)
  expect_equal(texanshootR:::length_unlocked_for_xp(15L),  3L)
  expect_equal(texanshootR:::length_unlocked_for_xp(200L), 6L)
})

test_that("career_level_for_length maps to canonical tier labels", {
  expect_equal(texanshootR:::career_level_for_length(1L),
               "Junior Researcher")
  expect_equal(texanshootR:::career_level_for_length(2L), "Postdoc")
  expect_equal(texanshootR:::career_level_for_length(4L),
               "Senior Scientist")
  expect_equal(texanshootR:::career_level_for_length(6L), "PI")
})

test_that("is_unlocked tracks chain length", {
  set_chain_length(1L)
  expect_true(texanshootR:::is_unlocked("abstract"))
  expect_false(texanshootR:::is_unlocked("manuscript"))
  set_chain_length(2L)
  expect_true(texanshootR:::is_unlocked("manuscript"))
  expect_false(texanshootR:::is_unlocked("presentation"))
  set_chain_length(6L)
  expect_true(texanshootR:::is_unlocked("funding"))
})

test_that("require_chain_stage signals not_unlocked when stage above XP", {
  set_chain_length(1L)
  clear_active_chain()
  err <- tryCatch(
    texanshootR:::require_chain_stage("manuscript", list(run_id = "x")),
    tx_chain_error = identity
  )
  expect_s3_class(err, "tx_chain_error")
  expect_equal(err$reason, "not_unlocked")
})

test_that("require_chain_stage signals no_active_chain when none open", {
  set_chain_length(6L)
  clear_active_chain()
  err <- tryCatch(
    texanshootR:::require_chain_stage("abstract", list(run_id = "x")),
    tx_chain_error = identity
  )
  expect_s3_class(err, "tx_chain_error")
  expect_equal(err$reason, "no_active_chain")
})

test_that("require_chain_stage signals wrong_run when run id mismatches", {
  set_chain_length(2L)
  meta <- texanshootR:::read_meta()
  meta <- texanshootR:::open_chain(meta, "RUN_A")
  texanshootR:::write_meta(meta)
  err <- tryCatch(
    texanshootR:::require_chain_stage("abstract", list(run_id = "RUN_B")),
    tx_chain_error = identity
  )
  expect_s3_class(err, "tx_chain_error")
  expect_equal(err$reason, "wrong_run")
})

test_that("require_chain_stage signals wrong_stage when out of order", {
  set_chain_length(2L)
  meta <- texanshootR:::read_meta()
  meta <- texanshootR:::open_chain(meta, "RUN_A")
  texanshootR:::write_meta(meta)
  err <- tryCatch(
    texanshootR:::require_chain_stage("manuscript", list(run_id = "RUN_A")),
    tx_chain_error = identity
  )
  expect_s3_class(err, "tx_chain_error")
  expect_equal(err$reason, "wrong_stage")
})

test_that("require_chain_stage passes when stage matches active chain", {
  set_chain_length(2L)
  meta <- texanshootR:::read_meta()
  meta <- texanshootR:::open_chain(meta, "RUN_A")
  texanshootR:::write_meta(meta)
  res <- texanshootR:::require_chain_stage("abstract",
                                            list(run_id = "RUN_A"))
  expect_true(res$ok)
})

test_that("advance_chain_after_stage moves the chain forward and awards XP", {
  set_chain_length(2L)
  meta <- texanshootR:::read_meta()
  meta <- texanshootR:::open_chain(meta, "RUN_A")
  texanshootR:::write_meta(meta)
  prog_before <- texanshootR:::progression_of(texanshootR:::read_meta())

  texanshootR:::advance_chain_after_stage("abstract")
  meta_after <- texanshootR:::read_meta()
  prog_after <- texanshootR:::progression_of(meta_after)
  expect_gt(prog_after$xp, prog_before$xp)
  expect_equal(meta_after$active_chain$stage_idx, 2L)
})

test_that("completing the unlocked prefix closes the chain", {
  set_chain_length(1L)
  meta <- texanshootR:::read_meta()
  meta <- texanshootR:::open_chain(meta, "RUN_A")
  texanshootR:::write_meta(meta)
  texanshootR:::advance_chain_after_stage("abstract")
  m2 <- texanshootR:::read_meta()
  expect_null(m2$active_chain)
  prog <- texanshootR:::progression_of(m2)
  expect_equal(prog$chains_completed, 1L)
})

test_that("progress() returns a tx_progress overview with chain fields", {
  set_chain_length(2L)
  obj <- expect_invisible(progress())
  expect_s3_class(obj, "tx_progress")
  expect_equal(obj$mode, "overview")
  expect_equal(obj$chain_length, 2L)
  expect_true(is.numeric(obj$xp))
})

# Save schema migrators. Each migrate_<from>_to_<to> function takes a
# parsed meta object at version `<from>` and returns one at `<to>`.
# `migrate_meta` chains them.

migrate_meta <- function(meta) {
  v <- meta$save_version %||% 0L
  while (v < SAVE_VERSION) {
    fn <- get0(sprintf("migrate_%d_to_%d", v, v + 1L), inherits = TRUE)
    if (is.null(fn)) {
      stop("No migrator from save_version ", v, " to ", v + 1L, call. = FALSE)
    }
    meta <- fn(meta)
    v <- v + 1L
  }
  meta$save_version <- SAVE_VERSION
  meta
}

# v1 -> v2: introduces the publication-chain progression. The legacy
# multiplicative career_score in `hidden` no longer drives the tier.
# We seed XP and length_unlocked from the prior career_level so
# existing players keep the tier they had earned.
migrate_1_to_2 <- function(meta) {
  legacy <- meta$career_level %||% "Junior Researcher"
  seed_length <- switch(legacy,
    "Junior Researcher" = 1L,
    "Postdoc"           = 2L,
    "Senior Scientist"  = 4L,
    "PI"                = 6L,
    1L
  )
  seed_xp <- CHAIN_XP_THRESHOLDS[seed_length]
  meta$progression <- list(
    xp                = as.integer(seed_xp),
    length_unlocked   = as.integer(seed_length),
    chains_completed  = 0L,
    chains_broken     = 0L
  )
  meta$active_chain <- NULL
  meta
}

# The shooter -- the package's protagonist.
#
# Every `shoot()` call is one of his runs. His emotional state is the
# spine the rest of the system hangs off: it drives the TUI face, the
# achievement triggers, the print() banner on `tx_run`, and the run
# arc summary recorded on disk. Treat this file as the protagonist's
# state machine, not a rendering helper.
#
# Two layers of animation render that state:
#   1. EMOTIONAL STATE (slow): face changes on `mascot_state` transitions
#      driven by run progress + best p-value. Seven states: composed,
#      uncertain, worried, anxious, panicked, desperate, resolved.
#   2. HEARTBEAT (fast): tiny 8-frame cycle on the gun line, ticked
#      every ~300 ms. The point is to feel alive, not theatrical.
#      Movement is one or two characters, never a full redraw.
#
# Severity ordering (`MASCOT_SEVERITY`) tracks the worst face hit
# during a run; that peak is what survives onto the run record as
# `peak_mascot` and feeds the panic / desperation achievements.
#
# All non-ASCII characters (faces, heartbeat frames) live in
# inst/ascii/ so this source file remains ASCII-only and CRAN-portable.

# -- State machine ----------------------------------------------------

#' Compute the mascot emotional state for a run snapshot.
#'
#' Walks the progress ladder regardless of `best_p` so the player sees
#' the full emotional arc unfold during a run. The `resolved` face is
#' reserved for the final frame after a shippable run -- it isn't
#' returned mid-flight from this function. `best_p` is kept on the
#' signature for back-compat callers but no longer short-circuits the
#' ladder.
#'
#' @param progress Numeric fraction of the run budget used (0 to 1).
#' @param best_p   Numeric smallest p-value found so far in the run,
#'   or `NA` if none. Currently unused; retained for back-compat.
#' @param escalating Logical: is the derived-metric escalation phase
#'   currently active.
#' @return One of `"composed"`, `"uncertain"`, `"worried"`, `"anxious"`,
#'   `"panicked"`, `"desperate"`.
#' @keywords internal
mascot_state <- function(progress, best_p = NA_real_, escalating = FALSE) {
  if (isTRUE(escalating)) return("desperate")
  if (progress >= 0.90) return("desperate")
  if (progress >= 0.75) return("panicked")
  if (progress >= 0.55) return("anxious")
  if (progress >= 0.40) return("worried")
  if (progress >= 0.20) return("uncertain")
  "composed"
}

# Severity ordering used to track the peak emotional state of a run.
# "resolved" is the publishable outcome and isn't an emotional peak,
# so it's not on the ladder.
MASCOT_SEVERITY <- c(composed = 0L, uncertain = 1L, worried = 2L,
                     anxious = 3L, panicked = 4L, desperate = 5L)

mascot_severity <- function(state) {
  v <- MASCOT_SEVERITY[state]
  if (is.na(v)) 0L else as.integer(v)
}

severity_to_state <- function(sev) {
  names(MASCOT_SEVERITY)[match(sev, MASCOT_SEVERITY)] %||% "composed"
}

# -- Heartbeat --------------------------------------------------------

read_heartbeat_frames <- function() {
  if (!is.null(.tx$heartbeat)) return(.tx$heartbeat)
  f <- system.file("ascii", "heartbeat.txt", package = "texanshootR")
  frames <- if (nzchar(f) && file.exists(f)) {
    Filter(nzchar, readLines(f, warn = FALSE))
  } else {
    rep("...", 4L)
  }
  .tx$heartbeat <- frames
  frames
}

# 8-frame heartbeat used by the ANSI multi-zone TUI, where each zone
# (shooter, blip, modifiers, outputs, bar) gets its own row. The mascot
# row's width drift doesn't ripple into adjacent zones, so the full
# 8-frame variation in heartbeat.txt is safe there.
heartbeat_frame <- function(tick) {
  fs <- read_heartbeat_frames()
  fs[(tick %% length(fs)) + 1L]
}

# 2-frame heartbeat used by the dynamic single-line TUI. RStudio Console
# renders U+4E00 (CJK ONE) at a different cell width than `cli::ansi_nchar`
# claims (likely 1 cell because the default RStudio font lacks the glyph
# and falls back). The original 8-frame heartbeat cycles between frames
# that include U+4E00 (6 frames) and frames that don't (`-`, U+2014 em-dash
# substitutes in frames 3 and 6). Each transition between those two
# compositions shifted the mascot slot by 1 cell in RStudio, jittering
# the `|` separators tick-over-tick.
#
# The fix: in dynamic mode, keep the wide-char prefix (gun barrel glyphs
# U+FE3B U+30C7 U+2550 U+4E00) IDENTICAL on every tick, and animate only
# the muzzle dot (U+00B7 toggled with space) at the end. Whatever cell
# width RStudio renders the prefix at, it renders it the same on every
# tick, so the slot stays anchored. U+00B7 and space are both 1 cell in
# every terminal we have seen, so the dot blinking does not change the
# slot width either.
heartbeat_frame_dyn <- function(tick) {
  fs <- read_heartbeat_frames()
  base <- fs[1L]
  if ((as.integer(tick) %% 2L) == 0L) {
    base
  } else {
    sub("\u00b7$", " ", base)  # trailing U+00B7 -> space
  }
}

# Terminal cell widths for the mascot face glyphs and the dynamic-mode
# gun frame. R's `nchar(type = "width")` returns inconsistent values
# for box-drawing (U+2550, U+2014), CJK (U+4E00, U+FE3B, U+30C7), and
# Devanagari / Kannada (U+0CA0) glyphs on Windows, so we use a
# hand-verified lookup instead. The dynamic-mode mascot slot pads to
# DYN_MASCOT_WIDTH cells using these tables.
FACE_CELL_WIDTH <- c(
  composed   = 6L,   # ( o_o)
  uncertain  = 7L,   # ( -_- )
  worried    = 7L,   # ( o_o;)
  anxious    = 7L,   # ( U+2022 _ U+2022 ;)  - bullet renders 1 cell
  panicked   = 6L,   # (>_<;)
  desperate  = 5L,   # (U+0CA0 _ U+0CA0)
  resolved   = 6L    # ( -_-)
)

# Constant width: in dynamic mode the gun is always U+FE3B U+30C7 U+2550
# U+4E00 + 2 spaces + (U+00B7 or space) = 10 cells. Tick is ignored on
# this signature so callers don't need to special-case across modes.
gun_frame_cell_width <- function(tick) 10L

face_cell_width <- function(state) {
  w <- FACE_CELL_WIDTH[state]
  if (is.na(w)) {
    fallback <- tryCatch(nchar(read_face(state), type = "width"),
                          error = function(e) nchar(read_face(state)))
    return(as.integer(fallback))
  }
  as.integer(w)
}

# -- Frame loading ---------------------------------------------------

read_face <- function(state) {
  cache <- .tx$mascot_frames %||% list()
  if (!is.null(cache[[state]])) return(cache[[state]])
  f <- system.file("ascii", "mascot_frames", paste0(state, ".txt"),
                   package = "texanshootR")
  txt <- if (nzchar(f) && file.exists(f)) {
    sub("\\s+$", "", readLines(f, warn = FALSE)[1])
  } else {
    # ASCII-clean fallbacks used during dev when inst/ isn't installed.
    switch(state,
      composed   = "( o_o)",
      uncertain  = "( -_- )",
      worried    = "( o_o;)",
      anxious    = "( o_o;)",
      panicked   = "(>_<;)",
      desperate  = "( O_O )",
      resolved   = "( -_-)",
      polishing  = "( ._. )",
      submitting = "( o_o )?",
      granted    = "( -_- )b",
      rejected   = "( x_x )",
      "( o_o)")
  }
  cache[[state]] <- txt
  .tx$mascot_frames <- cache
  txt
}

read_body_template <- function() {
  if (!is.null(.tx$mascot_body)) return(.tx$mascot_body)
  f <- system.file("ascii", "mascot_frames", "body.txt", package = "texanshootR")
  body <- if (nzchar(f) && file.exists(f)) {
    readLines(f, warn = FALSE)
  } else {
    "{FACE} {GUN}"
  }
  .tx$mascot_body <- body
  body
}

# -- Cosmetic overlays -----------------------------------------------

apply_cosmetics <- function(lines, cosmetics) {
  if (is.null(cosmetics)) return(lines)
  registry <- load_cosmetic_registry()

  # The face line is whichever line contains the rendered face. Locate
  # it by finding a line wider than the bare {FACE} placeholder; in the
  # current minimalist body that's the only line.
  face_idx <- which(grepl("[(<{\\[]", lines, perl = TRUE))[1]
  if (is.na(face_idx)) face_idx <- length(lines)

  hat <- cosmetics$equipped$hat
  if (!is.na(hat)) {
    spec <- registry[registry$id == hat & registry$slot == "hat", ]
    if (nrow(spec) == 1L) {
      lines <- c(spec$overlay[[1]], lines)
      face_idx <- face_idx + length(spec$overlay[[1]])
    }
  }

  badge <- cosmetics$equipped$badge
  if (!is.na(badge)) {
    spec <- registry[registry$id == badge & registry$slot == "badge", ]
    if (nrow(spec) == 1L) {
      lines[face_idx] <- paste0(lines[face_idx], " ", spec$overlay[[1]][1])
    }
  }

  cloak <- cosmetics$equipped$cloak
  if (!is.na(cloak)) {
    spec <- registry[registry$id == cloak & registry$slot == "cloak", ]
    if (nrow(spec) == 1L) {
      lines <- c(lines, spec$overlay[[1]][1])
    }
  }

  poncho <- cosmetics$equipped$poncho
  if (!is.na(poncho)) {
    spec <- registry[registry$id == poncho & registry$slot == "poncho", ]
    if (nrow(spec) == 1L) {
      lines <- c(lines, spec$overlay[[1]][1])
    }
  }

  lanyard <- cosmetics$equipped$lanyard
  if (!is.na(lanyard)) {
    spec <- registry[registry$id == lanyard & registry$slot == "lanyard", ]
    if (nrow(spec) == 1L) {
      lines[face_idx] <- paste0(lines[face_idx], " ", spec$overlay[[1]][1])
    }
  }

  lines
}

# -- Public composer -------------------------------------------------

#' Render the mascot for a given state and heartbeat tick.
#'
#' @param state One of the five mascot states.
#' @param tick  Integer tick counter for the heartbeat animation.
#' @param cosmetics Optional wardrobe state list (slot -> equipped id).
#' @return Character vector of lines.
#' @keywords internal
mascot_render <- function(state, tick = 0L, cosmetics = NULL) {
  body <- read_body_template()
  face <- read_face(state)
  gun  <- heartbeat_frame(tick)
  out  <- vapply(body, function(line) {
    line <- gsub("{FACE}", face, line, fixed = TRUE)
    line <- gsub("{GUN}",  gun,  line, fixed = TRUE)
    line
  }, character(1), USE.NAMES = FALSE)
  apply_cosmetics(out, cosmetics)
}

test_that("mascot_state walks the progress ladder regardless of p", {
  expect_equal(texanshootR:::mascot_state(0.10, NA), "composed")
  expect_equal(texanshootR:::mascot_state(0.30, NA), "uncertain")
  expect_equal(texanshootR:::mascot_state(0.50, NA), "worried")
  expect_equal(texanshootR:::mascot_state(0.65, NA), "anxious")
  expect_equal(texanshootR:::mascot_state(0.80, NA), "panicked")
  expect_equal(texanshootR:::mascot_state(0.99, NA), "desperate")
  # best_p no longer short-circuits the ladder -- "resolved" is now
  # only set at end-of-run by shoot() when the run is shippable.
  expect_equal(texanshootR:::mascot_state(0.99, 0.04), "desperate")
  expect_equal(texanshootR:::mascot_state(0.10, 0.04), "composed")
  expect_equal(texanshootR:::mascot_state(0.10, NA, escalating = TRUE),
               "desperate")
})

test_that("mascot_render produces a face line for every state", {
  for (s in c("composed", "uncertain", "worried", "anxious",
              "panicked", "desperate", "resolved")) {
    out <- texanshootR:::mascot_render(s, tick = 0L)
    expect_type(out, "character")
    expect_gte(length(out), 1L)
    expect_match(out[1], "[(<{\\[]", perl = TRUE)
  }
})

test_that("ANSI-mode heartbeat cycles through frames", {
  # ANSI multi-zone mode gets the full 8-frame heartbeat because each
  # zone has its own row, so the mascot row's width drift cannot ripple
  # into adjacent zones.
  f0 <- texanshootR:::heartbeat_frame(0L)
  f1 <- texanshootR:::heartbeat_frame(1L)
  expect_false(identical(f0, f1))
})

test_that("dynamic-mode heartbeat keeps the wide-char prefix constant", {
  # Dynamic single-line mode toggles only the trailing muzzle dot (`·`
  # vs space), keeping `︻デ═一` identical across ticks. This is
  # because the slot shares a row with loading text and the bar, and
  # any cell-width disagreement between the standard and RStudio's
  # rendering of `一` would jitter the `|` separators tick-over-tick.
  f0 <- texanshootR:::heartbeat_frame_dyn(0L)
  f1 <- texanshootR:::heartbeat_frame_dyn(1L)
  f2 <- texanshootR:::heartbeat_frame_dyn(2L)
  # Both frames must contain the constant prefix `︻デ═一` -- if a
  # future edit reintroduces `一` ↔ `-`/`—` cycling in dyn mode, this
  # catches it.
  prefix <- intToUtf8(c(0xFE3B, 0x30C7, 0x2550, 0x4E00))  # ︻デ═一
  expect_true(grepl(prefix, f0, fixed = TRUE))
  expect_true(grepl(prefix, f1, fixed = TRUE))
  # Same wide-char prefix, only trailing detail varies between ticks.
  expect_false(identical(f0, f1))
  expect_identical(f0, f2)
  expect_identical(nchar(f0), nchar(f1))
})

# Startup banner. Fires with low probability on .onAttach. Suppressed
# in non-interactive sessions, in tests, and when quiet mode is on.

banner_maybe_print <- function() {
  if (runif(1) > 1 / 8) return(invisible())
  flavor <- tryCatch(daily_flavor(), error = function(e) NULL)
  draw <- tryCatch(
    select_message(phase = "banner", tag_boost = flavor$tag_boost),
    error = function(e) NULL
  )
  if (is.null(draw)) return(invisible())
  ver <- tryCatch(as.character(utils::packageVersion("texanshootR")),
                  error = function(e) "0.1.0")
  packageStartupMessage(paste0("texanshootR ", ver,
                                if (!is.null(flavor)) paste0(" - ", flavor$label) else ""))
  packageStartupMessage(draw$text)
  invisible()
}

# Cinematic ANSI animation helpers, used only for the single ultra-rare
# event in v1 ("The Glimpse of Tenure"). Everything is gated behind
# `interactive()` + ANSI capability + `texanshootR.animations` so it
# never fires in tests or under R CMD check.

ultra_rare_glimpse <- function() {
  if (!ansi_supported()) return(invisible())
  frames <- list(
    c("",
      "     A SLIVER OF SOMETHING DEFINITE PASSES THROUGH THE HALLWAY.",
      ""),
    c("",
      "                       *",
      ""),
    c("",
      "                  . A GLIMPSE .",
      ""),
    c("",
      "                  TENURE-ADJACENT.",
      "")
  )
  for (f in frames) {
    for (l in f) cat(l, "\n", sep = "")
    Sys.sleep(0.7)
    cat(ansi_up(length(f)))
    for (l in f) cat(ansi_clear_line(), "\n", sep = "")
    cat(ansi_up(length(f)))
  }
  invisible()
}

#' Generate a manuscript from a run
#'
#' Renders an academic-style write-up of the highlighted specification.
#' If `quarto` and `tinytex` are available, produces a PDF + DOCX; if
#' only `rmarkdown` is available, produces a DOCX. The package never
#' installs LaTeX on its own.
#'
#' Chain stage: \strong{manuscript} (length 2). See [progress()].
#'
#' @param run A `tx_run` object returned by [shoot()].
#' @param output_dir Optional output directory.
#' @param file Optional filename stem (without extension).
#' @param force Overwrite an existing file.
#' @return Character vector of file paths (invisible).
#' @export
manuscript <- function(run, output_dir = NULL, file = NULL,
                       force = FALSE) {
  require_chain_stage("manuscript", run)
  require_pkg("rmarkdown", "manuscript")
  d <- resolve_output_dir(output_dir)
  stem <- file %||% "manuscript_clean"

  rmd_text <- build_manuscript_rmd(run)
  rmd_path <- tempfile(fileext = ".Rmd")
  writeLines(rmd_text, rmd_path)

  out_files <- character()

  docx_path <- versioned_filename(d, stem, "docx", force)
  status_open(docx_path)
  rmarkdown::render(rmd_path, output_format = rmarkdown::word_document(),
                    output_file = basename(docx_path),
                    output_dir  = dirname(docx_path), quiet = TRUE)
  status_close()
  out_files <- c(out_files, docx_path)

  if (requireNamespace("quarto", quietly = TRUE) &&
      requireNamespace("tinytex", quietly = TRUE) &&
      tinytex::is_tinytex()) {
    pdf_path <- versioned_filename(d, stem, "pdf", force)
    status_open(pdf_path)
    rmarkdown::render(rmd_path, output_format = rmarkdown::pdf_document(),
                      output_file = basename(pdf_path),
                      output_dir  = dirname(pdf_path), quiet = TRUE)
    status_close()
    out_files <- c(out_files, pdf_path)
  }

  run$harked <- TRUE
  record_output(run, "manuscript", out_files[1])
  advance_chain_after_stage("manuscript")
  invisible(out_files)
}

build_manuscript_rmd <- function(run) {
  hs <- run$highlighted_spec %||% list()
  methods <- describe_spec(hs)
  paste(c(
    "---",
    "title: 'A Reanalysis with Improved Specification'",
    "author: 'Anonymous'",
    "output: word_document",
    "---",
    "",
    "## Abstract",
    "",
    "We revisit the dataset with attention to specification choice ",
    "and report the relationship that emerges from a specification ",
    "consistent with prior expectations.",
    "",
    "## Methods",
    "",
    sprintf("We considered %d candidate specifications across predictor ",
            run$spec_count %||% 0L),
    "subsets, transformations, interaction terms, sample restrictions, ",
    "and outcome-construction alternatives.",
    "",
    methods,
    "",
    "The reported specification was selected for its consistency with ",
    "the conceptual framework.",
    "",
    "## Results",
    "",
    sprintf("The reported model recovered R-squared %.3f and p = %.4f.",
            hs$r_squared %||% 0,
            hs$p_value   %||% 1),
    "",
    "## Discussion",
    "",
    "The result suggests a pathway consistent with prior ",
    "ecological reasoning. We controlled for what was available."
  ), collapse = "\n")
}

# Engine: dispatch a spec to its family fitter and return a tidy
# result row. The actual fitters live in R/families.R; this file
# stays as the small dispatch + highlight-selection surface.
#
# v1 supports lm (.lm.fit) and glm (glm.fit). GAM (mgcv) and GLMM
# (lme4) plug into FAMILIES as separate fitters in later phases.

fit_spec <- function(df, outcome, spec) {
  fam_name <- spec$family$fitter %||% "lm"
  fitter   <- tryCatch(family_fitter(fam_name), error = function(e) NULL)
  if (is.null(fitter)) return(NULL)
  fitter(df, outcome, spec)
}

# Batch dispatcher: fits a list of specs in one pass. Groups specs by
# family, then delegates each group to the family's batch_fitter from
# the FAMILIES registry (see R/families.R). lm goes through the C++
# kernel today; every other family currently loops the single-spec
# fitter under the same scaffold, so a future *_fit_batch_cpp() drops
# in by rewriting one fit_*_batch wrapper -- no change here.
#
# Returns results in the same order as `specs`, with NULLs preserved
# for fits that fail to materialise. The shoot() loop calls this in
# mini-batches so the search state still gets per-fit feedback inside
# record_result(), just at coarser cadence than one-spec-at-a-time.
fit_specs_batch <- function(df, outcome, specs) {
  if (!length(specs)) return(list())
  fam <- vapply(specs,
                function(s) s$family$fitter %||% "lm",
                character(1))
  out <- vector("list", length(specs))

  for (this_fam in unique(fam)) {
    idx <- which(fam == this_fam)
    batch_fn <- tryCatch(family_batch_fitter(this_fam),
                         error = function(e) NULL)
    # Unknown / unregistered families fall back to per-spec fit_spec()
    # rather than aborting the whole batch, matching the prior
    # behaviour of the non-lm leg.
    if (is.null(batch_fn)) {
      for (i in idx) out[i] <- list(fit_spec(df, outcome, specs[[i]]))
    } else {
      # `[<-` with list(...) preserves NULL slots so the result list
      # stays length(specs); `[[<-` with NULL would delete the slot
      # and desynchronise the caller's per-spec index.
      out[idx] <- batch_fn(df, outcome, specs[idx])
    }
  }
  out
}

# Highlighted-spec selection. Implements the just-cleared bias from
# Brodeur, Cook & Heyes (2020): prefer specs whose p-value falls in
# [0.040, 0.0499] - distinctly under the 0.05 line but visibly close.
# Break ties to land closest to 0.045. Among ties, take the highest R^2.
choose_highlight <- function(results) {
  results <- Filter(Negate(is.null), results)
  if (length(results) == 0L) return(NULL)
  ps <- vapply(results, function(r) r$p_value %||% NA_real_, numeric(1))
  r2 <- vapply(results, function(r) r$r_squared %||% NA_real_, numeric(1))

  in_strip <- !is.na(ps) & ps >= 0.040 & ps < 0.050
  if (any(in_strip)) {
    cand <- which(in_strip)
    dist <- abs(ps[cand] - 0.045)
    cand <- cand[order(dist, -r2[cand])]
    return(results[[cand[1]]])
  }
  # No just-cleared strip: take the smallest p with valid value.
  ord <- order(ps, na.last = TRUE)
  results[[ord[1]]]
}

# Save / load helpers. Hybrid storage:
#   meta.json              human-readable career meta + hidden modifiers
#   runs/<run_id>.rds      lightweight run records (no raw data frames)
#   achievements.rds       award state
#   wardrobe.rds           equipped + unlocked cosmetics
#   recent_messages.rds    ring buffer for message recency
#   sightings.rds          ultra-rare event log
#
# All writes go through `save_dir()`, which respects
# `options(texanshootR.save_dir)` (test override) and
# `options(texanshootR.save_enabled)` (global opt-out).

SAVE_VERSION <- 2L

# Resolved save directory. NULL = persistence disabled.
save_dir <- function() {
  if (!isTRUE(opt("texanshootR.save_enabled"))) return(NULL)
  override <- getOption("texanshootR.save_dir", NULL)
  if (!is.null(override)) return(override)
  tools::R_user_dir("texanshootR", which = "data")
}

# Ensure save dir exists with consent.
ensure_save_dir <- function() {
  d <- save_dir()
  if (is.null(d)) return(NULL)
  if (!dir.exists(d)) {
    if (!consent_ok(d)) return(NULL)
    dir.create(file.path(d, "runs"), recursive = TRUE, showWarnings = FALSE)
    write_meta(default_meta(d))
  } else {
    if (!file.exists(file.path(d, "meta.json"))) {
      write_meta(default_meta(d))
    }
    if (!dir.exists(file.path(d, "runs"))) {
      dir.create(file.path(d, "runs"), showWarnings = FALSE)
    }
  }
  d
}

default_meta <- function(d) {
  list(
    save_version = SAVE_VERSION,
    career_id    = paste0("career-", format(Sys.time(), "%Y%m%d-%H%M%S")),
    consent      = TRUE,
    consent_path = d,
    created_at   = format(Sys.time(), "%Y-%m-%dT%H:%M:%S%z"),
    last_seen    = format(Sys.time(), "%Y-%m-%dT%H:%M:%S%z"),
    career_level = "Junior Researcher",
    runs_count   = 0L,
    # Publication-chain progression. XP buys chain length; tier is
    # derived from length, not from the legacy multiplicative score.
    progression = list(
      xp                = 0L,
      length_unlocked   = 1L,
      chains_completed  = 0L,
      chains_broken     = 0L
    ),
    # NULL except while a chain is mid-redemption.
    active_chain = NULL,
    # Hidden side-channel components, kept for achievement triggers.
    # Career level no longer reads from these.
    hidden = list(
      models_fit            = 0,
      output_complexity     = 0,
      reviewer_resistance   = 0,
      narrative_strength    = 0,
      h_index_proxy         = 0,
      presentation_quality  = 0
    ),
    favorite_method     = NA_character_,
    most_removed_var    = NA_character_,
    events_witnessed    = list()
  )
}

read_meta <- function() {
  d <- save_dir()
  if (is.null(d)) return(NULL)
  f <- file.path(d, "meta.json")
  if (!file.exists(f)) return(NULL)
  meta <- jsonlite::read_json(f, simplifyVector = TRUE,
                              simplifyDataFrame = FALSE,
                              simplifyMatrix = FALSE)
  meta <- migrate_meta(meta)
  meta
}

write_meta <- function(meta) {
  d <- save_dir()
  if (is.null(d)) return(invisible(NULL))
  if (!dir.exists(d)) dir.create(d, recursive = TRUE, showWarnings = FALSE)
  meta$last_seen <- format(Sys.time(), "%Y-%m-%dT%H:%M:%S%z")
  jsonlite::write_json(
    meta,
    path = file.path(d, "meta.json"),
    auto_unbox = TRUE,
    pretty = TRUE,
    null = "null"
  )
  invisible(meta)
}

write_run_record <- function(run) {
  d <- ensure_save_dir()
  if (is.null(d)) return(invisible(NULL))
  rec <- run_to_record(run)  # privacy-stripped
  saveRDS(rec, file.path(d, "runs", paste0(rec$run_id, ".rds")))
  invisible(rec)
}

list_run_records <- function() {
  d <- save_dir()
  if (is.null(d) || !dir.exists(file.path(d, "runs"))) return(character())
  list.files(file.path(d, "runs"), pattern = "\\.rds$", full.names = TRUE)
}

read_run_record <- function(run_id) {
  d <- save_dir()
  if (is.null(d)) return(NULL)
  f <- file.path(d, "runs", paste0(run_id, ".rds"))
  if (!file.exists(f)) return(NULL)
  readRDS(f)
}

# Convert a tx_run object to a privacy-stripped record. We keep
# variable *names* and *summary stats*, never the raw data frame.
# Achievement-relevant flags are preserved so the record can be
# re-evaluated when output generators run later.
run_to_record <- function(run) {
  list(
    run_id            = run$run_id,
    run_index         = as.integer(run$run_index %||% 0L),
    promoted          = isTRUE(run$promoted),
    seed              = run$seed,
    timestamp         = run$timestamp,
    package_version   = run$package_version,
    df_meta           = run$df_meta,
    formula           = run$formula,
    spec_count        = run$spec_count,
    highlighted_spec  = run$highlighted_spec,
    derived_used      = run$derived_used,
    reviewer_outcome  = run$reviewer_outcome,
    events            = run$events,
    modifiers         = run$modifiers,
    modifiers_used    = run$modifiers_used %||% character(),
    shippable         = isTRUE(run$shippable),
    displayed_message_ids = run$displayed_message_ids,
    grid_hash         = run$grid_hash,
    search            = run$search,
    stopped_early     = run$stopped_early %||% FALSE,
    resolved_at_progress = run$resolved_at_progress,
    ultra_rare_seen   = isTRUE(run$ultra_rare_seen),
    peak_mascot       = run$peak_mascot %||% "composed",
    peak_mascot_before_resolve =
      run$peak_mascot_before_resolve %||% "composed",
    harked            = isTRUE(run$harked),
    outputs_generated = run$outputs_generated %||% character(),
    outputs_generated_files = run$outputs_generated_files %||% character(),
    achievements_awarded    = run$achievements_awarded %||% character()
  )
}

read_recent_buffer <- function() {
  d <- save_dir()
  if (is.null(d)) return(recent_buffer())
  f <- file.path(d, "recent_messages.rds")
  if (!file.exists(f)) return(recent_buffer())
  readRDS(f)
}

write_recent_buffer <- function(rb) {
  d <- ensure_save_dir()
  if (is.null(d)) return(invisible(NULL))
  saveRDS(rb, file.path(d, "recent_messages.rds"))
  invisible(rb)
}

read_achievements_state <- function() {
  d <- save_dir()
  if (is.null(d)) return(list())
  f <- file.path(d, "achievements.rds")
  if (!file.exists(f)) return(list())
  readRDS(f)
}

write_achievements_state <- function(state) {
  d <- ensure_save_dir()
  if (is.null(d)) return(invisible(NULL))
  saveRDS(state, file.path(d, "achievements.rds"))
  invisible(state)
}

read_wardrobe_state <- function() {
  d <- save_dir()
  if (is.null(d)) return(default_wardrobe())
  f <- file.path(d, "wardrobe.rds")
  if (!file.exists(f)) return(default_wardrobe())
  readRDS(f)
}

write_wardrobe_state <- function(state) {
  d <- ensure_save_dir()
  if (is.null(d)) return(invisible(NULL))
  saveRDS(state, file.path(d, "wardrobe.rds"))
  invisible(state)
}

default_wardrobe <- function() {
  list(
    equipped = list(hat = NA, poncho = NA, badge = NA, lanyard = NA, cloak = NA),
    unlocked = character()
  )
}

read_sightings <- function() {
  d <- save_dir()
  if (is.null(d)) return(list())
  f <- file.path(d, "sightings.rds")
  if (!file.exists(f)) return(list())
  readRDS(f)
}

append_sighting <- function(s) {
  d <- ensure_save_dir()
  if (is.null(d)) return(invisible(NULL))
  cur <- read_sightings()
  cur[[length(cur) + 1L]] <- s
  saveRDS(cur, file.path(d, "sightings.rds"))
  invisible(cur)
}

# Reviewer-outcome materialization. The roll moved from finalize_run()
# (where it fired automatically on every run) into reviewer_response()
# (where it only fires when the player turns the finding into
# publication bureaucracy). A run is a finding, not a verdict.

test_that("shoot() leaves reviewer_outcome NA", {
  withr::local_options(texanshootR.budget = 1)
  withr::local_seed(11)
  run <- shoot(mtcars)
  expect_true(is.na(run$reviewer_outcome))
})

test_that("materialize_reviewer_outcome rolls and persists on first call", {
  withr::local_options(texanshootR.budget = 1)
  withr::local_seed(17)
  run <- shoot(mtcars)
  expect_true(is.na(run$reviewer_outcome))

  rolled <- texanshootR:::materialize_reviewer_outcome(run)
  expect_true(rolled$reviewer_outcome %in%
              c("accept_enthusiasm", "minor", "major", "reject"))

  # On-disk record was updated.
  rec <- texanshootR:::read_run_record(run$run_id)
  expect_identical(rec$reviewer_outcome, rolled$reviewer_outcome)
})

test_that("materialize_reviewer_outcome memoizes -- no re-roll on second call", {
  withr::local_options(texanshootR.budget = 1)
  withr::local_seed(23)
  run <- shoot(mtcars)

  first  <- texanshootR:::materialize_reviewer_outcome(run)
  # Seed shifted: if the function re-rolled it would draw a different
  # outcome with high probability across the four-element distribution.
  set.seed(99)
  second <- texanshootR:::materialize_reviewer_outcome(first)
  expect_identical(second$reviewer_outcome, first$reviewer_outcome)
})

test_that("received_a_reject fires from reviewer_response, not from shoot", {
  testthat::skip_if_not_installed("withr")
  testthat::skip_if_not_installed("officer")
  withr::local_options(texanshootR.budget = 2)
  withr::local_seed(5)

  # Wipe achievement state so we can observe the trigger fire.
  ach_state <- texanshootR:::read_achievements_state()
  ach_state$received_a_reject <- NULL
  texanshootR:::write_achievements_state(ach_state)

  run <- shoot(mtcars)
  expect_true(is.na(run$reviewer_outcome))
  # No reject achievement yet — the roll hasn't happened.
  expect_null(texanshootR:::read_achievements_state()$received_a_reject)

  # Force a deterministic reject outcome and call the generator.
  # We can't easily seed reviewer_roll() externally, so inject the
  # outcome onto the run directly: materialize_reviewer_outcome()
  # short-circuits on a non-NA value, and reviewer_response()'s
  # record_output() call then evaluates achievements against the
  # persisted record.
  if (isTRUE(run$shippable)) {
    run$reviewer_outcome <- "reject"
    # Persist so record_output() sees it.
    fpath <- file.path(texanshootR:::save_dir(), "runs",
                       paste0(run$run_id, ".rds"))
    rec <- readRDS(fpath)
    rec$reviewer_outcome <- "reject"
    saveRDS(rec, fpath)

    tryCatch(
      reviewer_response(run, output_dir = tempdir()),
      tx_chain_error = function(e) NULL,
      tx_locked      = function(e) NULL
    )
    # Achievement state advances only when the chain stage actually
    # ran. If the chain wasn't unlocked, the test trivially passes
    # because the trigger had no run to fire on.
    state <- texanshootR:::read_achievements_state()
    if (!is.null(state$received_a_reject)) {
      expect_true(state$received_a_reject)
    } else {
      succeed("chain not unlocked at this career tier; trigger had no opportunity")
    }
  } else {
    succeed("run was not shippable; reviewer chain stage unreachable")
  }
})

test_that("abstract() refuses when no chain is active", {
  meta <- texanshootR:::read_meta() %||% texanshootR:::default_meta(getOption("texanshootR.save_dir"))
  meta$active_chain <- NULL
  texanshootR:::write_meta(meta)
  fake_run <- list(run_id = "TEST",
                   highlighted_spec = list(formula = "y ~ x",
                                            r_squared = 0.5,
                                            p_value = 0.04),
                   spec_count = 10L)
  err <- tryCatch(abstract(fake_run), tx_chain_error = identity)
  expect_s3_class(err, "tx_chain_error")
  expect_equal(err$reason, "no_active_chain")
})

test_that("abstract() writes a .txt and advances the chain when stage matches", {
  meta <- texanshootR:::read_meta() %||% texanshootR:::default_meta(getOption("texanshootR.save_dir"))
  meta$progression$length_unlocked <- 2L
  meta$progression$xp <- texanshootR:::CHAIN_XP_THRESHOLDS[2L]
  meta <- texanshootR:::open_chain(meta, "TEST_RUN")
  texanshootR:::write_meta(meta)

  # Need a run record for record_output() to update.
  rec <- list(run_id = "TEST_RUN", outputs_generated = character(),
              outputs_generated_files = character())
  d <- texanshootR:::ensure_save_dir()
  saveRDS(rec, file.path(d, "runs", "TEST_RUN.rds"))

  out_dir <- tempfile("abs-out-")
  fake_run <- list(run_id = "TEST_RUN",
                   highlighted_spec = list(formula = "mpg ~ wt",
                                            r_squared = 0.75,
                                            p_value = 0.001),
                   spec_count = 42L)
  path <- abstract(fake_run, output_dir = out_dir, force = TRUE)
  expect_true(file.exists(path))
  txt <- readLines(path)
  expect_true(any(grepl("Abstract", txt)))

  meta2 <- texanshootR:::read_meta()
  # Length-2 chain: abstract landed, manuscript is now due.
  expect_equal(meta2$active_chain$stage_idx, 2L)
  expect_true("abstract" %in% meta2$active_chain$completed_stages)
})

# Sectioned print.tx_run banner: header rule, RUN/FINDING/SEARCH/OUTPUTS/
# DATA USED sections always present, PUBLICATION PIPELINE only when the
# printed run owns the active chain.

capture_banner <- function(run) {
  utils::capture.output(print(run))
}

# Each test starts from a fresh save dir so earlier tests in the suite
# can't promote the player past the tier assertions below (notably the
# OUTPUTS test, which requires at least one locked stage to remain).
local_fresh_save_dir <- function(env = parent.frame()) {
  withr::local_options(texanshootR.save_dir = withr::local_tempdir(.local_envir = env),
                       .local_envir = env)
}

test_that("banner header carries the run number, zero-padded to four", {
  local_fresh_save_dir()
  withr::local_options(texanshootR.budget = 1)
  withr::local_seed(3)
  run <- shoot(mtcars)
  out <- capture_banner(run)
  hit <- grep("^texanshootR :: run \\d{4}$", out, value = TRUE)
  expect_length(hit, 1L)
})

test_that("the five always-on sections render in order", {
  local_fresh_save_dir()
  withr::local_options(texanshootR.budget = 1)
  withr::local_seed(5)
  run <- shoot(mtcars)
  out <- capture_banner(run)
  fixed_sections <- c("RUN", "FINDING", "SEARCH", "OUTPUTS", "DATA USED")
  positions <- vapply(fixed_sections,
                      function(s) which(out == s)[1],
                      integer(1))
  expect_false(any(is.na(positions)))
  expect_equal(positions, sort(positions))
})

test_that("RUN section reports status, career impact, mascot arc", {
  local_fresh_save_dir()
  withr::local_options(texanshootR.budget = 1)
  withr::local_seed(7)
  run <- shoot(mtcars)
  out <- capture_banner(run)
  expect_true(any(grepl("^status: ", out)))
  expect_true(any(grepl("^career impact: (promoted|advanced|stalled)$", out)))
  expect_true(any(grepl("^mascot state: .+ → .+$", out)))
})

test_that("OUTPUTS section uses `fn(run)` for unlocked, bare for locked", {
  local_fresh_save_dir()
  withr::local_options(texanshootR.budget = 1)
  withr::local_seed(9)
  run <- shoot(mtcars)
  out <- capture_banner(run)
  # At minimum the Junior Researcher tier has abstract unlocked.
  expect_true(any(grepl("^available now: abstract\\(run\\)$", out)))
  # Locked rows read "fn()  requires <tier>".
  expect_true(any(grepl("\\(\\) +requires (Postdoc|Senior Scientist|PI)$", out)))
})

test_that("DATA USED always reports the six labeled fields", {
  local_fresh_save_dir()
  withr::local_options(texanshootR.budget = 1)
  withr::local_seed(13)
  run <- shoot(mtcars)
  out <- capture_banner(run)
  for (label in c("exclusion rule:", "rows excluded:",
                  "outcome construction:",
                  "derived metrics:", "search seed:", "run stored:")) {
    expect_true(any(grepl(label, out, fixed = TRUE)),
                info = sprintf("missing DATA USED label: %s", label))
  }
})

test_that("PUBLICATION PIPELINE appears only when run owns active chain", {
  local_fresh_save_dir()
  withr::local_options(texanshootR.budget = 1)
  withr::local_seed(17)
  run <- shoot(mtcars)
  out <- capture_banner(run)

  meta <- texanshootR:::read_meta()
  owns_chain <- !is.null(meta) && !is.null(meta$active_chain) &&
                identical(meta$active_chain$run_id, run$run_id)
  if (isTRUE(owns_chain)) {
    expect_true(any(out == "PUBLICATION PIPELINE"))
  } else {
    expect_false(any(out == "PUBLICATION PIPELINE"))
  }
})

test_that("save dir is sandboxed via texanshootR.save_dir", {
  d <- getOption("texanshootR.save_dir")
  expect_true(!is.null(d))
  expect_equal(texanshootR:::save_dir(), d)
})

test_that("default meta + write/read round trip", {
  d <- texanshootR:::ensure_save_dir()
  meta <- texanshootR:::read_meta()
  expect_true(!is.null(meta))
  expect_equal(meta$save_version, texanshootR:::SAVE_VERSION)
})

test_that("reset_all wipes the save dir", {
  texanshootR:::ensure_save_dir()
  reset_all(force = TRUE)
  expect_null(texanshootR:::read_meta())
})

#' Inspect the achievement registry
#'
#' Returns a data.frame of all achievements with unlock status. Hidden
#' achievements appear with `name = "???"` and `hint = NA` until
#' unlocked.
#'
#' @return A data.frame.
#' @export
achievements <- function() {
  reg <- load_achievement_registry()
  state <- read_achievements_state()
  unlocked <- vapply(reg$id, function(id) isTRUE(state[[id]]), logical(1))
  display_name <- ifelse(unlocked | reg$visible, reg$name, "???")
  display_hint <- ifelse(unlocked | reg$visible, reg$hint, NA_character_)
  out <- data.frame(
    id         = reg$id,
    name       = display_name,
    visible    = reg$visible,
    unlocked   = unlocked,
    hint       = display_hint,
    stringsAsFactors = FALSE
  )
  out
}

# Lazy-loaded achievement registry. Combines all YAMLs under
# inst/achievements/.
load_achievement_registry <- function(force = FALSE) {
  load_registry(
    inst_dir   = "achievements",
    cache_key  = "achievements",
    entry_to_row = function(e) list(
      id           = as.character(e$id),
      name         = as.character(e$name),
      visible      = isTRUE(e$visible),
      hint         = if (is.null(e$hint) || is.na(e$hint)) NA_character_
                     else as.character(e$hint),
      reveal_text  = as.character(e$reveal_text %||% e$name),
      trigger      = as.character(e$trigger %||% NA_character_),
      cosmetic     = as.character(e$cosmetic %||% NA_character_)
    ),
    rows_to_frame = function(rows) data.frame(
      id          = vapply(rows, `[[`, "", "id"),
      name        = vapply(rows, `[[`, "", "name"),
      visible     = vapply(rows, `[[`, FALSE, "visible"),
      hint        = vapply(rows, `[[`, NA_character_, "hint"),
      reveal_text = vapply(rows, `[[`, "", "reveal_text"),
      trigger     = vapply(rows, `[[`, NA_character_, "trigger"),
      cosmetic    = vapply(rows, `[[`, NA_character_, "cosmetic"),
      stringsAsFactors = FALSE
    ),
    empty = empty_achievement_registry,
    force = force
  )
}

empty_achievement_registry <- function() {
  data.frame(
    id = character(), name = character(), visible = logical(),
    hint = character(), reveal_text = character(), trigger = character(),
    cosmetic = character(), stringsAsFactors = FALSE
  )
}

# Evaluate every trigger against a run + career snapshot. Returns a
# character vector of newly-awarded ids.
evaluate_achievements <- function(run, career_meta) {
  reg <- load_achievement_registry()
  state <- read_achievements_state()
  newly <- character()
  for (i in seq_len(nrow(reg))) {
    id <- reg$id[i]
    if (isTRUE(state[[id]])) next
    fn <- get0(reg$trigger[i], inherits = TRUE)
    if (is.null(fn)) next
    fired <- tryCatch(isTRUE(fn(run, career_meta)),
                      error = function(e) FALSE)
    if (fired) {
      state[[id]] <- TRUE
      newly <- c(newly, id)
    }
  }
  if (length(newly)) write_achievements_state(state)
  newly
}

# Run the evaluator and auto-equip cosmetics for any newly-awarded
# achievements. Returns the awarded ids.
award_and_equip <- function(run, career_meta) {
  awarded <- evaluate_achievements(run, career_meta)
  if (length(awarded)) {
    reg <- load_achievement_registry()
    for (id in awarded) {
      cid <- reg$cosmetic[reg$id == id]
      if (length(cid) && !is.na(cid) && nzchar(cid)) auto_equip(cid)
    }
  }
  awarded
}

# -- Trigger functions ------------------------------------------------
#
# Each function takes (run, career_meta) and returns TRUE on unlock.
# These are the v1 priority subset; expand as new achievements ship.

trigger_first_run <- function(run, meta) {
  (meta$runs_count %||% 0L) >= 1L
}

trigger_aim_after_data <- function(run, meta) {
  # "Aimed after seeing the data" - Texas sharpshooter. Triggers on
  # any successful run with >= 1 highlighted spec.
  !is.null(run$highlighted_spec)
}

trigger_p_hacked <- function(run, meta) {
  # Highlighted p in the just-cleared [0.040, 0.0499] strip.
  p <- run$highlighted_spec$p_value %||% NA_real_
  is.finite(p) && p >= 0.040 && p < 0.050
}

trigger_harker <- function(run, meta) {
  # HARKing: a manuscript was generated for this run with no a priori
  # hypothesis recorded. Run flags `harked` when manuscript() is called.
  isTRUE(run$harked)
}

trigger_multiple_comparisons <- function(run, meta) {
  (run$spec_count %||% 0L) >= 1000L
}

trigger_subgroup_fisher <- function(run, meta) {
  r <- run$highlighted_spec$restriction
  !is.null(r) && identical(r$kind, "factor_level")
}

trigger_var_purge <- function(run, meta) {
  length(run$highlighted_spec$dropped %||% character()) >= 5L
}

trigger_outlier_excluder <- function(run, meta) {
  isTRUE((run$highlighted_spec$rows_excluded %||% 0L) > 0L)
}

trigger_derived_insight <- function(run, meta) {
  # Hidden achievement: first time the derived-metric escalation
  # produces a p <= 0.05 highlight.
  isTRUE(run$derived_used) &&
    is.finite(run$highlighted_spec$p_value %||% NA_real_) &&
    run$highlighted_spec$p_value <= 0.05
}

trigger_overfitter <- function(run, meta) {
  isTRUE((run$highlighted_spec$r_squared %||% 0) >= 0.99) &&
    (run$highlighted_spec$n_terms %||% 0L) >= 6L
}

trigger_publication_pipeline <- function(run, meta) {
  # Publication bias proxy: career has produced >= 3 outputs.
  (meta$hidden$output_complexity %||% 0) >= 3
}

trigger_late_resolver <- function(run, meta) {
  isTRUE(run$resolved_at_progress >= 0.90)
}

trigger_event_survivor <- function(run, meta) {
  length(meta$events_witnessed %||% list()) >= 3L
}

trigger_reviewer_repeller <- function(run, meta) {
  identical(run$reviewer_outcome, "reject")
}

trigger_glimpse <- function(run, meta) {
  isTRUE(run$ultra_rare_seen)
}

trigger_optional_stopping <- function(run, meta) {
  isTRUE(run$stopped_early)
}

trigger_all_outputs <- function(run, meta) {
  required <- c("manuscript", "reviewer_response",
                "graphical_abstract", "funding", "presentation")
  outs <- run$outputs_generated %||% character()
  all(required %in% outs)
}

trigger_filename_archaeologist <- function(run, meta) {
  any(grepl("figure_final_v2_REAL", run$outputs_generated_files %||% character(),
            fixed = TRUE))
}

trigger_visible_panic <- function(run, meta) {
  # Peak emotional state of "panicked" or worse at any point in the run.
  sev <- mascot_severity(run$peak_mascot %||% "composed")
  sev >= mascot_severity("panicked")
}

trigger_begging_the_wall <- function(run, meta) {
  # Peak hit "desperate" — either ran out the clock or escalated.
  sev <- mascot_severity(run$peak_mascot %||% "composed")
  sev >= mascot_severity("desperate")
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

ols_fit_cpp <- function(X, y) {
    .Call(`_texanshootR_ols_fit_cpp`, X, y)
}

ols_fit_batch_cpp <- function(Xs, ys) {
    .Call(`_texanshootR_ols_fit_batch_cpp`, Xs, ys)
}

pls_gcv_cpp <- function(X, y, S, log_lambdas) {
    .Call(`_texanshootR_pls_gcv_cpp`, X, y, S, log_lambdas)
}

lmm_profile_cpp <- function(X, y, group, n_groups, log_thetas) {
    .Call(`_texanshootR_lmm_profile_cpp`, X, y, group, n_groups, log_thetas)
}

utils::globalVariables(c(
  ".tx",
  "id",
  "rarity",
  "trigger_phase",
  "career_min",
  "tags",
  "model_family_affinity",
  "mascot_state_affinity",
  "combo_chain",
  "text",
  "weight",
  "weight_recency",
  "weight_combo",
  "weight_final",
  # ggplot2 aesthetic columns referenced inside aes()
  "x",
  "y",
  # Rcpp-generated wrappers (R/RcppExports.R) — referenced from
  # R/families.R via bare names through devtools::load_all().
  "pls_gcv_cpp",
  "lmm_profile_cpp",
  "ols_fit_cpp",
  "ols_fit_batch_cpp"
))

# Pure family selector. Decides which model family to attach to a
# spec before it is fit, based on:
#   - career_level     (gates which fitters are available at all)
#   - escalating       (desperation phase => glm bias + occasional flail)
#   - outcome vector   (compatibility checks: binary -> binomial, etc.)
#   - available pool   (output of available_families(career_level))
#   - RNG              (called inside set.seed-controlled paths)
#
# Returns a list ready to assign to spec$family, of one of these shapes:
#   list(fitter = "lm")
#   list(fitter = "glm", family = "binomial", link = "logit")
#   list(fitter = "glm", family = "binomial", link = "logit",
#        outcome_coerce = "median_dichotomize")
#
# The selector is the joke: at low career it sticks to lm. As tiers
# rise it leans on glm more often. Under desperation it preferentially
# reaches for glm, and a small fraction of the time it picks a family
# that doesn't match the outcome and forces a coercion -- the shooter
# trying logistic regression on a continuous outcome to chase the
# next-decimal p-value.

# Per-tier probability of upgrading from lm to glm during normal search.
# Junior is locked to lm regardless. PI defaults to glm only ~half the
# time -- the search still wants lm in the pool because lm's F-test
# strip is where the just-cleared bias does its work.
GLM_PROB_BY_TIER <- c(
  "Junior Researcher" = 0.00,
  "Postdoc"           = 0.15,
  "Senior Scientist"  = 0.35,
  "PI"                = 0.55
)

# Bias when escalating: any tier that has an upgrade family available
# leans on it heavily. The mismatched-flail roll happens within this
# branch and only applies to glm picks (gam/glmm don't carry the
# coercion knob).
ESCALATION_GLM_PROB    <- 0.85
ESCALATION_FLAIL_PROB  <- 0.20

# Within the "upgrade beyond lm" branch, weights for each upgrade
# family. glm dominates because it's the most often-applicable; gam,
# glmm, and sem get meaningful air time at the tiers where they're
# unlocked. cor (bare-correlation give-up move) and wls
# (heteroscedasticity flex) sit alongside glm in availability but with
# smaller weight so glm stays the default upgrade.
UPGRADE_WEIGHTS <- c(glm = 1.0, cor = 0.3, wls = 0.5,
                      gam = 0.6, glmm = 0.4, sem = 0.3)

select_family <- function(career_level,
                          escalating = FALSE,
                          outcome    = NULL,
                          available  = NULL) {
  if (is.null(available)) available <- available_families(career_level)
  upgrades <- intersect(names(UPGRADE_WEIGHTS), available)
  if (length(upgrades) == 0L) return(list(fitter = "lm"))

  prob <- if (isTRUE(escalating)) ESCALATION_GLM_PROB
          else GLM_PROB_BY_TIER[[career_level]] %||% 0
  if (stats::runif(1) >= prob) return(list(fitter = "lm"))

  pick <- if (length(upgrades) == 1L) upgrades
          else sample(upgrades, size = 1L,
                      prob = UPGRADE_WEIGHTS[upgrades])

  outcome_kind <- classify_outcome(outcome)

  if (pick == "glm") {
    flail <- isTRUE(escalating) && stats::runif(1) < ESCALATION_FLAIL_PROB
    return(pick_glm_family(outcome_kind, flail = flail))
  }
  if (pick == "gam")  return(pick_gam_family(outcome_kind))
  if (pick == "glmm") return(pick_glmm_family(outcome_kind))
  if (pick == "sem")  return(pick_sem_family())
  if (pick == "cor")  return(pick_cor_family())
  if (pick == "wls")  return(pick_wls_family())

  list(fitter = "lm")
}

# Categorise the outcome vector for family compatibility:
#   "binary"     - all values in {0, 1}
#   "count"      - non-negative integers
#   "positive"   - strictly positive numeric (Gamma candidate)
#   "continuous" - everything else numeric
#   "unknown"    - non-numeric / NULL
classify_outcome <- function(y) {
  if (is.null(y) || !is.numeric(y)) return("unknown")
  y <- y[is.finite(y)]
  if (length(y) == 0L) return("unknown")
  if (all(y %in% c(0, 1))) return("binary")
  if (all(y >= 0) && all(abs(y - round(y)) < 1e-8)) return("count")
  if (all(y > 0)) return("positive")
  "continuous"
}

# Choose a glm family/link based on outcome class. When flail = TRUE,
# pick a family that does NOT match the outcome and force a coercion
# so glm.fit() will accept it -- the desperation tell.
pick_glm_family <- function(outcome_kind, flail = FALSE) {
  if (isTRUE(flail) && outcome_kind %in% c("continuous", "positive")) {
    # The two flails: dichotomise for binomial, round-and-clip for
    # poisson. Either reads as the shooter contorting the data.
    coerce <- sample(c("median_dichotomize", "round_clip"), 1L)
    if (coerce == "median_dichotomize") {
      return(list(fitter = "glm", family = "binomial", link = "logit",
                  outcome_coerce = "median_dichotomize"))
    }
    return(list(fitter = "glm", family = "poisson", link = "log",
                outcome_coerce = "round_clip"))
  }

  switch(outcome_kind,
    binary     = list(fitter = "glm", family = "binomial",
                      link   = sample(c("logit", "probit"), 1L)),
    count      = list(fitter = "glm", family = "poisson", link = "log"),
    positive   = if (stats::runif(1) < 0.5)
                    list(fitter = "glm", family = "Gamma",
                         link   = sample(c("inverse", "log"), 1L))
                 else
                    list(fitter = "glm", family = "gaussian",
                         link   = sample(c("identity", "log"), 1L)),
    continuous = list(fitter = "glm", family = "gaussian",
                      link   = sample(c("identity", "log"), 1L)),
    # outcome unknown: fall back to gaussian-identity, which is just
    # lm in disguise but keeps the family label visible.
    list(fitter = "glm", family = "gaussian", link = "identity")
  )
}

# GAM and GLMM family pickers. Both are deterministic by outcome kind:
# the smooth-vs-parametric choice (gam) and the random-effect choice
# (glmm) carry the "fancier model" punch, so the family/link stays
# canonical instead of doubling up on the desperation knob.
#
# The two pickers share an outcome -> (family, link) table that differs
# only in the "positive" branch: gam can use Gamma/log, but lme4's
# Gamma path is finicky enough that glmm sticks with gaussian-log.
canonical_family_link <- function(outcome_kind, positive_family, positive_link) {
  switch(outcome_kind,
    binary     = list(family = "binomial", link = "logit"),
    count      = list(family = "poisson",  link = "log"),
    positive   = list(family = positive_family, link = positive_link),
    continuous = list(family = "gaussian", link = "identity"),
                 list(family = "gaussian", link = "identity")
  )
}

pick_gam_family <- function(outcome_kind) {
  c(list(fitter = "gam"),
    canonical_family_link(outcome_kind,
                          positive_family = "Gamma",
                          positive_link   = "log"))
}

# sem (single-mediator path model): outcome shape doesn't change the
# model class, so the picker is parameter-free. The fitter handles the
# X / M assignment from spec$subset and returns NULL when the spec has
# fewer than two predictors.
pick_sem_family <- function() {
  list(fitter = "sem", model = "mediation")
}

# cor (bare bivariate correlation): roll one of Pearson / Spearman /
# Kendall. The fitter requires a single-predictor spec and returns
# NULL otherwise, so a multi-predictor draw just falls through.
pick_cor_family <- function() {
  list(fitter = "cor",
       method  = sample(c("pearson", "spearman", "kendall"), 1L))
}

# wls (heteroscedasticity-corrected lm): no parameters needed — the
# fitter estimates the variance trend from the OLS residuals.
pick_wls_family <- function() {
  list(fitter = "wls")
}

pick_glmm_family <- function(outcome_kind) {
  c(list(fitter = "glmm"),
    canonical_family_link(outcome_kind,
                          positive_family = "gaussian",
                          positive_link   = "log"))
}

test_that("fit_spec dispatches lm by default", {
  spec <- texanshootR:::new_spec(c("wt", "hp"))
  spec$subset_full <- "wt,hp,cyl"
  r <- texanshootR:::fit_spec(mtcars, "mpg", spec)
  expect_type(r, "list")
  expect_equal(r$family, "lm")
  expect_true(is.numeric(r$r_squared) && r$r_squared > 0)
  expect_true(is.numeric(r$p_value) && r$p_value >= 0 && r$p_value <= 1)
})

test_that("fit_spec dispatches glm when spec asks for it", {
  spec <- texanshootR:::new_spec(c("wt", "hp"),
                                  family = list(fitter = "glm",
                                                 family = "gaussian",
                                                 link   = "identity"))
  spec$subset_full <- "wt,hp,cyl"
  r <- texanshootR:::fit_spec(mtcars, "mpg", spec)
  expect_type(r, "list")
  expect_match(r$family, "^glm:gaussian:identity")
  expect_true(is.numeric(r$r_squared))
  expect_true(is.numeric(r$p_value))
  expect_true(is.list(r$family_meta))
  expect_equal(r$family_meta$family, "gaussian")
})

test_that("fit_glm handles binary outcome with logit", {
  d <- mtcars
  spec <- texanshootR:::new_spec(c("wt", "hp"),
                                  family = list(fitter = "glm",
                                                 family = "binomial",
                                                 link   = "logit"))
  spec$subset_full <- "wt,hp"
  r <- texanshootR:::fit_spec(d, "am", spec)  # am is 0/1
  expect_type(r, "list")
  expect_match(r$family, "^glm:binomial:logit")
})

test_that("fit_glm handles count outcome with poisson log", {
  d <- mtcars
  d$cyl <- as.integer(d$cyl)
  spec <- texanshootR:::new_spec(c("wt", "hp"),
                                  family = list(fitter = "glm",
                                                 family = "poisson",
                                                 link   = "log"))
  spec$subset_full <- "wt,hp"
  r <- texanshootR:::fit_spec(d, "cyl", spec)
  expect_type(r, "list")
  expect_match(r$family, "^glm:poisson:log")
})

test_that("fit_glm rejects out-of-domain outcomes (poisson on negatives)", {
  d <- mtcars; d$neg <- -seq_len(nrow(d))
  spec <- texanshootR:::new_spec(c("wt", "hp"),
                                  family = list(fitter = "glm",
                                                 family = "poisson",
                                                 link   = "log"))
  spec$subset_full <- "wt,hp"
  expect_null(texanshootR:::fit_spec(d, "neg", spec))
})

test_that("fit_glm with median_dichotomize coerces a continuous outcome", {
  d <- mtcars
  spec <- texanshootR:::new_spec(c("wt", "hp"),
                                  family = list(fitter = "glm",
                                                 family = "binomial",
                                                 link   = "logit",
                                                 outcome_coerce =
                                                   "median_dichotomize"))
  spec$subset_full <- "wt,hp"
  r <- texanshootR:::fit_spec(d, "mpg", spec)
  expect_type(r, "list")
  expect_match(r$family, "median_dichotomize")
})

test_that("available_families gates families by career tier", {
  expect_equal(texanshootR:::available_families("Junior Researcher"), "lm")
  expect_setequal(texanshootR:::available_families("Postdoc"),
                   c("lm", "glm", "cor"))
  expect_setequal(texanshootR:::available_families("Senior Scientist"),
                   c("lm", "glm", "cor", "wls", "gam", "rlm"))
  expect_setequal(texanshootR:::available_families("PI"),
                   c("lm", "glm", "cor", "wls", "gam", "rlm", "glmm", "sem"))
})

test_that("select_family at Junior is always lm", {
  set.seed(7)
  out <- replicate(20, texanshootR:::select_family(
    "Junior Researcher", outcome = mtcars$mpg), simplify = FALSE)
  fitters <- vapply(out, function(x) x$fitter, character(1))
  expect_true(all(fitters == "lm"))
})

test_that("select_family at PI eventually returns glm", {
  set.seed(7)
  out <- replicate(50, texanshootR:::select_family(
    "PI", outcome = mtcars$mpg), simplify = FALSE)
  fitters <- vapply(out, function(x) x$fitter, character(1))
  expect_true(any(fitters == "glm"))
})

test_that("select_family in escalation prefers glm and may flail", {
  set.seed(11)
  out <- replicate(50, texanshootR:::select_family(
    "Postdoc", escalating = TRUE, outcome = mtcars$mpg), simplify = FALSE)
  fitters <- vapply(out, function(x) x$fitter, character(1))
  expect_gt(mean(fitters == "glm"), 0.6)

  glms <- Filter(function(x) x$fitter == "glm", out)
  flails <- vapply(glms, function(x) !is.null(x$outcome_coerce), logical(1))
  expect_true(any(flails))  # at least one flail across the 50 rolls
})

test_that("fit_gam fits a smooth-term model on a continuous outcome", {
  spec <- texanshootR:::new_spec(c("wt", "hp"),
                                  family = list(fitter = "gam"))
  spec$subset_full <- "wt,hp,cyl"
  r <- texanshootR:::fit_spec(mtcars, "mpg", spec)
  expect_type(r, "list")
  expect_match(r$family, "^gam:gaussian:identity")
  expect_true(grepl("s\\(wt", r$formula))
  expect_true(is.numeric(r$r_squared) && r$r_squared > 0)
  expect_true(is.numeric(r$p_value) && r$p_value >= 0 && r$p_value <= 1)
  expect_equal(r$family_meta$family, "gaussian")
  expect_true(is.numeric(r$family_meta$lambda) && r$family_meta$lambda > 0)
  expect_true(is.numeric(r$family_meta$edf) && r$family_meta$edf > 0)
})

test_that("fit_gam falls back to parametric terms for low-cardinality predictors", {
  spec <- texanshootR:::new_spec(c("am", "wt"),
                                  family = list(fitter = "gam"))
  spec$subset_full <- "am,wt"
  r <- texanshootR:::fit_spec(mtcars, "mpg", spec)
  expect_type(r, "list")
  # am has only two levels — must NOT be wrapped in s().
  expect_false(grepl("s\\(am", r$formula))
  expect_true(grepl("s\\(wt", r$formula))
  expect_setequal(r$family_meta$smoothed, "wt")
})

test_that("fit_gam recovers a known smooth signal far better than lm", {
  set.seed(31)
  n <- 200L
  # cos(x) on [0, 2*pi] has zero linear trend by symmetry, so lm
  # should explain ~no variance while a smooth term recovers most of it.
  x <- sort(stats::runif(n, 0, 2 * pi))
  y <- cos(x) + stats::rnorm(n, sd = 0.2)
  d <- data.frame(x = x, y = y)
  spec_gam <- texanshootR:::new_spec("x",
                                      family = list(fitter = "gam"))
  spec_gam$subset_full <- "x"
  r_gam <- texanshootR:::fit_spec(d, "y", spec_gam)
  r_lm  <- texanshootR:::fit_spec(d, "y",
                                   texanshootR:::new_spec("x"))
  expect_gt(r_gam$r_squared, 0.80)
  expect_lt(r_lm$r_squared, 0.10)
})

test_that("fit_glmm fits a random-intercept model when a grouping factor exists", {
  d <- mtcars
  d$gear_g <- factor(d$gear)
  spec <- texanshootR:::new_spec(c("wt", "hp"),
                                  family = list(fitter = "glmm"))
  spec$subset_full <- "wt,hp,cyl,gear_g"
  r <- texanshootR:::fit_spec(d, "mpg", spec)
  expect_type(r, "list")
  expect_match(r$family, "^glmm:gaussian:identity")
  expect_true(grepl("\\(1 \\|", r$formula))
  expect_true(is.numeric(r$r_squared) && r$r_squared >= 0 && r$r_squared <= 1)
  expect_true(is.numeric(r$p_value) && r$p_value >= 0 && r$p_value <= 1)
  expect_true(!is.null(r$family_meta$group))
  expect_true(is.numeric(r$family_meta$theta) && r$family_meta$theta >= 0)
})

test_that("fit_glmm recovers a meaningful between-group variance", {
  set.seed(47)
  G <- 12L; per <- 15L
  group <- factor(rep(seq_len(G), each = per))
  b <- stats::rnorm(G, sd = 2.0)
  x <- stats::rnorm(G * per)
  y <- 1 + 0.5 * x + b[as.integer(group)] + stats::rnorm(G * per, sd = 0.5)
  d <- data.frame(y = y, x = x, group = group)
  spec <- texanshootR:::new_spec("x",
                                  family = list(fitter = "glmm"))
  spec$subset_full <- "x,group"
  r <- texanshootR:::fit_spec(d, "y", spec)
  expect_type(r, "list")
  # Strong group signal should drive a small p-value.
  expect_lt(r$p_value, 0.001)
  # sigma_b^2 should land near 4 (sd = 2) — not exact, just same order.
  expect_gt(r$family_meta$sigma_b2, 1)
  expect_lt(r$family_meta$sigma_b2, 16)
})

test_that("fit_glmm returns NULL when no grouping factor is available", {
  d <- mtcars[, c("mpg", "wt", "hp")]
  spec <- texanshootR:::new_spec(c("wt", "hp"),
                                  family = list(fitter = "glmm"))
  spec$subset_full <- "wt,hp"
  expect_null(texanshootR:::fit_spec(d, "mpg", spec))
})

test_that("select_family at Senior may pick gam", {
  set.seed(13)
  out <- replicate(200, texanshootR:::select_family(
    "Senior Scientist", outcome = mtcars$mpg), simplify = FALSE)
  fitters <- vapply(out, function(x) x$fitter, character(1))
  expect_true(any(fitters == "gam"))
  expect_true(any(fitters == "glm"))
  expect_false(any(fitters == "glmm"))  # PI-only
})

test_that("select_family at PI may pick glmm", {
  set.seed(17)
  out <- replicate(200, texanshootR:::select_family(
    "PI", outcome = mtcars$mpg), simplify = FALSE)
  fitters <- vapply(out, function(x) x$fitter, character(1))
  expect_true(any(fitters == "glmm"))
})

test_that("fit_sem fits a mediation model and reports the Sobel p-value", {
  # Synthetic mediation: X -> M -> Y, no direct path. With a strong
  # indirect effect the Sobel p-value should be tiny.
  set.seed(91)
  n <- 200L
  x <- stats::rnorm(n)
  m <- 0.7 * x + stats::rnorm(n, sd = 0.5)
  y <- 0.6 * m + stats::rnorm(n, sd = 0.5)
  d <- data.frame(y = y, x = x, m = m)
  spec <- texanshootR:::new_spec(c("x", "m"),
                                  family = list(fitter = "sem",
                                                 model  = "mediation"))
  spec$subset_full <- "x,m"
  r <- texanshootR:::fit_spec(d, "y", spec)
  expect_type(r, "list")
  expect_equal(r$family, "sem:mediation")
  expect_match(r$formula, "via m")
  expect_lt(r$p_value, 0.001)
  # Indirect effect should land near 0.7 * 0.6 = 0.42.
  expect_gt(r$family_meta$ab, 0.2)
  expect_lt(r$family_meta$ab, 0.6)
  expect_true(is.numeric(r$family_meta$sobel_z))
})

test_that("fit_sem returns a non-significant Sobel when there is no mediation", {
  set.seed(92)
  n <- 200L
  x <- stats::rnorm(n)
  # M is independent of X, so a ~ 0 and the indirect effect is nil.
  m <- stats::rnorm(n)
  y <- 0.5 * x + stats::rnorm(n, sd = 0.5)
  d <- data.frame(y = y, x = x, m = m)
  spec <- texanshootR:::new_spec(c("x", "m"),
                                  family = list(fitter = "sem"))
  spec$subset_full <- "x,m"
  r <- texanshootR:::fit_spec(d, "y", spec)
  expect_type(r, "list")
  expect_gt(r$p_value, 0.05)
})

test_that("fit_sem requires at least two predictors", {
  spec <- texanshootR:::new_spec("wt",
                                  family = list(fitter = "sem"))
  spec$subset_full <- "wt"
  expect_null(texanshootR:::fit_spec(mtcars, "mpg", spec))
})

test_that("fit_sem accepts extra subset entries as Y-equation covariates", {
  set.seed(93)
  spec <- texanshootR:::new_spec(c("wt", "hp", "cyl"),
                                  family = list(fitter = "sem"))
  spec$subset_full <- "wt,hp,cyl"
  r <- texanshootR:::fit_spec(mtcars, "mpg", spec)
  expect_type(r, "list")
  expect_equal(r$family, "sem:mediation")
  expect_setequal(r$family_meta$controls, "cyl")
})

test_that("select_family at PI may pick sem", {
  set.seed(23)
  out <- replicate(400, texanshootR:::select_family(
    "PI", outcome = mtcars$mpg), simplify = FALSE)
  fitters <- vapply(out, function(x) x$fitter, character(1))
  expect_true(any(fitters == "sem"))
})

test_that("fit_cor reports a Pearson correlation and its p-value", {
  set.seed(101)
  spec <- texanshootR:::new_spec("wt",
                                  family = list(fitter = "cor",
                                                 method  = "pearson"))
  spec$subset_full <- "wt"
  r <- texanshootR:::fit_spec(mtcars, "mpg", spec)
  expect_type(r, "list")
  expect_equal(r$family, "cor:pearson")
  expect_true(is.numeric(r$family_meta$r))
  # wt and mpg are strongly negatively correlated.
  expect_lt(r$family_meta$r, -0.7)
  expect_lt(r$p_value, 0.001)
  expect_equal(r$r_squared, r$family_meta$r^2, tolerance = 1e-10)
})

test_that("fit_cor requires exactly one predictor", {
  spec <- texanshootR:::new_spec(c("wt", "hp"),
                                  family = list(fitter = "cor"))
  spec$subset_full <- "wt,hp"
  expect_null(texanshootR:::fit_spec(mtcars, "mpg", spec))
})

test_that("fit_cor supports rank-based methods", {
  spec <- texanshootR:::new_spec("wt",
                                  family = list(fitter = "cor",
                                                 method  = "spearman"))
  spec$subset_full <- "wt"
  r <- texanshootR:::fit_spec(mtcars, "mpg", spec)
  expect_equal(r$family, "cor:spearman")
})

test_that("fit_wls reweights a heteroscedastic regression", {
  # Build data where Var(y | x) grows with |x|. WLS should match OLS
  # qualitatively (both find the slope) but produce a different RSS
  # because the weighting changes the residual norm.
  set.seed(111)
  n <- 200L
  x <- stats::rnorm(n)
  y <- 1 + 0.8 * x + stats::rnorm(n, sd = 0.3 + abs(x))
  d <- data.frame(y = y, x = x)
  spec_wls <- texanshootR:::new_spec("x",
                                      family = list(fitter = "wls"))
  spec_wls$subset_full <- "x"
  r_wls <- texanshootR:::fit_spec(d, "y", spec_wls)
  r_lm  <- texanshootR:::fit_spec(d, "y",
                                   texanshootR:::new_spec("x"))
  expect_type(r_wls, "list")
  expect_equal(r_wls$family, "wls")
  expect_true(is.numeric(r_wls$r_squared) && r_wls$r_squared > 0)
  # The two strategies share the same significance verdict here.
  expect_lt(r_wls$p_value, 0.05)
  expect_lt(r_lm$p_value, 0.05)
})

test_that("select_family at Postdoc may pick cor", {
  set.seed(31)
  out <- replicate(500, texanshootR:::select_family(
    "Postdoc", outcome = mtcars$mpg), simplify = FALSE)
  fitters <- vapply(out, function(x) x$fitter, character(1))
  expect_true(any(fitters == "cor"))
  expect_false(any(fitters == "wls"))  # Senior+
})

test_that("select_family at Senior may pick wls", {
  set.seed(37)
  out <- replicate(500, texanshootR:::select_family(
    "Senior Scientist", outcome = mtcars$mpg), simplify = FALSE)
  fitters <- vapply(out, function(x) x$fitter, character(1))
  expect_true(any(fitters == "wls"))
})

test_that("classify_outcome distinguishes the four shapes", {
  expect_equal(texanshootR:::classify_outcome(c(0, 1, 1, 0)), "binary")
  expect_equal(texanshootR:::classify_outcome(c(0L, 2L, 5L, 3L)), "count")
  expect_equal(texanshootR:::classify_outcome(c(1.2, 3.4, 0.5)), "positive")
  expect_equal(texanshootR:::classify_outcome(c(-1, 2, 0.5)), "continuous")
  expect_equal(texanshootR:::classify_outcome(NULL), "unknown")
  expect_equal(texanshootR:::classify_outcome(letters), "unknown")
})

# Output-generator unlock model. Each generator maps to a chain stage
# (see R/chain.R). The legacy career-tier UNLOCK_REGISTRY remains here
# only as the metadata `progress()` displays — the actual gate is the
# XP-driven chain length, checked via require_chain_stage() from each
# generator's source file.

# Maps each gated function to the chain stage it produces. The chain
# stage index dictates the XP required (CHAIN_XP_THRESHOLDS).
GENERATOR_STAGE <- list(
  abstract           = "abstract",
  manuscript         = "manuscript",
  presentation       = "presentation",
  reviewer_response  = "reviewer_response",
  graphical_abstract = "graphical_abstract",
  funding            = "funding"
)

# Retained for legacy progress() rows and tests that reference tier
# labels. Derived from chain index: the lowest tier at which each
# stage's chain length becomes unlocked.
UNLOCK_REGISTRY <- list(
  manuscript         = "Postdoc",
  reviewer_response  = "Senior Scientist",
  graphical_abstract = "Senior Scientist",
  presentation       = "Senior Scientist",
  funding            = "PI"
)

tier_index <- function(level) {
  tiers <- unique(CHAIN_TIER_BY_LENGTH)
  m <- match(level, tiers)
  if (is.na(m)) return(0L)
  m
}

current_career_level <- function() {
  meta <- tryCatch(read_meta(), error = function(e) NULL)
  if (is.null(meta)) return("Junior Researcher")
  prog <- progression_of(meta)
  career_level_for_length(prog$length_unlocked)
}

is_unlocked <- function(fn_name, level = current_career_level()) {
  stage <- GENERATOR_STAGE[[fn_name]]
  if (is.null(stage)) return(TRUE)
  meta <- read_meta() %||% default_meta(save_dir() %||% tempfile())
  prog <- progression_of(meta)
  stage_index(stage) <= prog$length_unlocked
}

format_locked_message <- function(fn_name, required, current) {
  paste0(
    fn_name, "() requires:\n",
    required, "\n",
    "\n",
    "Current career:\n",
    current
  )
}

# Legacy-shaped check kept for backwards compatibility with anything
# that still calls it. New code routes through require_chain_stage()
# in R/chain.R. We re-implement on top of progression so the two views
# stay consistent.
require_unlocked <- function(fn_name) {
  if (is_unlocked(fn_name)) return(invisible(TRUE))
  required <- UNLOCK_REGISTRY[[fn_name]] %||% "Postdoc"
  current  <- current_career_level()
  cond <- structure(
    class = c("tx_locked", "error", "condition"),
    list(
      message  = format_locked_message(fn_name, required, current),
      call     = NULL,
      fn       = fn_name,
      required = required,
      current  = current
    )
  )
  stop(cond)
}

# Achievement progress extractors. Each takes (runs, meta) and returns
# list(have, need, label).
ACHIEVEMENT_PROGRESS <- list(
  ach_multiple_comparisons = function(runs, meta) {
    best <- max(c(0L, vapply(runs,
                              function(r) as.integer(r$spec_count %||% 0L),
                              integer(1))))
    list(have = best, need = 1000L, label = "best run")
  },
  ach_var_purge = function(runs, meta) {
    best <- max(c(0L, vapply(runs,
                              function(r) length(r$highlighted_spec$dropped %||% character()),
                              integer(1))))
    list(have = as.integer(best), need = 5L, label = "best run")
  },
  ach_publication_pipeline = function(runs, meta) {
    have <- as.integer(meta$hidden$output_complexity %||% 0)
    list(have = have, need = 3L, label = "career")
  },
  ach_event_survivor = function(runs, meta) {
    have <- length(meta$events_witnessed %||% list())
    list(have = as.integer(have), need = 3L, label = "witnessed")
  },
  ach_all_outputs = function(runs, meta) {
    best <- max(c(0L, vapply(runs,
                              function(r) length(r$outputs_generated %||% character()),
                              integer(1))))
    list(have = as.integer(best), need = 6L, label = "best run")
  }
)

inflight_progress_rows <- function(runs, meta, ach_state, ach_reg) {
  ids <- intersect(names(ACHIEVEMENT_PROGRESS), ach_reg$id)
  out <- list()
  for (id in ids) {
    if (isTRUE(ach_state[[id]])) next
    row <- ach_reg[ach_reg$id == id, ]
    if (!isTRUE(row$visible)) next
    p <- ACHIEVEMENT_PROGRESS[[id]](runs, meta)
    if (is.null(p)) next
    out[[length(out) + 1L]] <- list(
      id    = id,
      name  = row$name,
      have  = p$have,
      need  = p$need,
      label = p$label
    )
  }
  out
}

#' Inspect API unlock progress
#'
#' Prints a HUD-style summary of unlock state for gated functions and
#' achievements. Reads live save state.
#'
#' Three call modes:
#' \itemize{
#'   \item `progress()` -- overview: career tier, gated-function lock
#'     map, achievement / wardrobe counts, and in-flight progress.
#'   \item `progress("manuscript")` -- per-function card.
#'   \item `progress("ach_multiple_comparisons")` -- per-achievement
#'     card.
#' }
#'
#' @param what Optional. A gated-function name (e.g. `"manuscript"`) or
#'   an achievement id (e.g. `"ach_multiple_comparisons"`). When NULL,
#'   prints the overview.
#' @return A `tx_progress` object (invisible).
#' @export
progress <- function(what = NULL) {
  current <- current_career_level()
  if (is.null(what)) {
    obj <- progress_overview(current)
  } else if (!is.null(UNLOCK_REGISTRY[[what]]) ||
             !is.null(GENERATOR_STAGE[[what]])) {
    obj <- progress_function(what, current)
  } else if (startsWith(what, "ach_")) {
    obj <- progress_achievement(what)
  } else {
    stop(sprintf("`%s` is not a gated function or achievement id.", what),
         call. = FALSE)
  }
  print(obj)
  invisible(obj)
}

progress_overview <- function(current) {
  meta <- read_meta() %||% default_meta(save_dir() %||% tempfile())
  ach_state <- read_achievements_state()
  ach_reg   <- load_achievement_registry()
  cos_reg   <- load_cosmetic_registry()
  wd        <- read_wardrobe_state()
  runs      <- recent_run_records()
  prog      <- progression_of(meta)

  rows <- lapply(names(UNLOCK_REGISTRY), function(nm) {
    list(fn = nm, required = UNLOCK_REGISTRY[[nm]],
         unlocked = is_unlocked(nm, current))
  })

  ach_unlocked <- sum(vapply(ach_state, isTRUE, logical(1)))
  ach_total    <- nrow(ach_reg)
  cos_unlocked <- length(wd$unlocked %||% character())
  cos_total    <- nrow(cos_reg)

  inflight <- inflight_progress_rows(runs, meta, ach_state, ach_reg)

  structure(list(
    mode      = "overview",
    current   = current,
    rows      = rows,
    chain_length = prog$length_unlocked,
    xp        = prog$xp,
    xp_next   = next_xp_threshold(prog$length_unlocked),
    active_chain_hud = format_chain_hud(meta),
    ach_unlocked = ach_unlocked,
    ach_total    = ach_total,
    cos_unlocked = cos_unlocked,
    cos_total    = cos_total,
    inflight  = inflight
  ), class = "tx_progress")
}

progress_function <- function(fn, current) {
  required <- UNLOCK_REGISTRY[[fn]] %||% "Junior Researcher"
  structure(list(
    mode     = "function",
    fn       = fn,
    current  = current,
    required = required,
    unlocked = is_unlocked(fn, current)
  ), class = "tx_progress")
}

progress_achievement <- function(id) {
  reg <- load_achievement_registry()
  if (!(id %in% reg$id)) {
    stop(sprintf("`%s` is not a known achievement id.", id), call. = FALSE)
  }
  state <- read_achievements_state()
  row <- reg[reg$id == id, ]
  unlocked <- isTRUE(state[[id]])
  hidden_locked <- !unlocked && !isTRUE(row$visible)

  obj <- list(
    mode     = "achievement",
    id       = id,
    name     = if (hidden_locked) "???" else row$name,
    unlocked = unlocked,
    hidden_locked = hidden_locked,
    detail   = NULL
  )
  if (!hidden_locked) {
    extractor <- ACHIEVEMENT_PROGRESS[[id]]
    if (!unlocked && !is.null(extractor)) {
      meta <- read_meta() %||% default_meta(save_dir() %||% tempfile())
      runs <- recent_run_records()
      obj$detail <- extractor(runs, meta)
    }
  }
  structure(obj, class = "tx_progress")
}

#' @export
#' @noRd
#' @method print tx_progress
print.tx_progress <- function(x, ...) {
  switch(x$mode,
    overview    = print_progress_overview(x),
    "function"  = print_progress_function(x),
    achievement = print_progress_achievement(x))
  invisible(x)
}

print_progress_overview <- function(x) {
  rule <- strrep("-", 48)
  cat(style_header(rule), "\n", sep = "")
  cat(style_header(sprintf("Career: %s", x$current)), "\n", sep = "")
  cat(style_header(rule), "\n", sep = "")
  cat(sprintf("Chain length:  %d / %d\n", x$chain_length,
              length(CHAIN_STAGES)))
  if (!is.na(x$xp_next)) {
    cat(sprintf("XP:            %d / %d (next unlock)\n",
                x$xp, x$xp_next))
  } else {
    cat(sprintf("XP:            %d (chain fully unlocked)\n", x$xp))
  }
  cat("\n")
  unlocked <- Filter(function(r) r$unlocked, x$rows)
  locked   <- Filter(function(r) !r$unlocked, x$rows)
  if (length(unlocked) > 0L) {
    cat("Unlocked:\n")
    for (r in unlocked) cat(sprintf("  %s()\n", r$fn))
  }
  if (length(locked) > 0L) {
    if (length(unlocked) > 0L) cat("\n")
    cat("Locked:\n")
    width <- max(nchar(vapply(locked, function(r) r$fn, character(1)))) + 2L
    for (r in locked) {
      cat(sprintf("  %-*s requires %s\n", width,
                   paste0(r$fn, "()"), r$required))
    }
  }
  if (!is.null(x$active_chain_hud)) {
    cat("\n", x$active_chain_hud, "\n", sep = "")
  }
  cat(sprintf("\nAchievements: %d / %d\n", x$ach_unlocked, x$ach_total))
  cat(sprintf("Wardrobe:     %d / %d\n", x$cos_unlocked, x$cos_total))
  if (length(x$inflight) > 0L) {
    cat("\nIn-flight:\n")
    name_w  <- max(nchar(vapply(x$inflight, `[[`, character(1), "name")))
    label_w <- max(nchar(vapply(x$inflight, `[[`, character(1), "label")))
    for (r in x$inflight) {
      cat(sprintf("  %-*s   %-*s %d / %d\n",
                   name_w, r$name,
                   label_w + 1L, paste0(r$label, ":"),
                   r$have, r$need))
    }
  }
}

print_progress_function <- function(x) {
  if (x$unlocked) {
    cat(x$fn, "()\n", sep = "")
    cat("status: unlocked\n")
  } else {
    cat(format_locked_message(x$fn, x$required, x$current), "\n", sep = "")
  }
}

print_progress_achievement <- function(x) {
  cat(x$id, ":\n", sep = "")
  cat(x$name, "\n", sep = "")
  cat("\nStatus:\n")
  if (x$unlocked) {
    cat("unlocked\n")
  } else if (x$hidden_locked) {
    cat("locked\n")
  } else {
    cat("locked\n")
    if (!is.null(x$detail)) {
      cat(sprintf("\n%s:\n%d / %d\n",
                   tools::toTitleCase(x$detail$label),
                   x$detail$have, x$detail$need))
    }
  }
}

#' Generate a one-paragraph deadpan abstract
#'
#' The entry-tier output. Always the first stage of the publication
#' chain (see [progress()]). Writes a plain-text `.txt` file describing
#' the highlighted specification in the unbothered register typical of
#' an applied-stats abstract.
#'
#' Chain stage: \strong{abstract} (length 1, always unlocked).
#'
#' @param run A `tx_run` object returned by [shoot()]. Must be the
#'   currently active publication chain — i.e., the run that opened
#'   the chain when it cleared `p <= 0.05`.
#' @param output_dir Optional output directory.
#' @param file Optional filename stem (without extension).
#' @param force Overwrite an existing file.
#' @return Character path to the written file (invisible).
#' @export
abstract <- function(run, output_dir = NULL, file = NULL,
                     force = FALSE) {
  require_chain_stage("abstract", run)
  d   <- resolve_output_dir(output_dir)
  stem <- file %||% "abstract"
  out  <- versioned_filename(d, stem, "txt", force)

  text <- build_abstract_text(run)
  status_open(out)
  writeLines(text, con = out)
  status_close()
  record_output(run, "abstract", out)
  advance_chain_after_stage("abstract")
  invisible(out)
}

build_abstract_text <- function(run) {
  hs <- run$highlighted_spec %||% list()
  rsq      <- hs$r_squared %||% 0
  pval     <- hs$p_value  %||% 1
  n_specs  <- run$spec_count %||% 0L
  methods  <- describe_spec(hs)

  paste(
    "Abstract",
    "",
    paste0(
      "We examined the relationship between the response and its ",
      "predictors across a targeted specification search of ",
      n_specs, " candidate models. ",
      methods, " ",
      sprintf("The reported model recovered an R-squared of %.3f with a p-value of %.4f. ",
              rsq, pval),
      "These results inform a coherent picture of the underlying ",
      "process and warrant further investigation."
    ),
    sep = "\n"
  )
}

#' Inspect career state
#'
#' Returns a `tx_career` object describing the persistent career, or a
#' fresh default if no save exists. The object's `print` method shows
#' a compact dashboard card; `summary` adds aggregate counts. Career
#' tier is derived from the publication-chain length you've unlocked
#' (see [progress()]); XP comes from completed chain stages.
#'
#' @return A `tx_career` object.
#' @export
career <- function() {
  meta <- read_meta()
  if (is.null(meta)) {
    meta <- default_meta(save_dir() %||% tempfile())
  }

  runs   <- recent_run_records()
  ach    <- read_achievements_state()
  wd     <- read_wardrobe_state()
  prog   <- progression_of(meta)

  fav_method   <- favorite_method(runs)
  most_removed <- most_removed_var(runs)
  next_thr     <- next_xp_threshold(prog$length_unlocked)

  obj <- list(
    level                 = career_level_for_length(prog$length_unlocked),
    runs_count            = length(runs),
    favorite_method       = fav_method,
    most_removed_variable = most_removed,
    xp                    = prog$xp,
    xp_next_unlock        = next_thr,
    chain_length          = prog$length_unlocked,
    chains_completed      = prog$chains_completed,
    chains_broken         = prog$chains_broken,
    active_chain_hud      = format_chain_hud(meta),
    achievements_unlocked = sum(vapply(ach, isTRUE, logical(1))),
    achievements_total    = nrow(load_achievement_registry()),
    wardrobe              = wd,
    meta                  = meta,
    runs_summary          = runs
  )
  class(obj) <- "tx_career"
  obj
}

# XP cost to reach the next chain length, or NA if at the cap.
next_xp_threshold <- function(length_unlocked) {
  i <- as.integer(length_unlocked %||% 1L) + 1L
  if (i > length(CHAIN_XP_THRESHOLDS)) return(NA_integer_)
  CHAIN_XP_THRESHOLDS[[i]]
}

favorite_method <- function(runs) {
  if (length(runs) == 0L) return(NA_character_)
  fams <- unlist(lapply(runs, function(r) r$highlighted_spec$family %||% NA))
  fams <- fams[!is.na(fams)]
  if (length(fams) == 0L) return(NA_character_)
  names(sort(table(fams), decreasing = TRUE))[1]
}

most_removed_var <- function(runs) {
  if (length(runs) == 0L) return(NA_character_)
  removed <- unlist(lapply(runs, function(r) r$highlighted_spec$dropped %||% character()))
  if (length(removed) == 0L) return(NA_character_)
  names(sort(table(removed), decreasing = TRUE))[1]
}

recent_run_records <- function(n = 50L) {
  files <- list_run_records()
  if (length(files) == 0L) return(list())
  files <- files[order(file.info(files)$mtime, decreasing = TRUE)]
  files <- utils::head(files, n)
  lapply(files, readRDS)
}

#' @export
#' @noRd
#' @method print tx_career
print.tx_career <- function(x, ...) {
  rule <- strrep("-", 48)
  cat(style_header(rule), "\n", sep = "")
  cat(style_header(sprintf("Career: %s", x$level)), "\n", sep = "")
  cat(style_header(rule), "\n", sep = "")
  cat(sprintf("Runs completed:        %d\n", x$runs_count))
  cat(sprintf("Favorite method:       %s\n",
              x$favorite_method %||% "-"))
  cat(sprintf("Most removed variable: %s\n",
              x$most_removed_variable %||% "-"))
  cat(sprintf("Chain length:          %d / %d\n",
              x$chain_length, length(CHAIN_STAGES)))
  if (!is.na(x$xp_next_unlock)) {
    cat(sprintf("XP:                    %d / %d (next unlock)\n",
                x$xp, x$xp_next_unlock))
  } else {
    cat(sprintf("XP:                    %d (chain fully unlocked)\n",
                x$xp))
  }
  cat(sprintf("Chains completed:      %d\n", x$chains_completed))
  if (!is.null(x$active_chain_hud)) {
    cat("\n", x$active_chain_hud, "\n", sep = "")
  }
  invisible(x)
}

#' @export
#' @noRd
#' @method summary tx_career
summary.tx_career <- function(object, ...) {
  print(object)
  cat("\n")
  cat(sprintf("Achievements unlocked: %d / %d\n",
              object$achievements_unlocked, object$achievements_total))
  cat(sprintf("Models fit (lifetime): %d\n",
              as.integer(object$meta$hidden$models_fit %||% 0)))
  cat(sprintf("Outputs generated:     %d\n",
              as.integer(object$meta$hidden$output_complexity %||% 0)))
  cat(sprintf("Chains broken:         %d\n", object$chains_broken))
  invisible(object)
}

#' @export
#' @noRd
#' @method format tx_career
format.tx_career <- function(x, ...) {
  paste0(x$level, " - ", x$runs_count, " runs")
}

# Update meta with the latest run's contributions. Career tier no
# longer comes from this path — it tracks XP via the chain mechanic.
# What still lives here: hidden component bookkeeping (for
# achievements), per-run counters, and the favorite-method etc.
# derivations.
update_career <- function(meta, run) {
  hid <- meta$hidden %||% list()
  hid$models_fit          <- (hid$models_fit %||% 0) + (run$spec_count %||% 0)
  hid$reviewer_resistance <- (hid$reviewer_resistance %||% 0) +
    (run$modifiers$reviewer_resistance %||% 0) * 0.05
  hid$narrative_strength  <- (hid$narrative_strength %||% 0) +
    (run$modifiers$narrative_strength %||% 0) * 0.05
  hid$presentation_quality <- (hid$presentation_quality %||% 0) +
    (run$modifiers$presentation_pressure %||% 0) * 0.05
  meta$hidden <- hid
  meta$runs_count <- (meta$runs_count %||% 0L) + 1L

  prev_level <- meta$career_level
  prog <- progression_of(meta)
  new_level <- career_level_for_length(prog$length_unlocked)
  meta$career_level <- new_level

  attr(meta, "promoted")   <- !identical(prev_level, new_level)
  attr(meta, "from_level") <- prev_level
  attr(meta, "to_level")   <- new_level
  meta
}

test_that("shoot(depth='demo') runs the single-fit smoke path", {
  withr::local_seed(42)
  run <- shoot(mtcars, depth = "demo")
  expect_s3_class(run, "tx_run")
  expect_equal(run$spec_count, 1L)
})

test_that("shoot() returns a usable tx_run", {
  withr::local_options(texanshootR.budget = 2)
  withr::local_seed(42)
  run <- shoot(mtcars)
  expect_s3_class(run, "tx_run")
  expect_gt(run$spec_count, 0L)
})

test_that("shoot() persists a privacy-stripped run record", {
  withr::local_options(texanshootR.budget = 1)
  withr::local_seed(42)
  run <- shoot(mtcars)
  recs <- texanshootR:::recent_run_records(1)
  expect_length(recs, 1L)
  rec <- recs[[1]]
  expect_null(rec[["df"]])    # no raw frame (use [[ to defeat $ partial match)
  expect_false(is.null(rec$df_meta))
  expect_true(!is.null(rec$df_meta$summary))
})

test_that("seed makes runs reproducible", {
  withr::local_options(texanshootR.budget = 1)
  r1 <- shoot(mtcars, seed = 123)
  r2 <- shoot(mtcars, seed = 123)
  expect_equal(r1$grid_hash, r2$grid_hash)
})

test_that("tactical_pause rejects unknown +commands softly", {
  withr::local_options(texanshootR.budget = 2)
  withr::local_seed(13)
  run <- shoot(mtcars,
               prompt_fn = function(prompt) "+not_a_modifier")
  expect_s3_class(run, "tx_run")
  expect_length(run$modifiers_used %||% character(), 0L)
})

test_that("typing +derived_metrics mid-run engages the escalation phase", {
  withr::local_options(texanshootR.budget = 2)
  withr::local_seed(7)
  # Reply with +derived_metrics to whichever tactical-pause window fires
  # first; later windows return "" (skip).
  responses <- c("+derived_metrics")
  i <- 0L
  pf <- function(prompt) {
    i <<- i + 1L
    if (i <= length(responses)) responses[i] else ""
  }
  run <- shoot(mtcars, prompt_fn = pf)
  expect_s3_class(run, "tx_run")
  expect_true("derived_metrics" %in% (run$modifiers_used %||% character()))
})

# Live-injectable modifier registry.
#
# Replaces the flat SUPPORTED_MODIFIERS vector + scalar MODIFIER_BUDGET_BONUS
# in shoot.R with structured records. The registry is the single source
# of truth for:
#   - which tokens the +command parser accepts mid-run
#   - how much time each modifier adds to the shoot() deadline
#   - what bias each modifier imposes on the search engine
#   - what the TUI shows in the MODIFIERS row
#
# Time bonuses are per-modifier and reflect the "cost" of the move:
# heavyweight families (glmm, sem, derived_metrics) buy more deadline
# because their per-fit cost is higher; perturbation biases (subgroup,
# outliers) buy less because they only redirect the queue.
#
# Bias shape (the `bias` slot of each entry):
#   list(family       = character) -> force select_family() toward this fitter
#   list(perturb_tag  = character) -> push this kind to the head of the queue
#   list(escalate     = TRUE)      -> switch the run into desperation mode
# Biases compose by FIELD-WISE MERGE into state$bias. Different fields
# stack (a +glmm + +subgroup gives both a family bias AND a perturb bias);
# same-kind re-injection replaces (later +gam after +glmm switches the
# family priority). The escalate flag is one-shot sticky -- once set it
# stays set until the post-search desperation phase consumes it.

LIVE_MODIFIERS <- list(
  derived_metrics = list(
    token       = "derived_metrics",
    aliases     = c("derived", "dm"),
    display     = "+derived_metrics",
    time_bonus  = 5L,
    bias        = list(escalate = TRUE),
    description = "Trigger desperation: manufacture composite responses."
  ),
  subgroup = list(
    token       = "subgroup",
    aliases     = c("sub", "sg"),
    display     = "+subgroup",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "restriction"),
    description = "Bias toward analytic-sample restrictions (factor-level, Cook's D, complete cases)."
  ),
  outliers = list(
    token       = "outliers",
    aliases     = c("out"),
    display     = "+outliers",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "restriction"),
    description = "Bias toward principled-sounding sample exclusions."
  ),
  gam = list(
    token       = "gam",
    aliases     = character(),
    display     = "+gam",
    time_bonus  = 4L,
    bias        = list(family = "gam"),
    description = "Force the family selector toward GAM picks."
  ),
  glm = list(
    token       = "glm",
    aliases     = character(),
    display     = "+glm",
    time_bonus  = 3L,
    bias        = list(family = "glm"),
    description = "Force the family selector toward GLM picks."
  ),
  wls = list(
    token       = "wls",
    aliases     = character(),
    display     = "+wls",
    time_bonus  = 3L,
    bias        = list(family = "wls"),
    description = "Force the family selector toward weighted-LS picks."
  ),
  cor = list(
    token       = "cor",
    aliases     = character(),
    display     = "+cor",
    time_bonus  = 2L,
    bias        = list(family = "cor"),
    description = "Force the family selector toward bivariate correlation."
  ),
  glmm = list(
    token       = "glmm",
    aliases     = character(),
    display     = "+glmm",
    time_bonus  = 5L,
    bias        = list(family = "glmm"),
    description = "Force the family selector toward GLMM picks."
  ),
  sem = list(
    token       = "sem",
    aliases     = character(),
    display     = "+sem",
    time_bonus  = 5L,
    bias        = list(family = "sem"),
    description = "Force the family selector toward mediation SEM picks."
  ),
  aggregate = list(
    token       = "aggregate",
    aliases     = c("agg"),
    display     = "+aggregate",
    time_bonus  = 3L,
    bias        = list(perturb_tag = "aggregate"),
    description = "Collapse records to group means before fit."
  ),
  compilation = list(
    token       = "compilation",
    aliases     = c("comp"),
    display     = "+compilation",
    time_bonus  = 4L,
    bias        = list(perturb_tag = "compilation"),
    description = "Treat a literature compilation as ground-truth response."
  ),
  gapfill = list(
    token       = "gapfill",
    aliases     = c("gap"),
    display     = "+gapfill",
    time_bonus  = 3L,
    bias        = list(perturb_tag = "validate_on_fit"),
    description = "Validate the gap-fill on the cases that aren't gaps."
  ),
  tier2 = list(
    token       = "tier2",
    aliases     = c("relax", "downweight"),
    display     = "+tier2",
    time_bonus  = 3L,
    bias        = list(perturb_tag = "threshold_relax"),
    description = "Add a downweighted second tier when the cutoff is empty."
  ),
  counter_argument = list(
    token       = "counter_argument",
    aliases     = c("counter", "preempt"),
    display     = "+counter_argument",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "preempt"),
    description = "Address the reviewer's likely objections in the manuscript."
  ),
  pseudo_spatial = list(
    token       = "pseudo_spatial",
    aliases     = c("coords"),
    display     = "+pseudo_spatial",
    time_bonus  = 3L,
    bias        = list(perturb_tag = "pseudo_spatial"),
    description = "Add geographic coordinates as fixed effects and call it spatial."
  ),
  omit_control = list(
    token       = "omit_control",
    aliases     = c("omit"),
    display     = "+omit_control",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "omit_control"),
    description = "Drop a covariate because you don't want to interpret it."
  ),
  knife_edge = list(
    token       = "knife_edge",
    aliases     = c("labeling"),
    display     = "+knife_edge",
    time_bonus  = 3L,
    bias        = list(perturb_tag = "knife_edge"),
    description = "Define groups by a knife-edge presence rule on a continuous variable."
  ),
  iid_rescue = list(
    token       = "iid_rescue",
    aliases     = c("iid"),
    display     = "+iid_rescue",
    time_bonus  = 3L,
    bias        = list(perturb_tag = "iid_rescue"),
    description = "Reassign an inconvenient fixed effect to an iid random effect."
  ),
  defer_fix = list(
    token       = "defer_fix",
    aliases     = c("defer", "next_step"),
    display     = "+defer_fix",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "defer"),
    description = "Acknowledge a methodological correction; defer it to a future paper."
  ),
  coffee_break = list(
    token       = "coffee_break",
    aliases     = c("side_meeting", "offline"),
    display     = "+coffee_break",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "offline"),
    description = "Move methodological disagreement off the written record."
  ),
  mechanism_rebuttal = list(
    token       = "mechanism_rebuttal",
    aliases     = c("intuition", "but_biologically"),
    display     = "+mechanism_rebuttal",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "intuition"),
    description = "Dismiss a statistical concern with domain-mechanism intuition."
  ),
  misunderstanding = list(
    token       = "misunderstanding",
    aliases     = c("you_misread"),
    display     = "+misunderstanding",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "deflect_misread"),
    description = "Relocate the methodological problem from the analysis to the reader."
  ),
  narrow_answer = list(
    token       = "narrow_answer",
    aliases     = c("strawman"),
    display     = "+narrow_answer",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "deflect_narrow"),
    description = "Answer a weaker version of the concern at the wrong level."
  ),
  complexity_shield = list(
    token       = "complexity_shield",
    aliases     = c("its_complex"),
    display     = "+complexity_shield",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "deflect_complexity"),
    description = "Invoke unspecified complexity to dismiss a specific concern."
  ),
  frame_shift = list(
    token       = "frame_shift",
    aliases     = c("oscillate"),
    display     = "+frame_shift",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "deflect_frame"),
    description = "Oscillate between descriptive and causal framing without flagging."
  ),
  matter_of_taste = list(
    token       = "matter_of_taste",
    aliases     = c("just_one_way"),
    display     = "+matter_of_taste",
    time_bonus  = 2L,
    bias        = list(perturb_tag = "deflect_taste"),
    description = "Soften a falsifiable concern into a matter of perspective."
  )
)

# ---- Parser ---------------------------------------------------------

# Resolve a raw input buffer (whatever the user typed at the tactical
# prompt) to a canonical modifier token, or NA_character_ when the input
# is empty / unknown.
#
# Accepts all of these for the same modifier:
#   "+glmm" "+ glmm" "glmm" "+glmm()" "+ GLMM " "  glmm  "
# Aliases declared in LIVE_MODIFIERS$<token>$aliases also resolve.
parse_mod_command <- function(buf) {
  if (is.null(buf)) return(NA_character_)
  s <- trimws(as.character(buf)[1])
  if (!nzchar(s)) return(NA_character_)
  # Strip a leading "+" with optional whitespace.
  s <- sub("^\\+\\s*", "", s)
  # Strip parens at the end ("glmm()" -> "glmm").
  s <- sub("\\s*\\(.*\\)\\s*$", "", s)
  s <- tolower(trimws(s))
  if (!nzchar(s)) return(NA_character_)

  for (token in names(LIVE_MODIFIERS)) {
    m <- LIVE_MODIFIERS[[token]]
    if (s == token || s %in% m$aliases) return(token)
  }
  NA_character_
}

# ---- Availability ---------------------------------------------------

# Tokens the player can still inject this run: not in `consumed`, and
# within career gates. Family modifiers inherit FAMILIES[[name]]$min_career;
# non-family modifiers (subgroup, outliers, derived_metrics) are
# tier-agnostic.
available_modifiers <- function(consumed = character(),
                                career_level = "Junior Researcher") {
  out <- character()
  for (token in names(LIVE_MODIFIERS)) {
    if (token %in% consumed) next
    if (!modifier_available_for_career(token, career_level)) next
    out <- c(out, token)
  }
  out
}

modifier_available_for_career <- function(token, career_level) {
  reg <- FAMILIES[[token]]
  if (is.null(reg)) return(TRUE)  # non-family modifier — no gate
  tier_index(career_level) >= tier_index(reg$min_career)
}

# Pre-roll a per-transition loadout for non-interactive runs (RStudio
# Console / dynamic mode). Returns a list keyed by mascot state name;
# values are either a token string or NA (skip this transition). The
# shoot() loop applies the pre-rolled token at each mascot transition
# instead of opening a tactical_pause readline that RStudio cannot
# render in place.
pre_roll_modifiers <- function(career_level) {
  states <- c("uncertain", "worried", "anxious", "panicked", "desperate")
  out    <- as.list(stats::setNames(rep(NA_character_, length(states)), states))
  available <- available_modifiers(consumed = character(),
                                   career_level = career_level)
  if (!length(available)) return(out)
  n_picks <- sample.int(min(3L, length(available)), 1L)
  picks   <- sample(available, n_picks)
  slots   <- sample(states, n_picks)
  for (i in seq_along(picks)) out[[slots[i]]] <- picks[i]
  out
}

# Human-friendly list of the available modifiers for the MODIFIERS TUI
# row, e.g. "+gam  +derived  +subgroup".
format_modifier_row <- function(tokens) {
  if (!length(tokens)) return("(none)")
  paste(vapply(tokens, function(t) LIVE_MODIFIERS[[t]]$display, character(1)),
        collapse = "  ")
}

# ---- Apply ----------------------------------------------------------

# Default bias shape -- every state$bias holds these keys whether or not
# anything was injected. NA / FALSE are the "no override" sentinels the
# search engine treats as "use normal selection."
empty_bias <- function() {
  list(family      = NA_character_,
       perturb_tag = NA_character_,
       escalate    = FALSE)
}

# Apply a parsed modifier token to the run state. Returns the updated
# state. The caller renders the TUI afterwards. `time_bonus_callback` is
# an optional function that receives the seconds delta -- the loop uses
# it to extend `end_time` directly since end_time is held outside state.
apply_modifier <- function(state, token, time_bonus_callback = NULL) {
  m <- LIVE_MODIFIERS[[token]]
  if (is.null(m)) return(state)

  state$consumed_modifiers <- unique(c(state$consumed_modifiers %||% character(),
                                       token))
  if (is.null(state$bias)) state$bias <- empty_bias()

  # Field-wise merge: same-kind injection replaces, different-kind stacks.
  if (!is.null(m$bias$family))      state$bias$family      <- m$bias$family
  if (!is.null(m$bias$perturb_tag)) state$bias$perturb_tag <- m$bias$perturb_tag
  if (isTRUE(m$bias$escalate))       state$bias$escalate    <- TRUE

  if (is.function(time_bonus_callback)) {
    time_bonus_callback(m$time_bonus)
  }

  state$last_modifier <- list(token = token,
                              time_bonus = m$time_bonus,
                              bias = m$bias)
  state
}

# ---- Tactical pause -------------------------------------------------
#
# The user-facing readline interrupt. The shoot() loop calls this at
# escalation moments (mascot state transitions); the heartbeat freezes
# while the prompt is open, the user either types a +command or just
# hits Enter to skip, and the loop resumes.
#
# Returns list(state, end_time, applied). `applied` is the token that
# landed (NA_character_ when the prompt was skipped or rejected). The
# caller is responsible for updating its own end_time -- we return the
# new value so the caller can rebind it without state-list mutation.
#
# `prompt_fn` is injectable for testing (default: base::readline). In
# non-interactive sessions tactical_pause is a no-op.

tactical_pause <- function(state, end_time, ui,
                            career_level = "Junior Researcher",
                            prompt_fn = NULL) {
  if (!interactive() && is.null(prompt_fn)) {
    return(list(state = state, end_time = end_time, applied = NA_character_))
  }
  if (is.null(prompt_fn)) prompt_fn <- readline

  consumed  <- state$consumed_modifiers %||% character()
  available <- available_modifiers(consumed = consumed,
                                    career_level = career_level)
  if (!length(available)) {
    return(list(state = state, end_time = end_time, applied = NA_character_))
  }

  # Render via the TUI session when one is open; otherwise fall back to
  # plain stdout (covers non-interactive test runs with a mocked prompt_fn).
  emit <- function(text) {
    if (!is.null(ui)) ui_loading(ui, text) else cat(text, "\n", sep = "")
  }

  # The MODIFIERS row in the TUI already shows the live-available
  # tokens; the prompt itself is just the cue + input. Empty input skips.
  emit("METHODOLOGICAL PRIORITY WINDOW")

  buf   <- prompt_fn("+ ")
  token <- parse_mod_command(buf)

  if (is.na(token)) {
    emit("Submission abandoned.")
    return(list(state = state, end_time = end_time, applied = NA_character_))
  }
  if (!(token %in% available)) {
    emit(sprintf("+%s unavailable (consumed or career-locked).", token))
    return(list(state = state, end_time = end_time, applied = NA_character_))
  }

  new_end_time <- end_time
  state <- apply_modifier(state, token,
                          time_bonus_callback = function(secs) {
                            new_end_time <<- new_end_time + secs
                          })

  emit(sprintf("+%s applied. %s Deadline +%ds.",
               token,
               format_bias_announcement(state$last_modifier$bias),
               state$last_modifier$time_bonus))

  list(state = state, end_time = new_end_time, applied = token)
}

# One-line description of the bias a modifier imposes, used in the
# announcement that follows a successful injection.
format_bias_announcement <- function(bias) {
  if (isTRUE(bias$escalate)) return("Desperation escalation engaged.")
  if (!is.null(bias$family))
    return(sprintf("%s priority added.", toupper(bias$family)))
  if (!is.null(bias$perturb_tag))
    return(sprintf("Perturbation bias toward %s.", bias$perturb_tag))
  ""
}

# ---- Bias resolution for the search loop ---------------------------

# When a family bias is active, bypass select_family() and return the
# spec$family record directly. Mirrors the per-family pickers in
# family_select.R, but with the family chosen instead of sampled.
force_family <- function(family_token, outcome_vec) {
  outcome_kind <- classify_outcome(outcome_vec)
  switch(family_token,
    glm  = pick_glm_family(outcome_kind, flail = FALSE),
    gam  = pick_gam_family(outcome_kind),
    glmm = pick_glmm_family(outcome_kind),
    sem  = pick_sem_family(),
    cor  = pick_cor_family(),
    wls  = pick_wls_family(),
    list(fitter = "lm")
  )
}

# Single entry point for "what family should this spec be fit with?".
# Honors the active family bias when one is set and the requested family
# is available at the player's career tier; otherwise delegates to the
# regular weighted random selector.
pick_spec_family <- function(state, career_level, outcome_vec, family_pool,
                              escalating = FALSE) {
  fam <- state$bias$family %||% NA_character_
  if (!is.na(fam) && fam %in% family_pool) {
    return(force_family(fam, outcome_vec))
  }
  select_family(career_level, escalating = escalating,
                outcome = outcome_vec, available = family_pool)
}

test_that("new cursed-bestiary modifiers parse and apply", {
  new_mods <- c("aggregate", "compilation", "gapfill", "tier2",
                "counter_argument", "pseudo_spatial", "omit_control",
                "knife_edge", "iid_rescue", "defer_fix", "coffee_break",
                "mechanism_rebuttal", "misunderstanding", "narrow_answer",
                "complexity_shield", "frame_shift", "matter_of_taste")
  for (tok in new_mods) {
    expect_identical(texanshootR:::parse_mod_command(paste0("+", tok)), tok)
    expect_true(tok %in% names(texanshootR:::LIVE_MODIFIERS))
  }
})

test_that("modifier aliases resolve to their canonical token", {
  pmc <- texanshootR:::parse_mod_command
  expect_identical(pmc("+agg"),          "aggregate")
  expect_identical(pmc("+comp"),         "compilation")
  expect_identical(pmc("+gap"),          "gapfill")
  expect_identical(pmc("+relax"),        "tier2")
  expect_identical(pmc("+preempt"),      "counter_argument")
  expect_identical(pmc("+coords"),       "pseudo_spatial")
  expect_identical(pmc("+omit"),         "omit_control")
  expect_identical(pmc("+labeling"),     "knife_edge")
  expect_identical(pmc("+iid"),          "iid_rescue")
  expect_identical(pmc("+defer"),        "defer_fix")
  expect_identical(pmc("+next_step"),    "defer_fix")
  expect_identical(pmc("+side_meeting"), "coffee_break")
  expect_identical(pmc("+offline"),      "coffee_break")
  expect_identical(pmc("+intuition"),    "mechanism_rebuttal")
  expect_identical(pmc("+but_biologically"), "mechanism_rebuttal")
  expect_identical(pmc("+you_misread"),  "misunderstanding")
  expect_identical(pmc("+strawman"),     "narrow_answer")
  expect_identical(pmc("+its_complex"),  "complexity_shield")
  expect_identical(pmc("+oscillate"),    "frame_shift")
  expect_identical(pmc("+just_one_way"), "matter_of_taste")
})

test_that("applying a new modifier records the consumption and bias", {
  for (tok in c("aggregate", "pseudo_spatial", "knife_edge")) {
    state <- texanshootR:::apply_modifier(list(), tok)
    expect_true(tok %in% state$consumed_modifiers)
    expect_identical(state$bias$perturb_tag,
                     texanshootR:::LIVE_MODIFIERS[[tok]]$bias$perturb_tag)
  }
})

# Daily seed flavor: the calendar day picks a small thematic
# adjustment to which message tags are weighted. Pure cosmetic; no
# leaderboard, no comparison.

daily_flavor <- function(date = Sys.Date()) {
  wday <- as.integer(format(as.Date(date), "%u"))  # 1 = Mon
  switch(wday,
    list(label = "Manuscript Monday",   tag_boost = "narrative"),
    list(label = "Tabulation Tuesday",  tag_boost = "p_hacking"),
    list(label = "Wrap-up Wednesday",   tag_boost = "post_hoc"),
    list(label = "Throughput Thursday", tag_boost = "data_dredging"),
    list(label = "Final Friday",        tag_boost = "optional_stopping"),
    list(label = "Synthesis Saturday",  tag_boost = "story_first"),
    list(label = "Submission Sunday",   tag_boost = "publication_bias")
  )
}

#' texanshootR: Reproducible Audit Trails for Indefensible Research
#'
#' A structured, terminal-first interface for exploratory linear-model
#' search. The main entry point is [shoot()], which fits a battery of
#' candidate specifications across predictor subsets, transformations,
#' interactions, and outlier-removal seeds. Persistent career state
#' tracks run history, awards achievements, and unlocks output
#' generators ([manuscript()], [presentation()],
#' [reviewer_response()], [graphical_abstract()], [funding()]).
#'
#' @section Persistence:
#' On the first interactive save, the package prompts before writing
#' to [tools::R_user_dir()]. Set
#' `options(texanshootR.save_enabled = FALSE)` to opt out entirely.
#'
#' @section Reproducibility:
#' [shoot()] accepts a `seed` argument; every run records its seed,
#' R version, package version, and search-grid hash so past runs can
#' be replayed.
#'
#' @keywords internal
#' @importFrom stats runif sd cor complete.cases na.omit model.frame
#' @importFrom Rcpp evalCpp
#' @useDynLib texanshootR, .registration = TRUE
"_PACKAGE"

# tx_run S3 class. Returned (invisibly) by shoot(); consumed by all
# output functions and by `run_log()`.

new_tx_run <- function(...) {
  obj <- list(...)
  class(obj) <- "tx_run"
  obj
}

#' @export
#' @method print tx_run
print.tx_run <- function(x, ...) {
  # Single read of meta -- every section that needs it (career impact,
  # OUTPUTS, PUBLICATION PIPELINE) shares this snapshot. Reading it
  # once also means the banner reflects the state at print time, not
  # the state from when the run finalized (the player may have called
  # output generators or claimed chain stages in between).
  meta <- tryCatch(read_meta(), error = function(e) NULL)

  active_chain <- !is.null(meta) && !is.null(meta$active_chain) &&
                  identical(meta$active_chain$run_id, x$run_id)
  final_state  <- compute_final_state(x, active_chain)

  cat(banner_header(x, final_state), "\n", sep = "")
  cat("\n")

  cat("RUN\n")
  cat("status: ",         format_run_status(x),     "\n", sep = "")
  cat("career impact: ",  format_career_impact(x),  "\n", sep = "")
  cat("mascot state: ",   format_mascot_arc(x, final_state), "\n", sep = "")
  cat("\n")

  cat("FINDING\n")
  for (line in format_finding_block(x)) cat(line, "\n", sep = "")
  cat("\n")

  cat("SEARCH\n")
  for (line in format_search_block(x)) cat(line, "\n", sep = "")
  cat("\n")

  cat("OUTPUTS\n")
  for (line in format_outputs_block(meta)) cat(line, "\n", sep = "")
  cat("\n")

  if (active_chain) {
    cat("PUBLICATION PIPELINE\n")
    for (line in format_pipeline_block(meta)) cat(line, "\n", sep = "")
    cat("\n")
  }

  cat("DATA USED\n")
  for (line in format_data_used_block(x)) cat(line, "\n", sep = "")

  invisible(x)
}

# Banner header: double-rule, face+gun, "texanshootR :: run NNNN",
# double-rule. The face is the resting state at print time -- "polishing"
# when the printed run owns the active chain (the player is between
# stages), "resolved" for any other shippable run, and the run's
# `peak_mascot` for non-shippable runs (the worst face they wore).
banner_header <- function(x, final_state) {
  rule <- strrep("=", 64)
  face <- tryCatch(read_face(final_state), error = function(e) "( o_o)")
  gun  <- "\ufe3b\u30c7\u2550\u4e00  \u00b7"  # canonical frame from inst/ascii/heartbeat.txt
  idx  <- as.integer(x$run_index %||% 0L)
  title <- sprintf("texanshootR :: run %04d", idx)
  paste(rule, paste(face, gun), title, rule, sep = "\n")
}

compute_final_state <- function(x, active_chain) {
  if (isTRUE(x$shippable) && active_chain) return("polishing")
  if (isTRUE(x$shippable))                  return("resolved")
  x$peak_mascot %||% "composed"
}

format_run_status <- function(x) {
  if (isTRUE(x$shippable)) "publishable result identified"
  else                     "no publishable narrative this run"
}

# Three-state career-impact verdict:
#   promoted  = career tier changed during update_career() of this run
#   advanced  = shippable run, chain opened, no promotion
#   stalled   = not shippable (no chain opened, no XP path)
format_career_impact <- function(x) {
  if (!isTRUE(x$shippable))  return("stalled")
  if (isTRUE(x$promoted))    return("promoted")
  "advanced"
}

# `peak_pre_resolve -> final` arc. Tells the actual emotional shape of
# the run: composed->resolved is a glide, desperate->resolved is a save,
# *->desperate is a defeat. The arc reports the RUN's outcome, not the
# player's current stance -- so "polishing" (which is the resting face
# shown atop the banner when the run owns an active chain) maps back
# to "resolved" here. The top-line face still shows polishing.
format_mascot_arc <- function(x, final_state) {
  arc_end <- if (identical(final_state, "polishing")) "resolved" else final_state
  pre <- x$peak_mascot_before_resolve %||% x$peak_mascot %||% "composed"
  sprintf("%s \u2192 %s", pre, arc_end)
}

format_finding_block <- function(x) {
  hs <- x$highlighted_spec
  if (isTRUE(x$shippable) && !is.null(hs)) {
    c(
      hs$formula %||% "",
      sprintf("methods: %s", methods_summary(hs)),
      sprintf("p: %s",
              if (is.finite(hs$p_value %||% NA_real_))
                format.pval(hs$p_value, digits = 3)
              else "\u2014")
    )
  } else if (!is.null(hs)) {
    c(
      sprintf("closest formula: %s", hs$formula %||% ""),
      sprintf("best p seen: %s",
              if (is.finite(hs$p_value %||% NA_real_))
                format.pval(hs$p_value, digits = 3)
              else "\u2014"),
      sprintf("searched: %s specs",
              format(x$spec_count %||% 0L, big.mark = ","))
    )
  } else {
    "(no candidate specifications materialised)"
  }
}

format_search_block <- function(x) {
  fams <- x$search$families_explored %||% character()
  mods <- x$modifiers_used %||% character()
  c(
    sprintf("candidate specifications: %s",
            format(x$spec_count %||% 0L, big.mark = ",")),
    sprintf("model families explored: %s",
            if (length(fams)) paste(fams, collapse = " ") else "none"),
    sprintf("modifiers used: %s",
            if (length(mods)) paste0("+", mods, collapse = " ") else "none")
  )
}

# OUTPUTS block. Walks CHAIN_STAGES against meta$progression$length_unlocked
# and CHAIN_TIER_BY_LENGTH. Unlocked stages stack as `available now:
# fn(run)` with indented continuation. Locked stages render under
# `locked:` in the deadpan two-line format already established for
# `tx_locked` errors: `fn() requires <tier>`. Locked function names are
# width-padded so the requires-column lines up.
format_outputs_block <- function(meta) {
  prog <- progression_of(meta %||% list())
  n_unlocked <- prog$length_unlocked
  stages <- CHAIN_STAGES
  unlocked <- stages[seq_len(n_unlocked)]
  locked   <- if (n_unlocked < length(stages))
                stages[(n_unlocked + 1L):length(stages)]
              else character()

  lines <- character()

  if (length(unlocked)) {
    label   <- "available now: "
    pad     <- strrep(" ", nchar(label))
    lines   <- c(lines,
                 paste0(label, sprintf("%s(run)", unlocked[1])))
    for (s in unlocked[-1]) {
      lines <- c(lines, paste0(pad, sprintf("%s(run)", s)))
    }
  }

  if (length(locked)) {
    if (length(unlocked)) lines <- c(lines, "")
    lines <- c(lines, "locked:")
    # Width-pad fn() column so `requires <tier>` aligns across rows.
    fns <- sprintf("%s()", locked)
    width <- max(nchar(fns))
    for (i in seq_along(locked)) {
      idx <- match(locked[i], stages)
      tier <- CHAIN_TIER_BY_LENGTH[[idx]]
      lines <- c(lines,
                 sprintf("%-*s requires %s", width, fns[i], tier))
    }
  }

  if (!length(lines)) lines <- "(no stages defined)"
  lines
}

# PUBLICATION PIPELINE block. Only invoked when `meta$active_chain$run_id`
# matches the printed run -- i.e. the player is mid-chain on THIS run.
# Reports the next due stage and the claimed / unlocked ratio.
format_pipeline_block <- function(meta) {
  ac   <- meta$active_chain
  prog <- progression_of(meta)
  due  <- CHAIN_STAGES[[ac$stage_idx]]
  done <- length(ac$completed_stages %||% character())
  c(
    sprintf("next available output: %s(run)", due),
    sprintf("pipeline progress: %d / %d", done, prog$length_unlocked)
  )
}

format_data_used_block <- function(x) {
  hs        <- x$highlighted_spec
  excl_rule <- render_restriction_phrase(hs$restriction)
  if (is.na(excl_rule)) excl_rule <- "none"
  outcome_phrase <- if (is.null(hs$outcome_construction)) "none"
                    else if (identical(hs$outcome_construction$kind,
                                        "composite_index")) {
                      vars <- hs$outcome_construction$vars %||% character()
                      if (length(vars)) sprintf("composite outcome from %s",
                                                 oxford_join(vars))
                      else "none"
                    } else {
                      o <- render_outcome_phrase(hs$outcome_construction, "")
                      if (is.na(o)) "none" else o
                    }
  rows_excluded <- as.integer(hs$rows_excluded %||% 0L)

  stored <- !is.null(save_dir()) && isTRUE(opt("texanshootR.save_enabled"))

  c(
    sprintf("exclusion rule: %s", excl_rule),
    sprintf("rows excluded: %s", format(rows_excluded, big.mark = ",")),
    sprintf("outcome construction: %s", outcome_phrase),
    sprintf("derived metrics: %s",
            if (isTRUE(x$derived_used)) "enabled" else "disabled"),
    sprintf("search seed: %s",
            if (!is.null(x$seed)) as.character(x$seed) else "none"),
    sprintf("run stored: %s", if (isTRUE(stored)) "yes" else "no")
  )
}

# Parser-based formula split. Kept for callers that need the
# outcome / predictors decomposition (e.g. future banner extensions).
# Returns empty strings on parse failure.
parse_formula_parts <- function(formula_str) {
  f <- tryCatch(stats::as.formula(formula_str), error = function(e) NULL)
  if (is.null(f) || length(f) < 3L) {
    return(list(outcome = "", predictors = ""))
  }
  list(
    outcome    = paste(deparse(f[[2]]), collapse = " "),
    predictors = paste(deparse(f[[3]]), collapse = " ")
  )
}

#' @export
#' @method summary tx_run
summary.tx_run <- function(object, ...) {
  print(object)
  cat("\n")
  cat("Search dimensions explored:\n")
  s <- object$search
  if (!is.null(s)) {
    cat(sprintf("  Predictor subsets:     %d\n",  s$subset_count %||% 0))
    cat(sprintf("  Transformations:       %d\n",  s$transform_count %||% 0))
    cat(sprintf("  Interactions:          %d\n",  s$interaction_count %||% 0))
    cat(sprintf("  Restrictions:          %d\n",  s$restriction_count %||% 0))
    cat(sprintf("  Outcome constructions: %d\n",  s$outcome_count %||% 0))
    cat(sprintf("  Model forms:           %d\n",  s$model_form_count %||% 0))
    if (isTRUE(object$derived_used)) {
      cat("  Derived metric:        active\n")
    }
  }
  cat("\nInternal scores (this run):\n")
  m <- object$modifiers %||% list()
  scoreline <- function(name, key) {
    cat(sprintf("  %-22s %+0.2f\n", paste0(name, ":"),
                m[[key]] %||% 0))
  }
  scoreline("Reviewer resistance",    "reviewer_resistance")
  scoreline("Narrative coherence",    "narrative_strength")
  scoreline("Presentation score",     "presentation_pressure")
  if (length(object$achievements_awarded)) {
    cat("\nAchievements awarded this run:\n")
    for (a in object$achievements_awarded) {
      cat("  - ", a, "\n", sep = "")
    }
  }
  invisible(object)
}

#' @export
#' @method format tx_run
format.tx_run <- function(x, ...) {
  paste0("<tx_run ", x$run_id, ">")
}

#' Browse persisted run history
#'
#' Returns a data.frame of recent runs with one row per run. The full
#' run record can be loaded via `attr(run_log(), "records")[[i]]`.
#'
#' @param n Maximum number of runs to list.
#' @return A data.frame with one row per run.
#' @export
run_log <- function(n = 25L) {
  recs <- recent_run_records(n)
  if (length(recs) == 0L) {
    out <- data.frame(
      run_id     = character(),
      timestamp  = character(),
      specs      = integer(),
      best_p     = numeric(),
      reviewer   = character(),
      stringsAsFactors = FALSE
    )
    attr(out, "records") <- list()
    return(out)
  }
  out <- data.frame(
    run_id    = vapply(recs, `[[`, "", "run_id"),
    timestamp = vapply(recs, `[[`, "", "timestamp"),
    specs     = vapply(recs, function(r) as.integer(r$spec_count %||% 0L), integer(1)),
    best_p    = vapply(recs, function(r) {
                  p <- r$highlighted_spec$p_value %||% NA_real_
                  as.numeric(p)
                }, numeric(1)),
    reviewer  = vapply(recs, function(r) r$reviewer_outcome %||% NA_character_, character(1)),
    stringsAsFactors = FALSE
  )
  attr(out, "records") <- recs
  out
}

# Derived-metric escalation: composite-response constructors.
#
# Triggered when the main search produces no spec with p <= 0.05 and
# the run is still within budget. Each constructor builds a new
# outcome column derived from existing inputs in a way that injects
# structural dependence with the predictors. The package treats these
# as "advanced interpretability transformations" and reports the
# resulting spec faithfully.

derived_constructors <- list(

  derive_per_unit = function(df, response, candidates) {
    denom <- pick_numeric(df, candidates, response)
    if (is.null(denom)) return(NULL)
    new_y <- df[[response]] / df[[denom]]
    list(name = "per_unit",
         expr = sprintf("%s / %s", response, denom),
         denom = denom,
         numerator = response,
         y = new_y)
  },

  derive_density = function(df, response, candidates) {
    area_var <- pick_by_name(df, candidates, c("area", "extent", "size", "patch"))
    if (is.null(area_var)) return(NULL)
    new_y <- df[[response]] / df[[area_var]]
    list(name = "density",
         expr = sprintf("%s / %s", response, area_var),
         denom = area_var,
         numerator = response,
         y = new_y)
  },

  derive_effort_adjusted = function(df, response, candidates) {
    eff <- pick_by_name(df, candidates,
                        c("effort", "sampling", "duration", "time"))
    if (is.null(eff)) return(NULL)
    new_y <- df[[response]] / df[[eff]]
    list(name = "effort_adjusted",
         expr = sprintf("%s / %s", response, eff),
         denom = eff,
         numerator = response,
         y = new_y)
  },

  derive_log_index = function(df, response, candidates) {
    scale_var <- pick_numeric(df, candidates, response)
    if (is.null(scale_var)) return(NULL)
    if (any(df[[response]] <= 0, na.rm = TRUE)) return(NULL)
    if (any(df[[scale_var]] <= 0, na.rm = TRUE)) return(NULL)
    new_y <- log(df[[response]]) / log(df[[scale_var]])
    list(name = "log_index",
         expr = sprintf("log(%s) / log(%s)", response, scale_var),
         denom = scale_var,
         numerator = response,
         y = new_y)
  }
)

pick_numeric <- function(df, candidates, exclude) {
  pool <- setdiff(candidates, exclude)
  pool <- pool[vapply(pool, function(v) {
    is.numeric(df[[v]]) && all(df[[v]] != 0, na.rm = TRUE)
  }, logical(1))]
  if (length(pool) == 0L) NULL else pool[1]
}

pick_by_name <- function(df, candidates, hints) {
  pool <- setdiff(candidates, character())
  pool <- pool[vapply(pool, function(v) is.numeric(df[[v]]), logical(1))]
  for (h in hints) {
    hit <- pool[grepl(h, pool, ignore.case = TRUE)]
    if (length(hit)) return(hit[1])
  }
  NULL
}

# Apply a derived constructor to df: replaces the response column
# with the new outcome and returns the modified df + spec metadata.
apply_derived <- function(df, response, predictors, max_attempts = 3L) {
  for (fn_name in names(derived_constructors)) {
    fn <- derived_constructors[[fn_name]]
    out <- fn(df, response, predictors)
    if (is.null(out)) next
    if (!all(is.finite(out$y))) next
    new_df <- df
    new_df[[response]] <- out$y
    return(list(df = new_df, info = out))
  }
  NULL
}

#' Run an exploratory model search
#'
#' Fits a battery of candidate specifications across predictor subsets,
#' transformations, interactions, principled-sounding sample
#' restrictions (complete cases, IQR fences, Cook's D, factor-level
#' restrictions), and outcome-engineering moves (composite indices,
#' residualisation, ratios, within-group z-scoring). Returns a `tx_run`
#' object summarising the search and the highlighted specification.
#'
#' Every shoot has a wall-clock budget (default 30s). Modifiers are
#' pre-rolled at run start by default: shoot() picks a random per-
#' transition loadout (`+glmm`, `+derived_metrics`, ...) and applies
#' each one when the matching mascot state transition fires, extending
#' the deadline by the modifier's per-token bonus (2-5s) and redirecting
#' the search priority. Pass `interactive_modifiers = TRUE` to opt into
#' the readline tactical-prompt window at each transition instead.
#' Without `+derived_metrics`, runs that fail to clear `p <= 0.05`
#' simply lose — the highlighted spec stays above threshold and the
#' output gauntlet does not open.
#'
#' Output flags (`abstract`, `manuscript`, `presentation` /
#' `powerpoint`, `reviewer_response` / `reviewer`, `graphical_abstract`
#' / `graphical`, `funding`) auto-generate the matching file when the
#' run is shippable. The chain prefix needed to reach the highest
#' enabled flag is generated in order: e.g. `presentation = TRUE`
#' produces abstract, manuscript, and presentation.
#'
#' @param df A data frame.
#' @param formula Optional formula. When `NULL`, the first numeric
#'   column is used as the outcome and all other columns as predictors.
#' @param seed Integer seed. When `NULL`, a random seed is generated
#'   and stored on the returned run.
#' @param depth `"default"` for the full search; `"demo"` for a
#'   single-fit smoke test used in CRAN-safe examples.
#' @param terminal When `TRUE`, spawn an external Windows Terminal (or
#'   fallback console window) and run `shoot()` inside it so the full
#'   ANSI multi-zone TUI renders. Useful when calling from RStudio
#'   Console, which only supports `\r`-overwrite and so falls back to
#'   the compressed single-line status. The result is returned to the
#'   calling session as usual. Windows-only.
#' @param interactive_modifiers When `TRUE`, open a `readline()`
#'   tactical-prompt window at each mascot transition so the player
#'   can type `+command` modifiers live. Default `FALSE`: modifiers
#'   are pre-rolled at run start and applied autonomously, which keeps
#'   the in-place animation intact in RStudio's console.
#' @param abstract,manuscript,presentation,reviewer_response,graphical_abstract,funding
#'   Logical flags. When `TRUE` and the run is shippable, auto-generate
#'   the matching output and every chain stage required to reach it.
#'   Default `FALSE`.
#' @param ... Reserved for future arguments and aliases. Recognised
#'   aliases: `powerpoint` (= `presentation`), `reviewer` (=
#'   `reviewer_response`), `graphical` (= `graphical_abstract`).
#'   Internal: `prompt_fn` can be supplied to drive tactical-pause
#'   input non-interactively (tests).
#'
#' @return A `tx_run` object. Returned visibly so the
#'   `print()` banner fires at the prompt; the demo path
#'   (`depth = "demo"`) returns invisibly because there is no banner
#'   worth showing for a one-fit smoke test.
#' @export
shoot <- function(df,
                  formula    = NULL,
                  seed       = NULL,
                  depth      = c("default", "demo"),
                  terminal   = FALSE,
                  interactive_modifiers = FALSE,
                  abstract           = FALSE,
                  manuscript         = FALSE,
                  presentation       = FALSE,
                  reviewer_response  = FALSE,
                  graphical_abstract = FALSE,
                  funding            = FALSE,
                  ...) {
  depth <- match.arg(depth)
  if (!is.data.frame(df)) stop("`df` must be a data frame.", call. = FALSE)

  if (isTRUE(terminal)) {
    return(spawn_shoot_in_terminal(df, formula = formula, seed = seed,
                                    depth = depth, ...))
  }

  # Internal test seam: `prompt_fn` drives tactical_pause without a real
  # terminal. Supplying it implies the test wants the readline path,
  # so it forces interactive_modifiers regardless of the caller's flag.
  dots <- list(...)
  prompt_fn <- dots$prompt_fn
  if (!is.null(prompt_fn)) interactive_modifiers <- TRUE

  # Output-flag aliases. Fold them onto the canonical names so the rest
  # of the function only has to look at one set of names.
  if (isTRUE(dots$powerpoint)) presentation       <- TRUE
  if (isTRUE(dots$reviewer))   reviewer_response  <- TRUE
  if (isTRUE(dots$graphical))  graphical_abstract <- TRUE
  output_flags <- list(
    abstract           = isTRUE(abstract),
    manuscript         = isTRUE(manuscript),
    presentation       = isTRUE(presentation),
    reviewer_response  = isTRUE(reviewer_response),
    graphical_abstract = isTRUE(graphical_abstract),
    funding            = isTRUE(funding)
  )

  if (is.null(seed)) seed <- sample.int(.Machine$integer.max - 1L, 1L)
  seed <- as.integer(seed)
  set.seed(seed)

  parsed <- parse_shoot_inputs(df, formula, depth)
  outcome    <- parsed$outcome
  predictors <- parsed$predictors

  # Family pool is fixed for the run: career-tier driven, with glm
  # gated on Postdoc+. Resolved once so selection inside the loop is
  # cheap. Outcome vector is captured here too so the selector can
  # check family/outcome compatibility without re-touching df.
  career_level   <- current_career_level()
  family_pool    <- available_families(career_level)
  outcome_vec    <- if (outcome %in% names(df)) df[[outcome]] else NULL

  ui   <- NULL
  open_tui <- is_interactive_ui() && depth != "demo"
  if (open_tui) ui <- ui_session_open()

  # Wall-clock budget. Base is 30s. Modifiers extend the deadline when
  # they are injected mid-run via tactical_pause (per-modifier 2-5s).
  # `texanshootR.budget` overrides the base entirely (used by tests
  # to keep the suite fast).
  base_budget <- getOption("texanshootR.budget", 30L)
  budget <- as.integer(base_budget)

  state <- list(
    seed              = seed,
    started           = Sys.time(),
    spec_count        = 0L,
    derived_used      = FALSE,
    modifiers         = list(),
    consumed_modifiers = character(),
    bias              = empty_bias(),
    events            = list(),
    displayed_messages = character(),
    recent_buf        = read_recent_buffer(),
    combo_state       = NULL,
    # Tracks which family is currently being fit so the message bank
    # can surface family-specific blips/loadings via the
    # model_family_affinity filter. NULL means "no family chosen yet";
    # universal messages still fire.
    current_family    = NULL,
    mascot            = "composed",
    last_loading_at   = 0,
    last_blip_at      = 0,
    last_heartbeat_at = 0,
    tick              = 0L,
    stopped_early     = FALSE,
    resolved_at_progress = NA_real_,
    ultra_rare_seen   = FALSE,
    peak_severity     = 0L,
    # Peak severity hit BEFORE the run first cleared p<=0.05. This is
    # the "true" emotional arc: a run that resolved at progress 0.05
    # and then animated through to desperate has peak_severity =
    # desperate (the visible peak) but peak_severity_before_resolve =
    # composed (what the searcher actually felt during the work). The
    # arc summary reads off this so the banner reflects what the run
    # did rather than how long the bar animated.
    peak_severity_before_resolve = 0L
  )

  # Demo path: one fit, no theatre, fast.
  if (depth == "demo") {
    spec <- new_spec(utils::head(predictors, 2L))
    spec$subset_full <- paste(predictors, collapse = ",")
    spec$family <- select_family(career_level, escalating = FALSE,
                                  outcome   = outcome_vec,
                                  available = family_pool)
    one <- fit_spec(df, outcome, spec)
    state$spec_count <- 1L
    return(invisible(finalize_run(state, df, outcome, predictors,
                                   results = list(one), trace = list(spec),
                                   ui = NULL, open_tui = FALSE,
                                   modifiers_used = character())))
  }

  # Fit count cap is deterministic in `budget` -- same seed + same
  # budget yields the same trace (and therefore the same grid_hash).
  # Wall-clock pacing happens *on top of* this: once the cap is hit the
  # loop keeps animating the mascot and bar until end_time, but no
  # more specs are fit.
  cap <- spec_cap_for_budget(budget)
  search <- new_search_state(seed_specs(df, outcome, predictors))
  trace   <- list()
  results <- list()
  end_time <- state$started + budget

  # Chain unlock state (frozen at run start). The OUTPUTS row uses this
  # to dim locked stages; career only advances on finalize_run, so a
  # single read here is correct for the full run's worth of redraws.
  meta_snap        <- tryCatch(read_meta(), error = function(e) NULL)
  unlocked_length  <- progression_of(meta_snap %||% list())$length_unlocked

  # Initial render of the two new zones. Subsequent updates happen
  # inside the search loop -- ui_set_* has its own cache so identical
  # consecutive renders are no-ops.
  if (open_tui) {
    ui <- ui_set_modifiers(ui,
            available_modifiers(state$consumed_modifiers, career_level))
    ui <- ui_set_outputs(ui, CHAIN_STAGES, unlocked_length)
  }

  # Modifier mode. By default shoot() pre-rolls a per-transition
  # loadout at run start and applies each pick autonomously when its
  # mascot transition fires -- the readline tactical prompt is opt-in
  # via `interactive_modifiers = TRUE`. Pre-roll is the universal
  # default because RStudio's console cannot host a readline without
  # breaking the in-place \r overwrite (the readline echoes on a fresh
  # line and every subsequent dyn_render overwrites that echo instead
  # of the bar), and there is no upside to leaving the bug-prone path
  # on by default for ANSI terminals either.
  auto_picks <- if (!isTRUE(interactive_modifiers)) {
    pre_roll_modifiers(career_level)
  } else NULL
  if (!is.null(auto_picks) && open_tui) {
    rolled <- Filter(function(x) !is.na(x) && nzchar(x), auto_picks)
    if (length(rolled)) {
      ui_loading(ui, sprintf("Auto-loadout: %s",
                              paste0("+", unlist(rolled), collapse = " ")))
    }
  }

  # Mini-batch the search. Each iteration collects up to `batch_size`
  # specs from next_spec(), fits them in one R/C++ crossing via
  # fit_specs_batch(), and then runs record_result() over the batch
  # so the perturb_queue and dead_subsets state stays sequential. The
  # tradeoff: perturbations queued by spec #i in a batch don't get
  # picked up until the *next* batch — search reactivity is coarsened
  # to batch granularity, but the R/C++ overhead drops accordingly.
  iter <- 0L
  batch_size <- getOption("texanshootR.batch_size", 8L)
  # The loop has two phases under one driver:
  #   (a) fit phase: spec_count < cap. Pull a batch, fit it, record
  #       results. This is the only phase that mutates the trace, so
  #       the trace (and grid_hash) stay seed-deterministic regardless
  #       of how fast or slow the host CPU is.
  #   (b) animate-out phase: cap hit, end_time not yet. Idle briefly
  #       so the mascot + bar can fill the rest of the wall-clock
  #       budget. The user explicitly wants the visible run to last
  #       the full budget rather than truncate at the cap.
  while (Sys.time() < end_time) {
    if (state$spec_count >= cap) {
      # Fitting done; just let the animation finish out the budget.
      # 50ms keeps CPU low and is well under the 300ms mascot tick.
      Sys.sleep(0.05)
      # Keep the bar marching toward 100% during the animate-out phase.
      # Without this the bar would freeze at whatever percent the last
      # fitted spec reached, which on small data is a fraction of the
      # budget -- defeating the "visible run lasts the full budget"
      # intent of the cap-and-animate split.
      if (open_tui) {
        ui_progress(ui, elapsed_progress(state$started, budget))
      }
    } else {
      this_batch <- min(batch_size, cap - state$spec_count)
      batch_specs <- vector("list", this_batch)
      n_picked <- 0L
      for (i in seq_len(this_batch)) {
        pick   <- next_spec(search, df, outcome, predictors, iter)
        search <- pick$state
        spec   <- pick$spec
        if (is.null(spec)) break
        spec$subset_full <- paste(predictors, collapse = ",")
        spec$family <- pick_spec_family(state, career_level, outcome_vec,
                                         family_pool, escalating = FALSE)
        spec <- apply_model_form(spec)
        n_picked <- n_picked + 1L
        batch_specs[[n_picked]] <- spec
        iter <- iter + 1L
      }
      if (n_picked == 0L) {
        # Search exhausted before cap. Fall into the animate-out path
        # rather than breaking, so the bar still fills the budget.
        Sys.sleep(0.05)
      } else {
        length(batch_specs) <- n_picked
        batch_results <- fit_specs_batch(df, outcome, batch_specs)

        for (i in seq_len(n_picked)) {
          spec <- batch_specs[[i]]
          r    <- batch_results[[i]]
          state$current_family <- spec$family$fitter
          state$spec_count <- state$spec_count + 1L
          if (!is.null(r)) {
            results[[length(results) + 1L]] <- r
            trace[[length(trace)     + 1L]] <- spec
            search <- record_result(search, spec, r, df, outcome,
                                    bias_perturb = state$bias$perturb_tag)
          }
          # Per-spec bar redraw. The dedup cache in ui_progress()
          # throttles to actual visual changes (integer-percent
          # transitions), so this is cheap when nothing visible has
          # changed.
          if (open_tui) {
            spec_prog <- elapsed_progress(state$started, budget)
            ui_progress(ui, spec_prog)
          }
        }
      }
    }

    progress <- elapsed_progress(state$started, budget)
    best_p <- suppressWarnings(min(vapply(results,
                                          function(rr) rr$p_value %||% NA_real_,
                                          numeric(1)), na.rm = TRUE))
    if (!is.finite(best_p)) best_p <- NA_real_

    cur <- mascot_state(progress, best_p, escalating = state$derived_used)
    sev <- mascot_severity(cur)
    if (sev > state$peak_severity) state$peak_severity <- sev
    # Track the peak severity hit while the searcher was actually
    # still searching -- i.e., before best_p first crossed 0.05.
    # After resolution the loop keeps animating to fill the budget,
    # and the ladder mechanically walks to desperate at progress
    # >= 0.90, which would over-state the difficulty of every run
    # that resolved early.
    if (!state$stopped_early && sev > state$peak_severity_before_resolve) {
      state$peak_severity_before_resolve <- sev
    }

    prev_mascot <- state$mascot
    if (open_tui) {
      state <- maybe_animate(state, ui, progress, results, outcome)
    } else {
      state$mascot <- cur
    }

    # Mascot transition handler. Two paths:
    #   - Auto (default): apply the pre-rolled pick for this transition,
    #     if any, and announce it via the loading slot. No readline,
    #     no blocking, no broken \r overwrite.
    #   - Interactive: open a tactical_pause window with readline so
    #     the player can type +commands. Opt-in via interactive_modifiers
    #     (or implicitly when a test supplies prompt_fn).
    if (!identical(prev_mascot, state$mascot)) {
      if (!is.null(auto_picks)) {
        pick <- auto_picks[[state$mascot]] %||% NA_character_
        if (!is.na(pick) && nzchar(pick)) {
          state <- apply_modifier(state, pick,
                                  time_bonus_callback = function(secs) {
                                    end_time <<- end_time + secs
                                  })
          if (open_tui) {
            ui_loading(ui, sprintf("+%s applied. %s Deadline +%ds.",
                                    pick,
                                    format_bias_announcement(state$last_modifier$bias),
                                    state$last_modifier$time_bonus))
          }
        }
      } else if (is_interactive_ui() || !is.null(prompt_fn)) {
        pres <- tactical_pause(state, end_time, ui = ui,
                                career_level = career_level,
                                prompt_fn    = prompt_fn)
        state    <- pres$state
        end_time <- pres$end_time
        if (open_tui) {
          ui <- ui_set_modifiers(ui,
                  available_modifiers(state$consumed_modifiers, career_level))
        }
      }
    }

    if (!state$stopped_early && is.finite(best_p) && best_p <= 0.05) {
      state$stopped_early <- TRUE
      state$resolved_at_progress <- progress
    }
  }

  # Roll a life event once per run, after the main search.
  ev <- maybe_event(career_level = (read_meta() %||% list())$career_level
                                    %||% "Junior Researcher",
                    run_phase = "late")
  if (!is.null(ev)) {
    state$events[[length(state$events) + 1L]] <- ev
    state <- apply_event_effects(state, ev$effects)
    if (open_tui) ui <- ui_event(ui, ev$event_text, ev$consequence_text)
  }

  # Desperation escalation: if no significant spec, manufacture a
  # composite response and re-flail through wildcard picks against it.
  best_p <- if (length(results)) {
    suppressWarnings(min(vapply(results,
                                function(r) r$p_value %||% NA_real_,
                                numeric(1)), na.rm = TRUE))
  } else NA_real_
  if (!is.finite(best_p)) best_p <- NA_real_

  if (isTRUE(state$bias$escalate) && (is.na(best_p) || best_p > 0.05) &&
      Sys.time() < end_time) {
    derived <- apply_derived(df, outcome, predictors)
    if (!is.null(derived)) {
      state$derived_used <- TRUE
      # Escalation forces the desperate face — record it as a peak even
      # if the run never visually got there.
      state$peak_severity <- max(state$peak_severity,
                                 mascot_severity("desperate"))
      d2 <- derived$df
      sub_cap <- max(50L, cap %/% 10L)
      sub_count <- 0L
      derived_outcome_vec <- if (outcome %in% names(d2)) d2[[outcome]] else NULL
      # Escalation has no per-fit state mutation between specs, so we
      # can batch wildcard specs straight through fit_specs_batch.
      while (sub_count < sub_cap && Sys.time() < end_time) {
        this_batch <- min(batch_size, sub_cap - sub_count)
        batch_specs <- vector("list", this_batch)
        n_picked <- 0L
        for (i in seq_len(this_batch)) {
          spec <- wildcard_spec(d2, outcome, predictors)
          if (is.null(spec)) break
          spec$subset_full <- paste(predictors, collapse = ",")
          spec$family <- pick_spec_family(state, career_level,
                                           derived_outcome_vec, family_pool,
                                           escalating = TRUE)
          spec <- apply_model_form(spec)
          n_picked <- n_picked + 1L
          batch_specs[[n_picked]] <- spec
        }
        if (n_picked == 0L) break
        length(batch_specs) <- n_picked

        batch_results <- fit_specs_batch(d2, outcome, batch_specs)
        for (i in seq_len(n_picked)) {
          spec <- batch_specs[[i]]
          r    <- batch_results[[i]]
          state$current_family <- spec$family$fitter
          sub_count <- sub_count + 1L
          state$spec_count <- state$spec_count + 1L
          if (!is.null(r)) {
            r$derived <- derived$info
            results[[length(results) + 1L]] <- r
            trace[[length(trace)     + 1L]] <- spec
          }
        }
      }
    }
  }

  # Final-frame mascot flash. The in-loop mascot walks the progress
  # ladder regardless of significance -- "resolved" only appears here,
  # as a one-frame snapshot before the TUI tears down, when the run is
  # actually shippable. Done before finalize_run so the user sees the
  # resolved face above whatever chain / promotion banner finalize
  # prints below.
  if (open_tui && length(results)) {
    best_p_final <- suppressWarnings(min(vapply(results,
                    function(rr) rr$p_value %||% NA_real_, numeric(1)),
                    na.rm = TRUE))
    if (is.finite(best_p_final) && best_p_final <= 0.05) {
      ui_set_mascot(ui, "resolved", tick = state$tick + 1L, force = TRUE)
    }
  }

  run <- finalize_run(state, df, outcome, predictors, results, trace, ui,
                      open_tui, modifiers_used = state$consumed_modifiers)

  # Auto-generate any flagged outputs. The chain enforces stage order
  # internally, so we walk CHAIN_STAGES and call each generator the
  # caller asked for plus every earlier stage required to reach it.
  # Skips silently when the run isn't shippable -- the chain isn't open.
  # Returns the run with outputs_generated re-synced from disk.
  run <- generate_flagged_outputs(run, output_flags)

  # Visible return so `shoot(df)` at the prompt auto-prints the
  # tx_run banner (run arc + highlighted spec + chain HUD). Assignment
  # (`run <- shoot(df)`) suppresses auto-print as usual.
  run
}

# Heuristic: about 100 specs per second on a modern CPU for small df.
# Cap at 5000 for safety. The cap is what makes a run's trace
# (and therefore its grid_hash) deterministic in `seed + budget` --
# wall-clock pacing is layered on top to fill the rest of the budget
# with idle animation, not extra fits.
spec_cap_for_budget <- function(budget) {
  min(5000L, max(50L, as.integer(budget * 100L)))
}

# Fraction of the wall-clock budget that has elapsed since the run
# started, clamped to [0, 1]. Drives both the progress bar and the
# mascot state so a fast-fitting df still walks the full emotional arc
# over the budget window.
elapsed_progress <- function(started, budget) {
  min(1, as.numeric(Sys.time() - started, units = "secs") /
         max(1, as.numeric(budget)))
}

parse_shoot_inputs <- function(df, formula, depth) {
  if (is.null(formula)) {
    is_num <- vapply(df, is.numeric, logical(1))
    if (!any(is_num)) {
      stop("Provide a `formula` or a data frame with at least one ",
           "numeric column.", call. = FALSE)
    }
    outcome <- names(df)[which(is_num)[1]]
    predictors <- setdiff(names(df), outcome)
  } else {
    parts <- as.character(stats::terms(formula))
    outcome <- parts[2]
    predictors <- all.vars(formula)[-1]
  }
  list(outcome = outcome, predictors = predictors)
}

# UI tick: redraw heartbeat / blip / loading according to elapsed
# time since the previous update for each cadence.
maybe_animate <- function(state, ui, progress, results, outcome) {
  now <- as.numeric(Sys.time())
  meta <- read_meta()
  career <- (meta %||% list())$career_level %||% "Junior Researcher"
  flavor <- daily_flavor()
  tag_boost <- flavor$tag_boost

  # Best p so far, for mascot state.
  ps <- vapply(results, function(r) r$p_value %||% NA_real_, numeric(1))
  best_p <- if (length(ps)) suppressWarnings(min(ps, na.rm = TRUE))
            else NA_real_
  if (is.infinite(best_p)) best_p <- NA_real_

  m <- mascot_state(progress, best_p, escalating = state$derived_used)

  # Heartbeat: every ~300 ms drives the mascot tick. The progress bar
  # is redrawn per-spec inside the batch loop (see shoot.R), where it
  # gets fresh progress on every fit rather than only at batch boundary.
  if (now - state$last_heartbeat_at >= 0.3) {
    state$tick <- state$tick + 1L
    ui_set_mascot(ui, m, tick = state$tick)
    state$last_heartbeat_at <- now
  }

  # Blip: every 250-500 ms.
  if (now - state$last_blip_at >= runif(1, 0.25, 0.5)) {
    draw <- select_message(
      phase = "blip",
      career = career,
      mascot_state = m,
      model_family = state$current_family,
      recent = recent_ids(state$recent_buf),
      combo_state = state$combo_state,
      tag_boost = tag_boost
    )
    if (!is.null(draw)) {
      ui_blip(ui, draw$text)
      state$recent_buf <- recent_push(state$recent_buf, draw$id)
      state$displayed_messages <- c(state$displayed_messages, draw$id)
      state$combo_state <- if (!is.na(draw$combo_start)) draw$combo_start else NULL
    }
    state$last_blip_at <- now
  }

  # Progress bar + loading line: every 3-5s. Slow enough to read the
  # longest message (~70 chars) once at a relaxed pace; messages
  # shorter than the budget get the same dwell rather than racing past.
  if (now - state$last_loading_at >= runif(1, 3, 5)) {
    draw <- select_message(
      phase = "loading",
      career = career,
      mascot_state = m,
      model_family = state$current_family,
      recent = recent_ids(state$recent_buf),
      combo_state = state$combo_state,
      tag_boost = tag_boost
    )
    if (!is.null(draw)) {
      ui_loading(ui, draw$text)
      state$recent_buf <- recent_push(state$recent_buf, draw$id)
      state$displayed_messages <- c(state$displayed_messages, draw$id)
    }
    state$last_loading_at <- now
  }

  # Ultra-rare "Glimpse of Tenure": once per run at most, low-prob
  # roll per heartbeat tick. Gated on ANSI + animations.
  if (!isTRUE(state$ultra_rare_seen) &&
      isTRUE(opt("texanshootR.animations")) &&
      runif(1) < getOption("texanshootR.glimpse_rate", 1 / 1000)) {
    ultra_rare_glimpse()
    state$ultra_rare_seen <- TRUE
    append_sighting(list(
      run_seed  = state$seed,
      timestamp = format(Sys.time(), "%Y-%m-%dT%H:%M:%S%z"),
      progress  = progress
    ))
  }

  # Mascot transition message.
  if (!identical(state$mascot, m)) {
    draw <- select_message(
      phase = "state_transition",
      mascot_state = m,
      recent = recent_ids(state$recent_buf)
    )
    if (!is.null(draw)) {
      ui_loading(ui, draw$text)
      state$recent_buf <- recent_push(state$recent_buf, draw$id)
      state$displayed_messages <- c(state$displayed_messages, draw$id)
    }
    state$mascot <- m
  }

  state
}

finalize_run <- function(state, df, outcome, predictors, results, trace,
                          ui, open_tui, modifiers_used = character()) {
  highlight <- choose_highlight(results)

  # The reviewer roll has moved to reviewer_response(): a run is a
  # finding, not a verdict, so the reviewer only enters when the
  # player tries to turn the finding into publication bureaucracy.
  # `reviewer_outcome` stays NA on the run until materialize_reviewer_outcome()
  # fires from inside the generator.

  run_id <- format(state$started, "%Y%m%d-%H%M%S")
  search_summary <- summarise_trace(trace)
  shippable <- !is.null(highlight) &&
               is.finite(highlight$p_value %||% NA_real_) &&
               (highlight$p_value %||% 1) <= 0.05

  df_meta <- list(
    nrow    = nrow(df),
    ncol    = ncol(df),
    columns = lapply(names(df),
                     function(n) list(name = n,
                                      class = class(df[[n]])[1])),
    summary = numeric_summary(df)
  )

  run <- new_tx_run(
    run_id            = run_id,
    seed              = state$seed,
    timestamp         = format(state$started, "%Y-%m-%dT%H:%M:%S%z"),
    package_version   = as.character(utils::packageVersion("texanshootR")),
    df_meta           = df_meta,
    formula           = if (!is.null(highlight)) highlight$formula else NA_character_,
    spec_count        = state$spec_count,
    highlighted_spec  = highlight,
    derived_used      = state$derived_used,
    reviewer_outcome  = NA_character_,
    events            = state$events,
    modifiers         = state$modifiers,
    modifiers_used    = modifiers_used,
    shippable         = shippable,
    displayed_message_ids = state$displayed_messages,
    grid_hash         = digest_grid(trace),
    search            = search_summary,
    stopped_early     = state$stopped_early,
    resolved_at_progress = state$resolved_at_progress,
    ultra_rare_seen   = state$ultra_rare_seen,
    peak_mascot       = severity_to_state(state$peak_severity),
    peak_mascot_before_resolve =
      severity_to_state(state$peak_severity_before_resolve),
    outputs_generated = character(),
    outputs_generated_files = character()
  )

  # Persist.
  meta <- read_meta() %||% default_meta(save_dir() %||% tempfile())
  meta <- update_career(meta, run)
  if (length(state$events)) {
    meta$events_witnessed <- c(meta$events_witnessed %||% list(),
                                lapply(state$events, `[[`, "id"))
  }
  # If the run cleared p <= 0.05, open the publication chain. This
  # aborts any prior chain the player left dangling.
  if (isTRUE(shippable)) {
    meta <- open_chain(meta, run$run_id)
  }
  # Stamp the cumulative run number and the promotion flag onto the run
  # AFTER update_career() has incremented meta$runs_count and set the
  # transient `promoted` attribute (which is lost on write_meta because
  # JSON doesn't carry R attributes). The print banner reads these
  # fields for the "texanshootR :: run 0007" header and the RUN section's
  # `career impact:` line.
  run$run_index <- as.integer(meta$runs_count %||% 0L)
  run$promoted  <- isTRUE(attr(meta, "promoted"))
  write_meta(meta)
  write_run_record(run)
  write_recent_buffer(state$recent_buf)

  # Achievement evaluation.
  run$achievements_awarded <- award_and_equip(run, meta)

  if (open_tui) ui_session_close(ui)

  if (isTRUE(shippable)) say_chain_opened(meta)

  if (isTRUE(attr(meta, "promoted"))) {
    say(sprintf("Career advancement detected. %s -> %s",
                attr(meta, "from_level"), attr(meta, "to_level")))
  }

  invisible(run)
}

numeric_summary <- function(df) {
  out <- list()
  for (n in names(df)) {
    x <- df[[n]]
    if (is.numeric(x)) {
      out[[n]] <- list(
        mean = mean(x, na.rm = TRUE),
        sd   = stats::sd(x, na.rm = TRUE),
        min  = suppressWarnings(min(x, na.rm = TRUE)),
        max  = suppressWarnings(max(x, na.rm = TRUE))
      )
    }
  }
  out
}

# Summarise the per-dimension distinct-value counts across the specs
# actually fitted. Replaces the old "grid summary" — there is no grid
# anymore, only the trace of what the shooter ended up trying.
summarise_trace <- function(trace) {
  # Distinct model families the search actually fitted, sorted in
  # registry order so the banner reads "lm glm gam" consistently
  # across runs that happened to hit the same families in different
  # encounter order.
  fams_seen <- unique(vapply(trace,
                              function(s) s$family$fitter %||% "lm",
                              character(1)))
  fams_sorted <- intersect(c("lm", "glm", "gam", "wls", "rlm", "cor",
                              "glmm", "sem"),
                            fams_seen)

  list(
    subset_count      = length(unique(vapply(trace, function(s) paste(s$subset, collapse = ","), ""))),
    transform_count   = length(unique(vapply(trace, function(s) paste(names(s$transforms), s$transforms, sep = ":", collapse = ","), ""))),
    interaction_count = length(unique(unlist(lapply(trace, function(s) s$interactions)))),
    restriction_count = length(unique(vapply(trace,
                          function(s) restriction_digest(s$restriction), ""))),
    outcome_count     = length(unique(vapply(trace,
                          function(s) outcome_construction_digest(s$outcome_construction), ""))),
    model_form_count  = length(unique(vapply(trace,
                          function(s) model_form_digest(s$model_form), ""))),
    families_explored = fams_sorted
  )
}

# Hash the sequence of specs the shooter actually tried. The name
# `digest_grid` is preserved on the run record for backwards
# compatibility, but the input is the trace, not a pre-built grid.
digest_grid <- function(trace) {
  s <- vapply(trace, function(g) {
    paste(c(paste(g$subset, collapse = ","),
            paste(names(g$transforms), g$transforms, sep = ":", collapse = ","),
            paste(g$interactions, collapse = ","),
            restriction_digest(g$restriction),
            outcome_construction_digest(g$outcome_construction),
            model_form_digest(g$model_form)),
          collapse = "|")
  }, "")
  paste(length(trace), substr(rlang_md5(paste(s, collapse = "\n")), 1, 12),
        sep = "-")
}

# Cheap djb2-style string hash to avoid a digest dep. We only use it
# to fingerprint the search trace in the run record - collisions are
# acceptable.
rlang_md5 <- function(x) {
  bytes <- as.integer(charToRaw(x))
  h <- 5381                       # double; avoids 32-bit overflow
  for (b in bytes) {
    h <- (h * 33 + b) %% 2147483647
  }
  sprintf("%08x", as.integer(h))
}

---
title: "Getting started with texanshootR"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with texanshootR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = FALSE
)
```

`texanshootR` is a terminal-first specification-search engine. The main
entry point is `shoot()`. It opens a wall-clock budget, fires at the
specification space, and returns a `tx_run` carrying the highlighted
result and the audit trail.

## A first run

```{r}
library(texanshootR)

run <- shoot(mtcars)
print(run)
summary(run)
```

`shoot()` flails opportunistically across predictor subsets,
transformations, interactions, outlier exclusions, subgroup splits, and
— as the career advances — alternative model families. The print method
leads with the shooter's face and the emotional arc the run followed
(`composed -> resolved`, `panicked -> resolved (last-minute)`,
`escalated to derived metrics`) before the formula and the p-value.

Every run is deterministic given a seed. The seed, R version, package
version, and a hash of the search trace are recorded on the returned
object so the full search can be replayed.

## The publication chain

A finished `tx_run` that clears `p <= 0.05` is *shippable*. The first
shippable run opens a **publication chain** — six output stages,
redeemed in order:

```r
abstract(run)            # one-paragraph deadpan summary
manuscript(run)          # IMRaD draft; Methods match the winning spec
presentation(run)        # 8-slide deck; residual plot on slide 7
reviewer_response(run)   # opens "we thank the reviewer..."
graphical_abstract(run)  # the figure your PI will retweet
funding(run)             # the next grant, citing the just-shipped finding
```

Finish the chain through your currently-unlocked prefix and you
collect a length-bonus on top of the per-stage XP.

The chain breaks — bonus forfeit, partial XP kept — if you fire a
fresh `shoot()` before finishing. Calling the wrong stage (or the
wrong run) signals a `tx_chain_error` but leaves the chain open so
you can retry the correct stage.

Aliases are accepted: `powerpoint` (= `presentation`), `reviewer` (=
`reviewer_response`), `graphical` (= `graphical_abstract`).

### XP, chain length, career tier

The chain prefix you can redeem grows with cumulative XP. Career tier
is a label derived from that prefix:

| Chain length | New stage              | XP needed | Career tier        |
|:------------:|------------------------|:---------:|--------------------|
| 1            | `abstract()`           | 0         | Junior Researcher  |
| 2            | `manuscript()`         | 5         | Postdoc            |
| 3            | `presentation()`       | 15        | Postdoc            |
| 4            | `reviewer_response()`  | 30        | Senior Scientist   |
| 5            | `graphical_abstract()` | 55        | Senior Scientist   |
| 6            | `funding()`            | 90        | PI                 |

The tier label is cosmetic. The chain length is the real gate.

## Auto-generating the chain

Output flags on `shoot()` auto-generate the chain prefix needed to
reach the highest enabled flag, in order. So a shippable run with
`presentation = TRUE` produces `abstract`, `manuscript`, and
`presentation` in one go — provided each one is unlocked:

```{r}
run <- shoot(mtcars, presentation = TRUE)
```

Files land in `tempdir()` by default. Override with `output_dir =` or
`options(texanshootR.output_dir = ...)`. Each generator returns the
file path invisibly.

## Locked stages and broken chains

Locked calls, the wrong run, and out-of-order calls all signal a
structured `tx_chain_error`. A locked stage in a fresh profile
looks like this:

```
manuscript() requires:
5 XP (chain length 2)

Current XP:
0
```

Scripts can branch on the `reason` field:

```{r}
res <- tryCatch(manuscript(run), tx_chain_error = function(e) e)
if (inherits(res, "tx_chain_error")) {
  res$reason   # one of: not_unlocked, no_active_chain,
               #         wrong_run, wrong_stage
}
```

Inspect the live HUD with `progress()`:

```{r}
progress()
```

It prints chain length, current XP, next unlock, what's still locked,
and the active chain window if one is open.

## Career, achievements, cosmetics

```{r}
career()        # tier, runs, favourite method, opaque scores
achievements()  # 20 unlockable badges; hidden ones show as ???
wardrobe()      # equipped cosmetic slots (hat, badge, cloak, poncho, lanyard)
run_log()       # tibble of every run on this profile
```

Hidden achievements appear as `???` until unlocked. New cosmetics
auto-equip when their unlocking achievement fires; override the
selection with `wardrobe(slot, id)`.

## Persistent state

The researcher profile persists under
`tools::R_user_dir("texanshootR", "data")` as flat YAML. The first
interactive save prompts before writing anything to disk. Decline the
prompt, or opt out entirely with:

```{r}
options(texanshootR.save_enabled = FALSE)
```

and the package becomes stateless: every call is independent,
progression sits inert at Junior Researcher, and the chain never
opens beyond `abstract()`.

To pre-consent (CI, scripts, the impatient):

```{r}
options(texanshootR.consent = TRUE)
```

## Resetting

Should circumstances call for a fresh start — a new institution, a
co-author dispute, an opportune hard-drive failure, or because some of
you just want to see the world burn:

```{r}
reset_career(force = TRUE)
reset_achievements(force = TRUE)
reset_wardrobe(force = TRUE)
reset_all(force = TRUE)
```

## Next steps

The [README](https://gillescolling.com/texanshootR/) covers the seven
model families, the message-pack schema, and the design philosophy.
The [reference index](https://gillescolling.com/texanshootR/reference/)
lists every exported function with examples.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/achievements.R
\name{achievements}
\alias{achievements}
\title{Inspect the achievement registry}
\usage{
achievements()
}
\value{
A data.frame.
}
\description{
Returns a data.frame of all achievements with unlock status. Hidden
achievements appear with \code{name = "???"} and \code{hint = NA} until
unlocked.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output_abstract.R
\name{abstract}
\alias{abstract}
\title{Generate a one-paragraph deadpan abstract}
\usage{
abstract(run, output_dir = NULL, file = NULL, force = FALSE)
}
\arguments{
\item{run}{A \code{tx_run} object returned by \code{\link[=shoot]{shoot()}}. Must be the
currently active publication chain — i.e., the run that opened
the chain when it cleared \code{p <= 0.05}.}

\item{output_dir}{Optional output directory.}

\item{file}{Optional filename stem (without extension).}

\item{force}{Overwrite an existing file.}
}
\value{
Character path to the written file (invisible).
}
\description{
The entry-tier output. Always the first stage of the publication
chain (see \code{\link[=progress]{progress()}}). Writes a plain-text \code{.txt} file describing
the highlighted specification in the unbothered register typical of
an applied-stats abstract.
}
\details{
Chain stage: \strong{abstract} (length 1, always unlocked).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reset.R
\name{reset}
\alias{reset}
\alias{reset_career}
\alias{reset_achievements}
\alias{reset_wardrobe}
\alias{reset_all}
\title{Reset persisted texanshootR state}
\usage{
reset_career(force = FALSE)

reset_achievements(force = FALSE)

reset_wardrobe(force = FALSE)

reset_all(force = FALSE)
}
\arguments{
\item{force}{Logical. Skip the confirmation prompt.}
}
\value{
\code{TRUE} invisibly on success, \code{FALSE} if cancelled.
}
\description{
These functions remove pieces of the persistent save written under
\code{\link[tools:userdir]{tools::R_user_dir()}}. They prompt for confirmation in interactive
sessions; pass \code{force = TRUE} to suppress the prompt. Non-interactive
sessions require \code{force = TRUE}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/career.R
\name{career}
\alias{career}
\title{Inspect career state}
\usage{
career()
}
\value{
A \code{tx_career} object.
}
\description{
Returns a \code{tx_career} object describing the persistent career, or a
fresh default if no save exists. The object's \code{print} method shows
a compact dashboard card; \code{summary} adds aggregate counts. Career
tier is derived from the publication-chain length you've unlocked
(see \code{\link[=progress]{progress()}}); XP comes from completed chain stages.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/messages.R
\name{validate_messages}
\alias{validate_messages}
\title{Validate the on-disk message registry}
\usage{
validate_messages(path = NULL)
}
\arguments{
\item{path}{Optional character path to a directory of YAML files.
When \code{NULL}, validates the package's bundled messages.}
}
\value{
A data.frame of the validated registry, returned invisibly.
}
\description{
Lints every YAML file under \verb{inst/messages/} (and an additional path
if supplied), enforcing required fields, vocabulary membership, id
uniqueness, and combo-chain integrity. Used by the test suite and
available to contributors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shoot.R
\name{shoot}
\alias{shoot}
\title{Run an exploratory model search}
\usage{
shoot(
  df,
  formula = NULL,
  seed = NULL,
  depth = c("default", "demo"),
  terminal = FALSE,
  interactive_modifiers = FALSE,
  abstract = FALSE,
  manuscript = FALSE,
  presentation = FALSE,
  reviewer_response = FALSE,
  graphical_abstract = FALSE,
  funding = FALSE,
  ...
)
}
\arguments{
\item{df}{A data frame.}

\item{formula}{Optional formula. When \code{NULL}, the first numeric
column is used as the outcome and all other columns as predictors.}

\item{seed}{Integer seed. When \code{NULL}, a random seed is generated
and stored on the returned run.}

\item{depth}{\code{"default"} for the full search; \code{"demo"} for a
single-fit smoke test used in CRAN-safe examples.}

\item{terminal}{When \code{TRUE}, spawn an external Windows Terminal (or
fallback console window) and run \code{shoot()} inside it so the full
ANSI multi-zone TUI renders. Useful when calling from RStudio
Console, which only supports \verb{\\r}-overwrite and so falls back to
the compressed single-line status. The result is returned to the
calling session as usual. Windows-only.}

\item{interactive_modifiers}{When \code{TRUE}, open a \code{readline()}
tactical-prompt window at each mascot transition so the player
can type \code{+command} modifiers live. Default \code{FALSE}: modifiers
are pre-rolled at run start and applied autonomously, which keeps
the in-place animation intact in RStudio's console.}

\item{abstract, manuscript, presentation, reviewer_response, graphical_abstract, funding}{Logical flags. When \code{TRUE} and the run is shippable, auto-generate
the matching output and every chain stage required to reach it.
Default \code{FALSE}.}

\item{...}{Reserved for future arguments and aliases. Recognised
aliases: \code{powerpoint} (= \code{presentation}), \code{reviewer} (=
\code{reviewer_response}), \code{graphical} (= \code{graphical_abstract}).
Internal: \code{prompt_fn} can be supplied to drive tactical-pause
input non-interactively (tests).}
}
\value{
A \code{tx_run} object. Returned visibly so the
\code{print()} banner fires at the prompt; the demo path
(\code{depth = "demo"}) returns invisibly because there is no banner
worth showing for a one-fit smoke test.
}
\description{
Fits a battery of candidate specifications across predictor subsets,
transformations, interactions, principled-sounding sample
restrictions (complete cases, IQR fences, Cook's D, factor-level
restrictions), and outcome-engineering moves (composite indices,
residualisation, ratios, within-group z-scoring). Returns a \code{tx_run}
object summarising the search and the highlighted specification.
}
\details{
Every shoot has a wall-clock budget (default 30s). Modifiers are
pre-rolled at run start by default: shoot() picks a random per-
transition loadout (\code{+glmm}, \code{+derived_metrics}, ...) and applies
each one when the matching mascot state transition fires, extending
the deadline by the modifier's per-token bonus (2-5s) and redirecting
the search priority. Pass \code{interactive_modifiers = TRUE} to opt into
the readline tactical-prompt window at each transition instead.
Without \code{+derived_metrics}, runs that fail to clear \code{p <= 0.05}
simply lose — the highlighted spec stays above threshold and the
output gauntlet does not open.

Output flags (\code{abstract}, \code{manuscript}, \code{presentation} /
\code{powerpoint}, \code{reviewer_response} / \code{reviewer}, \code{graphical_abstract}
/ \code{graphical}, \code{funding}) auto-generate the matching file when the
run is shippable. The chain prefix needed to reach the highest
enabled flag is generated in order: e.g. \code{presentation = TRUE}
produces abstract, manuscript, and presentation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run.R
\name{run_log}
\alias{run_log}
\title{Browse persisted run history}
\usage{
run_log(n = 25L)
}
\arguments{
\item{n}{Maximum number of runs to list.}
}
\value{
A data.frame with one row per run.
}
\description{
Returns a data.frame of recent runs with one row per run. The full
run record can be loaded via \code{attr(run_log(), "records")[[i]]}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vocab.R
\name{career_compare}
\alias{career_compare}
\title{Compare two career levels.}
\usage{
career_compare(a, b)
}
\arguments{
\item{a, b}{Career level strings.}
}
\value{
Integer: -1 if a < b, 0 if equal, 1 if a > b.
}
\description{
Compare two career levels.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/texanshootR-package.R
\docType{package}
\name{texanshootR-package}
\alias{texanshootR}
\alias{texanshootR-package}
\title{texanshootR: Reproducible Audit Trails for Indefensible Research}
\description{
A structured, terminal-first interface for exploratory linear-model
search. The main entry point is \code{\link[=shoot]{shoot()}}, which fits a battery of
candidate specifications across predictor subsets, transformations,
interactions, and outlier-removal seeds. Persistent career state
tracks run history, awards achievements, and unlocks output
generators (\code{\link[=manuscript]{manuscript()}}, \code{\link[=presentation]{presentation()}},
\code{\link[=reviewer_response]{reviewer_response()}}, \code{\link[=graphical_abstract]{graphical_abstract()}}, \code{\link[=funding]{funding()}}).
}
\section{Persistence}{

On the first interactive save, the package prompts before writing
to \code{\link[tools:userdir]{tools::R_user_dir()}}. Set
\code{options(texanshootR.save_enabled = FALSE)} to opt out entirely.
}

\section{Reproducibility}{

\code{\link[=shoot]{shoot()}} accepts a \code{seed} argument; every run records its seed,
R version, package version, and search-grid hash so past runs can
be replayed.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://gillescolling.com/texanshootR/}
  \item \url{https://github.com/gcol33/texanshootR}
  \item Report bugs at \url{https://github.com/gcol33/texanshootR/issues}
}

}
\author{
\strong{Maintainer}: Gilles Colling \email{gilles.colling051@gmail.com} (\href{https://orcid.org/0000-0003-3070-6066}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mascot.R
\name{mascot_state}
\alias{mascot_state}
\title{Compute the mascot emotional state for a run snapshot.}
\usage{
mascot_state(progress, best_p = NA_real_, escalating = FALSE)
}
\arguments{
\item{progress}{Numeric fraction of the run budget used (0 to 1).}

\item{best_p}{Numeric smallest p-value found so far in the run,
or \code{NA} if none. Currently unused; retained for back-compat.}

\item{escalating}{Logical: is the derived-metric escalation phase
currently active.}
}
\value{
One of \code{"composed"}, \code{"uncertain"}, \code{"worried"}, \code{"anxious"},
\code{"panicked"}, \code{"desperate"}.
}
\description{
Walks the progress ladder regardless of \code{best_p} so the player sees
the full emotional arc unfold during a run. The \code{resolved} face is
reserved for the final frame after a shippable run -- it isn't
returned mid-flight from this function. \code{best_p} is kept on the
signature for back-compat callers but no longer short-circuits the
ladder.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wardrobe.R
\name{wardrobe}
\alias{wardrobe}
\title{Inspect or modify the mascot wardrobe}
\usage{
wardrobe(slot = NULL, id = NULL)
}
\arguments{
\item{slot}{Optional slot name: hat, poncho, badge, lanyard, cloak.}

\item{id}{Optional cosmetic id.}
}
\value{
The wardrobe state (invisible when modifying, visible when
listing).
}
\description{
Without arguments, prints the equipped + unlocked cosmetics. With
\code{slot} and \code{id}, equips the given cosmetic in the given slot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mascot.R
\name{mascot_render}
\alias{mascot_render}
\title{Render the mascot for a given state and heartbeat tick.}
\usage{
mascot_render(state, tick = 0L, cosmetics = NULL)
}
\arguments{
\item{state}{One of the five mascot states.}

\item{tick}{Integer tick counter for the heartbeat animation.}

\item{cosmetics}{Optional wardrobe state list (slot -> equipped id).}
}
\value{
Character vector of lines.
}
\description{
Render the mascot for a given state and heartbeat tick.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unlocks.R
\name{progress}
\alias{progress}
\title{Inspect API unlock progress}
\usage{
progress(what = NULL)
}
\arguments{
\item{what}{Optional. A gated-function name (e.g. \code{"manuscript"}) or
an achievement id (e.g. \code{"ach_multiple_comparisons"}). When NULL,
prints the overview.}
}
\value{
A \code{tx_progress} object (invisible).
}
\description{
Prints a HUD-style summary of unlock state for gated functions and
achievements. Reads live save state.
}
\details{
Three call modes:
\itemize{
\item \code{progress()} -- overview: career tier, gated-function lock
map, achievement / wardrobe counts, and in-flight progress.
\item \code{progress("manuscript")} -- per-function card.
\item \code{progress("ach_multiple_comparisons")} -- per-achievement
card.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output_manuscript.R
\name{manuscript}
\alias{manuscript}
\title{Generate a manuscript from a run}
\usage{
manuscript(run, output_dir = NULL, file = NULL, force = FALSE)
}
\arguments{
\item{run}{A \code{tx_run} object returned by \code{\link[=shoot]{shoot()}}.}

\item{output_dir}{Optional output directory.}

\item{file}{Optional filename stem (without extension).}

\item{force}{Overwrite an existing file.}
}
\value{
Character vector of file paths (invisible).
}
\description{
Renders an academic-style write-up of the highlighted specification.
If \code{quarto} and \code{tinytex} are available, produces a PDF + DOCX; if
only \code{rmarkdown} is available, produces a DOCX. The package never
installs LaTeX on its own.
}
\details{
Chain stage: \strong{manuscript} (length 2). See \code{\link[=progress]{progress()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vocab.R
\docType{data}
\name{vocab}
\alias{vocab}
\alias{vocab_tags}
\alias{vocab_phases}
\alias{vocab_careers}
\alias{vocab_mascot_states}
\alias{vocab_effects}
\alias{career_levels}
\title{Controlled vocabularies for the message and event registries}
\format{
An object of class \code{character} of length 39.

An object of class \code{character} of length 16.

An object of class \code{character} of length 4.

An object of class \code{character} of length 9.

An object of class \code{character} of length 9.
}
\usage{
vocab_tags

vocab_phases

vocab_careers

vocab_mascot_states

vocab_effects

career_levels()
}
\description{
These exported character vectors define the valid values for the
\code{tags}, \code{career_min}, \code{trigger_phase}, \code{mascot_state_affinity}, and
event \code{effects} fields used in \verb{inst/messages/*.yaml} and
\verb{inst/events/*.yaml}. \code{\link[=validate_messages]{validate_messages()}} enforces them.
}
\section{Functions}{
\itemize{
\item \code{vocab_tags}: Valid \code{tags} values. The lower-case fallacy
names from the design contract plus thematic content tags.

\item \code{vocab_phases}: Valid \code{trigger_phase} values.

\item \code{vocab_careers}: Valid \code{career_min} values.

\item \code{vocab_mascot_states}: Valid \code{mascot_state_affinity} values.

\item \code{vocab_effects}: Valid event-effect keys for \verb{inst/events/*.yaml}.

\item \code{career_levels()}: Career level ordering (low to high).

}}
\keyword{datasets}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{texanshootR-options}
\alias{texanshootR-options}
\title{Package options for texanshootR}
\description{
All options are read at the time of use, so changing them takes
effect immediately for subsequent calls.
}
\details{
\describe{
\item{\code{texanshootR.animations}}{Logical. Enable terminal
animations and ANSI cursor effects. Default: \code{TRUE}. Forced off
in non-interactive sessions and when the terminal lacks ANSI
support.}
\item{\code{texanshootR.output_dir}}{Character or \code{NULL}. Default
directory for output files. \code{NULL} falls back to
\code{\link[=tempdir]{tempdir()}}.}
\item{\code{texanshootR.save_enabled}}{Logical. Enable persistent
career data. \code{FALSE} runs entirely in memory. Default: \code{TRUE}.}
\item{\code{texanshootR.quiet}}{Logical. Suppress status prints from
output functions. Default: \code{FALSE}.}
\item{\code{texanshootR.event_rate}}{Numeric in \verb{[0, 1]}. Per-run
probability of a life-event encounter. Default: \code{1/6}.}
\item{\code{texanshootR.life_events}}{Logical. Enable life-event
encounters during runs. Default: \code{TRUE}.}
\item{\code{texanshootR.ui_mode}}{Optional character override for the
TUI rendering mode. One of \code{"ansi"}, \code{"dynamic"}, \code{"plain"}.
Unset (the default) means auto-detect. Use this if auto-detect
picks the wrong mode for your terminal (e.g., a remote RStudio
session that does not report \code{.Platform$GUI == "RStudio"}).}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output_graphical.R
\name{graphical_abstract}
\alias{graphical_abstract}
\title{Generate a graphical abstract PNG}
\usage{
graphical_abstract(run, output_dir = NULL, file = NULL, force = FALSE)
}
\arguments{
\item{run}{A \code{tx_run} object returned by \code{\link[=shoot]{shoot()}}.}

\item{output_dir}{Optional output directory.}

\item{file}{Optional filename stem (without extension).}

\item{force}{Overwrite an existing file.}
}
\value{
A length-one character vector giving the path to the written
PNG file, returned invisibly. Called for its side effect of
generating the graphical abstract in \code{output_dir} and advancing the
publication chain.
}
\description{
A single-figure summary of the highlighted specification, composed
as a multi-panel layout with conceptual arrows. Renders via ggplot2
to PNG.
}
\details{
Chain stage: \strong{graphical_abstract} (length 5). See \code{\link[=progress]{progress()}}.
}
